# Methods for communication with Octave.
# Octave process will be opened/closed for each call. Our usage-patterns meant 
# open/close times generally were being dwarfed by time needed to move datasets 
# between octave and python anyways.
# This script converts between octave/cygwin paths on windows, but please do 
# contact me in case of bugs/unexpected behaviour etc: reidarst@scicraft.org
   

def eval_single(line, retvals=1, loadpath=None, **variables):
    """ line : expression to compute, retvals : number of values returned, 
    loadpath : if set, added to octave loadpath
    any variables set will be available in octave through their given name.
         
    example:
        a = octpy.eval_single("a * rand(3)", a=[10, 20, 30], retvals = 1)"""
        
    import os
    assert isinstance(line, str) # checking if input is a string
    assert isinstance(retvals, int)  # checking if retvals is an int
    assert not loadpath or os.path.exists(loadpath)
    
    if retvals==1:
        ret = __call_octave__(line, variables, retvals, loadpath, is_single=True)
        if ret:
            return ret[0]
        else:
            return None
    else:
        return __call_octave__(line, variables, retvals, loadpath, is_single=True)

def eval_all(text, loadpath=None, **variables):
    '''lines : octave script to run, 
    loadpath : if set, automatically added to octave loadpath
    variables set will be available in octave through their given name
    all variables available in the current namespace 
    will be returned through a dictionary
 
    example:
        octscript = """b = a * rand(3*c) 
d = b * rand(3)"""
        ret_dict = octpy.eval_all(octscript, a=[10, 20, 30], c=7)'''
    import os
    assert isinstance(text, str) # checking if text is a string
    assert not loadpath or os.path.exists(loadpath) # checking if path exists and is valid

    return __call_octave__(text, variables, -1, loadpath)


def __call_octave__(text, args , retvals, loadpath, is_function = False, is_single = False):

    """ internal method which call octave. Parameters are highly magical. 
    Use convience methods eval_single / eval_all for slightly saner method signatures."""
    import matfile, OctaveFileWriter, scipy
    import tempfile, time, subprocess, os, exceptions


    def make_cygdrivepath(path):
        return "/cygdrive/" + path.replace(":\\", "/").replace("\\", "/")


    if os.name != "nt":
        # using shellcommand 'octave -q' gives file not found error for some reason.
        shellcommand = ' -q'
        executable   = 'octave'
        workdir = None      
    else:
        dirs  = [os.environ['PROGRAMFILES']+"\\"+a for a in os.listdir(os.environ['PROGRAMFILES']) if "GNU OCTAVE" in a.upper()]
        dirs += [os.path.abspath("..")+"\\"+a for a in os.listdir("..") if "GNU OCTAVE" in a.upper()]
        dirs.sort(), dirs.reverse()     # If version numbers are included, we want to check latest first
        for direc in dirs:
            if os.path.exists(direc+"\\bin\\sh.exe") and os.path.exists(direc+"\\octave.sh"):
                workdir      = direc
                shellcommand = "\"" + direc+"\\bin\\sh.exe\" ./octave.sh"
                executable   = None
                break        
            if os.path.exists(direc+"\\bin\\sh.exe") and os.path.exists(direc+"\\bin\\start_octave.sh"):
                # Old structure
                workdir = direc
                shellcommand = "\""+ direc+"\\bin\\sh.exe\" /bin/start_octave.sh"
                executable   = None
                break
                
        if not shellcommand:
            shellcommand = r'octave'
            workdir = None
                              

    
    # -- create temporary variable names --
    outdata         = None          
    variable_dict   = {}
    var_names       = []
    if isinstance(args, list):
        # create variable dict
        for x in xrange(len(args)):
            var_names.append("temp_var"+str(x))
            variable_dict[var_names[x]] = args[x]
    elif isinstance(args, dict):
        var_names = args.keys()
        variable_dict = args  
     
    inputDescr, resultDescr, input_path, result_path = None, None, None, None
    try:
        # -- setup filenames --
        inputDescr ,  input_path    = tempfile.mkstemp(suffix='.mat')
        resultDescr,  result_path   = tempfile.mkstemp(suffix='.mat')
    
    
        # -- building scriptfile --
        script = ["echo off;"]
        path_to_add = ""
        
        
        if loadpath and os.name == "nt":
            path_to_add = make_cygdrivepath(os.path.dirname(os.path.realpath(path_to_add))) 
        elif loadpath:
            path_to_add += ":" + os.path.dirname(os.path.realpath(loadpath)) + "//"
            
        if loadpath:            
            script.append('path("'+ path_to_add + '", LOADPATH);')
                    
        if variable_dict:
            OctaveFileWriter.OctaveFileWriter().write_variable_dict(variable_dict, input_path)
            script.append("load " + input_path+";")
        
        # -- make script -- 
        line = ""
        return_vars = []
        if retvals > 0:
            return_vars = ["temp_var"+str(x) for x in xrange(retvals)]
            line = "["+",".join(return_vars)+ "] = "
    
        result_path_application = result_path # the resultpath-path as viewed from the application
        if os.name == 'nt':
            result_path_application = make_cygdrivepath(result_path)
    
        if is_function:
            script.append(line + text + "(" + ",".join(var_names)+");")
            script.append("save -mat-binary " + result_path_application + " " + " ".join(return_vars)+";")
        elif is_single and retvals:
            script.append(line+text)
            script.append("save -mat-binary " + result_path_application + " " + " ".join(return_vars)+",")
        else:
            script.append(text)
            script.append("save -mat-binary " + result_path_application +";")
    
        script.append("exit;")
        script = "\n".join(script)+"\n"
        
        try:
            use_shell = os.name == "nt"
            p = subprocess.Popen(shellcommand, executable=executable,
                                 shell=use_shell, stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.STDOUT, cwd=workdir)
        except exceptions.OSError:
            raise exceptions.OSError, "Octave not installed, or not available."

        stdout, stderr = p.communicate(input=script)
    
        if "error" in stdout:
            # better error handling?
            print stdout
        elif "'octave' is not recognized as an internal or external command" in stdout:
            # better error handling?
            print "Octave not installed, or path variable not set correctly"
        elif 'You have multiple copies of cygwin1.dll on your system.' in stdout:
            # another fun error encountered. better error handling?
            print stdout 
            
        elif os.path.exists(result_path):
            outdata =  matfile.read(result_path)
            if "__nargin__" in outdata.keys():
                del outdata['__nargin__']
            if retvals!=-1:
                # if we used temp variables, unpack into list since the var names will be useless
                keys = outdata.keys()
                keys.sort()
                outdata = [outdata[key] for key in keys]

    finally:
        try:
            if os.path.exists(result_path):
                os.close(resultDescr)
                os.remove(result_path)
        except IOError, e:
            print 'Could not delete temporary file: ', e
            pass
        except OSError, e:
            print 'Could not delete temporary file: ', e
            pass
        
        try:            
            if os.path.exists(input_path):
                os.close(inputDescr)
                os.remove(input_path)
        except IOError, e:
            print 'Could not delete temporary file: ', e
            pass
        except OSError, e:
            print 'Could not delete temporary file: ', e
            pass

    return outdata


      



        









