import gtk, gobject
import scipy, Numeric, rpy, scipy.stats
import copy
import os
import sys
from system import logger, dataset
from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
from matplotlib.backends.backend_gtkagg import NavigationToolbar2GTK
from matplotlib.figure import Figure
from matplotlib.widgets import Slider
from matplotlib.numerix import arange, sin, pi
from matplotlib import cm
import matplotlib
from Numeric import array

DATADIR = os.path.dirname(sys.modules['system'].__file__)
GLADEFILENAME = os.path.join(DATADIR, 'scicraft.glade')

""" Get functions are convience methods for getting plots. All settings are best-effort. 
For specific control over what gets shown, generate the plot yourself! """

def get_multiple_lines_plot(data):
    if len(data.shape)!=2:
        logger.log("warning","Line Plot does not support dataset with more than two dimensions")
        return    
    lineplot = MultipleLinePlot(data,Bar=False,Line=True,Points=False) 
    return [lineplot]
def get_line_plot(data):
    if len(data.shape)!=2:
        logger.log("warning","Line Plot does not support dataset with more than two dimensions")
        return    
    lineplot = IndexPlot(data,Bar=False,Line=True,Points=False) 
    return [lineplot]

def get_bar_plot(data):
    if len(data.shape)!=2:
        logger.log("warning","Line Plot does not support dataset with more than two dimensions")
        return    
    lineplot = IndexPlot(data,Bar=True,Line=False,Points=False) 
    return [lineplot]

def get_scatter_plot(data1, data2=None):
    if len(data1.shape)!=2:
        logger.log("warning","Scatter Plot does not support dataset with more than two dimensions")
        return
    if not data2:
        if not 1 in data1.shape:
            dim = data1.get_dim_names()[list(data1.shape).index(min(data1.shape))]
            id1, id2 = data1.get_identifiers(dim)[0:2]
            return [ScatterPlotBrowse(data1,data1,(dim,id1),(dim,id2))]
        else:
            return [IndexPlot(data1,Bar=False,Line=False,Points=True)]        
    
    # min(union(data1.shape & data2.shape)
    length = min([num for num in data1.shape if num in data2.shape])        
    dim1 = data1.get_dim_names()[list(data1.shape).index(length)]
    dim2 = data2.get_dim_names()[list(data2.shape).index(length)]        
    id1 = data1.get_identifiers(dim1)[0]
    id2 = data2.get_identifiers(dim2)[1]
    return [ScatterPlotBrowse(data1,data2,(dim1,id1),(dim2,id2))]

def get_density_plot(data):
    # min(union(data1.shape & data2.shape)
    dim = list(data.shape).index(min(data.shape))            
    return [DensityPlot(data,data,(data[dim],data[dim,0]),(data[dim],data[dim,1]),use_objectnames=False,name="Density Plot")]
    
def get_histogram_plot(data):
    return [HistogramPlot(data)]    
    
def get_table(data, only_selected=False,inverted=False):
    return [TablePlot(data,only_selected=only_selected, inverted=inverted)]

def get_imageplot(data):
    if len(data.shape)==2:
        return [ImagePlot(data)]
    elif len(data.shape)==3:
        sc = IndexPlot(dataset.Dataset(data.asarray()[0,0]))
        im = ImagePlots(data)
        im.set_scatterplot(sc)        
        return [im, sc]
        
    logger.log("warning","invalid argument was passed to get_imageplot")        
    return [] 

class Saver(type):
    """Magic that stores initial creation arguments.

    Objects from classes using this class as a metaclass can access
    their initially creation arguments as self._args_ and
    self._kwargs_.
    """
    def __call__(mcs, *args, **kwargs):
        obj = type.__call__(mcs, *args, **kwargs)

        # almost shallowcopy
        obj._args_ = []
        for arg in args:
            obj._args_.append(copy.copy(arg))
    
        obj._kwargs_ = {}
        for key, value in kwargs.items():
            obj._kwargs_[key] = copy.copy(value)
        
        return obj

class Plot (gtk.Frame):
    # Lets us access initial creation arguments on pickling
    class __metaclass__(gtk.Frame.__class__, Saver): pass

    def __getstate__(self):
        # Part of the magic that allows us to pickle complicated
        # objects (that usually can't be pickled), based on their
        # creation arguments. Be careful: All settings to object after
        # creation are lost with regards to saving/loading.
        return {'args' : self._args_, 'kwargs' : self._kwargs_}

    def __setstate__(self, dictionary):
        # Part of the magic that allows us to pickle complicated
        # objects (that usually can't be pickled), based on their
        # creation arguments. Be careful: All settings to object after
        # creation are lost with regards to saving/loading.
        self._args_ = dictionary['args']
        self._kwargs_ = dictionary['kwargs']
        self.__init__(*dictionary['args'], **dictionary['kwargs'])


    def __init__(self, title):
        gtk.Frame.__init__(self)
        self.sel_obj = None
        self.active = False
        self.selection_listener = None

        self.title = None
        self.set_title(title)
        
        self.description = "<b>Plot</b>\nTitle: " + self.get_name()  
                
        self.set_shadow_type(gtk.SHADOW_NONE)

    def get_title(self):
        return self.title

    def set_title(self, title):
        self.title = str(title)
        self.set_name(self.title)

    def selection_changed(self, selection):
        """Subclasses can override to respond to selection changes."""
        pass

    def set_selection_listener(self, listener):
        """Allow project to listen to selections.

        The selection will propagate back to all plots through the
        selection_changed() method. The listener will be called as
        listener(dimension_name, ids).
        """
        self.selection_listener = listener

    def get_toolbar(self):
        """Subclasses can override this to give a toolbar widget."""
        return None

    def get_popupmenu(self):
        """ Subclasses implementing popup-menu's should override this method """
        return None

    def get_description(self):
        """ return a string description of plot, (semi-static, meant for information-pane)"""
        return self.description

    def set_description(self, string):
        self.description = str(string)              
        
    def on_key_press(self,key,event):
        """ Provided for access to keypress events in subclasses. """
        pass
    
    def on_key_release(self,key,event):
        """ Provided for access to keyrelase events in subclasses """
        pass                                          
        
class EmptyPlot (Plot):
    def __init__(self):
        Plot.__init__(self, 'Empty view')
        label = gtk.Label('No view')
        ebox = gtk.EventBox()
        ebox.add(label)
        self.add(ebox)
        
        label.show()
        ebox.show()
        self.show()                    

class ScatterPlot(Plot):
    def __init__(self, dataset1, dataset2, identifiers_1, identifiers_2, 
        name="Scatter plot",xaxis_label = "%s, %s" , yaxis_label = "%s, %s"
        , use_grid=False, use_objectnames=True):
        self._use_axis = False
        """General Scatter Plot. 
        Accepts two datasets.
        
        x_identifiers = tuple containing dim_name, id_name. eg: ('x','x_0') used on x_data
        y_identifiers = tuple containing dim_name, id_name. eg: ('y','y_0') used on y_data
           
        xaxis_label / yaxis_label  = Labels for the axis'. If there is a '%s' present
        , id_name will be shown.
        """
        Plot.__init__(self, name)
                
        # checking inputs
        assert isinstance(dataset1[identifiers_1],int)     
        assert isinstance(dataset2[identifiers_2],int)
            
        # setting various variables
        self._dataset1 = dataset1
        self._dataset2 = dataset2
        self._dim_1 = identifiers_1[0]
        self._dim_2 = identifiers_2[0]
        self._id_1  = identifiers_1[1]
        self._id_2  = identifiers_2[1]        
        self._xaxis_label = xaxis_label
        self._yaxis_label = yaxis_label    
        self._indexset_name =  "index_set"
        
        self._use_grid = use_grid
        self._use_objectnames = use_objectnames
        self._use_flipx = 1
        self._use_flipy = 1
                                          
        self._fig = Figure(figsize=(5,4), dpi=72)
        self._canvas = FigureCanvas(self._fig)
        self._ax = self._fig.add_subplot(111)        
        self.add(self._canvas)
        self._canvas.show()
        
        self.selection = {}
        
        self._init_toolbar()
        self._init_popupmenu()                                         
        self.update()

    def toggle_axis(self, waa):
        self._use_axis = not self._use_axis
        self.update() 
    
    def switch_grid(self,*args):
        self._use_grid = not self._use_grid
        self.update()

    def switch_flipx(self,*args):
        self._use_flipx = self._use_flipx * -1
        self.update()

    def switch_flipy(self,*args):
        self._use_flipy = self._use_flipy * -1
        self.update()                
        
    def switch_named_objects(self,*args):
        self._use_objectnames = not self._use_objectnames
        self.update()
        
    def get_popupmenu(self):
        return self._popupmenu
            
    def get_toolbar(self):            
        return self._toolbar
   
    def _init_popupmenu(self):
        self._popupmenu = CustomPopupMenu(self)
        self._popupmenu.init_from_toolbar(self.get_toolbar())
        self._popupmenu.add_togglebutton("Show X Flipped",self.switch_flipx,self._use_flipx==-1)
        self._popupmenu.add_togglebutton("Show Y Flipped",self.switch_flipy,self._use_flipy==-1)
        self._popupmenu.add_separator()
        self._popupmenu.add_togglebutton("Use Axis",self.toggle_axis,self._use_axis)        
        self._popupmenu.add_togglebutton("Use Grid",self.switch_grid,self._use_grid)
        self._popupmenu.add_togglebutton("Named Objects",self.switch_named_objects,self._use_objectnames)
        
                                

    def _init_toolbar(self):
        # create toolbar
        self._toolbar = ScatterPlotToolbar(self._canvas, None) 
        self._toolbar.set_property('show-arrow', False)
                                
        # connect various
        self._toolbar.set_select_callback(self.rectangle_select_callback)
                          
    def update(self,selections=None):    
        """Refreshes the plot according to the new parameters."""
        self._ax.clear()
                    
        # set names
        self._ax.set_title(self.get_title())     
        self._ax.set_xlabel(self._xaxis_label % (self._dim_2, self._id_1,)\
                            [2-self._xaxis_label.count("%s"):2])        
        self._ax.set_ylabel(self._yaxis_label % (self._dim_1, self._id_2,)\
                            [2-self._yaxis_label.count("%s"):2])
            
        # fetch data                           
        data_to_plot1 = self._dataset1.get_data_slice(self._dim_1,self._id_1)
        data_to_plot2 = self._dataset2.get_data_slice(self._dim_2,self._id_2)
        
        # draw text
        opposite_dim2 = [dim for dim in self._dataset2.get_dim_names() if dim != self._dim_2][0]
        if self._use_objectnames:
            for x,id in enumerate(self._dataset2.get_identifiers(opposite_dim2)):
                self._ax.text(data_to_plot1[x]*self._use_flipx,data_to_plot2[x]*self._use_flipy,id)
                
        # enable grid?
        if self._use_grid:
            self._ax.grid(b=True)

        # enanble axis?
        if self._use_axis:
            self._ax.axhline(0.0000001,linewidth=2)
            self._ax.axvline(0.0000001,linewidth=2)                             
                        
        if data_to_plot1.shape != data_to_plot2.shape:
            logger.log('warning', "Error trying to plot vectors of different sizes")
        else:                      
            self.draw_plot(data_to_plot1,data_to_plot2)
            if self.selection:                
                results = self.parse_selection(self.selection)             
                for x,y in results.values():
                    self.draw_selection(x,y)
            self.ensure_scale()                                   
            self._canvas.draw()

    def ensure_scale(self):
        # ensures sane scales are set by adding by expanding limits by 10%
        ymin,ymax = self._ax.get_ylim()
        increase = (ymax-ymin)*0.05           
        self._ax.set_ylim(ymin-increase,ymax+increase)

        xmin,xmax = self._ax.get_xlim()
        increase = (xmax-xmin)*0.05           
        self._ax.set_xlim(xmin-increase,xmax+increase)
    
    def draw_plot(self,data_to_plot1,data_to_plot2):
        """ Draws the actual plot """         
        self._ax.plot(data_to_plot1*self._use_flipx, data_to_plot2*self._use_flipy,'ob')                
            
    def draw_selection(self,data_to_plot1, data_to_plot2):
        """ Redraw marked items with different colours """    
        self._ax.plot(data_to_plot1*self._use_flipx, data_to_plot2*self._use_flipy,'or')
                            
    def rectangle_select_callback(self, x1, y1, x2, y2):
     
        # convert id, dim to numbers
        id_number1  = self._dataset1.get_identifiers(self._dim_1).index(self._id_1) 
        id_number2  = self._dataset2.get_identifiers(self._dim_2).index(self._id_2)
        dim_number1 = self._dataset1.get_dim_names().index(self._dim_1)
        dim_number2 = self._dataset2.get_dim_names().index(self._dim_2)
        
        # when comparing ids we want dimension data runs *along*, not dimension it's selected *from*
        opposite_dim1 = self._dataset1.get_dim_names()[dim_number1 * -1 + 1]
        opposite_dim2 = self._dataset2.get_dim_names()[dim_number2 * -1 + 1]
        
        # fetch data
        xdata = scipy.ravel(scipy.take(self._dataset1.asarray() ,[id_number1], axis=dim_number1))
        ydata = scipy.ravel(scipy.take(self._dataset2.asarray() ,[id_number2], axis=dim_number2))
        
        # selections may start on the right/bottom and go up/leftwards
        x , y = [x1*self._use_flipx , x2*self._use_flipx ], [y1*self._use_flipy, y2*self._use_flipy]
        x.sort(), y.sort()
        
        # find indices of selected area
        index = scipy.nonzero((xdata>x[0]) & (xdata<x[1]) & (ydata>y[0]) &
                                     (ydata<y[1]))
        if type(index)==tuple: # index has been returned as tuple on some systems
            index = index[0]

        ids1 = self._dataset1.get_identifiers(opposite_dim1, index) 
        ids2 = self._dataset2.get_identifiers(opposite_dim2, index)
        
        logger.log("debug",str(len(ids1))+ " selected on dim: " + opposite_dim1)
        self.selection_listener(opposite_dim1, ids1)
        if (ids1 != ids2 or opposite_dim1 != opposite_dim2):
            # no need to update same names twice!
            logger.log("debug",str(len(ids1))+ " selected on dim: " + opposite_dim1)
            self.selection_listener(opposite_dim2, ids2)        
        
    def selection_changed(self, selection):
        if selection != self.selection:
            self.selection = selection.copy()
            self.update()

    def parse_selection(self, selection):
        result = {}
        for key in selection:
            # getting dim data runs *along*
            opposite_dim1 = [dim for dim in self._dataset1.get_dim_names() if dim != self._dim_1][0]
            opposite_dim2 = [dim for dim in self._dataset2.get_dim_names() if dim != self._dim_2][0]
        
            # is this selection relevant?
            if key not in [opposite_dim1, opposite_dim2]:
                continue               
            indices = []
            if key == opposite_dim1:
                available_ids1 = self._dataset1.get_identifiers(opposite_dim1)
                ids1_union = [sel for sel in selection[key] if sel in available_ids1]
                indices.extend(self._dataset1.get_indices(opposite_dim1, ids1_union))
                
            if key == opposite_dim2:   
                available_ids2 = self._dataset2.get_identifiers(opposite_dim2)
                ids2_union = [sel for sel in selection[key] if sel in available_ids2]
                indices.extend(self._dataset2.get_indices(opposite_dim2, ids2_union))
                
            # in the event that all dim names so far have been equal, make indices list unique
            [indices.remove(item) for item in list(indices) if indices.count(item)!=1]

            # fetch data, filter it.
            data_to_plot1 = self._dataset1.get_data_slice(self._dim_1, self._id_1) 
            data_to_plot2 = self._dataset2.get_data_slice(self._dim_2, self._id_2)                 
            data_to_plot1 = scipy.take(data_to_plot1, indices)
            data_to_plot2 = scipy.take(data_to_plot2, indices)

            # now start drawing
            result[opposite_dim1] = (data_to_plot1, data_to_plot2)         
        return result
        
class ScatterPlotBrowse(ScatterPlot):
    # Adds up/down/back/forward buttons for selecting slice to show to 
    
    def _init_toolbar(self):
        ScatterPlot._init_toolbar(self)
        
        # adding buttons
        tbutton_up = self._toolbar.add_up_button()
        tbutton_down = self._toolbar.add_down_button()
        tbutton_switch = self._toolbar.add_switch_button()
        tbutton_back = self._toolbar.add_left_button()
        tbutton_forward = self._toolbar.add_right_button()
                                
        # connect to buttons                                                                
        tbutton_up.connect('clicked', self._up_clicked)
        tbutton_down.connect('clicked', self._down_clicked)
        tbutton_switch.connect('clicked',self._switch_clicked)
        tbutton_back.connect('clicked', self._back_clicked)
        tbutton_forward.connect('clicked', self._forward_clicked)

    def _switch_clicked(self, button):
        self._dim_1 = self._dataset1.get_dim_names()[self._dataset1.get_dim_names().index(self._dim_1)-1]
        self._dim_2 = self._dataset2.get_dim_names()[self._dataset2.get_dim_names().index(self._dim_2)-1] 
        self._id_1 = self._dataset1.get_identifiers(self._dim_1)[0]
        self._id_2 = self._dataset2.get_identifiers(self._dim_2)[1]
        
        # temporary code 8.june.07 - reidar
        if not self._dataset1.get_data_slice(self._dim_1,self._id_1).any():
            self._id_1 = self._dataset1.get_identifiers(self._dim_1)[2]
            
        self.update() 
        
    #Slots for GUI handling when selecting what data to display    
    def _forward_clicked(self, *ignored):
        x_number = self._dataset1.get_indices(self._dim_1,[self._id_1])[0]
        if x_number+1 < self._dataset1.get_dimension_length(self._dim_1):                
            self._id_1 = self._dataset1.get_identifiers_from_indices(self._dim_1,[x_number+1])[0]                        
        self.update()
                                
    def _back_clicked(self, *ignored):
        x_number = self._dataset1.get_indices(self._dim_1,[self._id_1])[0]
        if x_number > 0:
            self._id_1 = self._dataset1.get_identifiers_from_indices(self._dim_1,[x_number-1])[0]
        self.update()
            
    def _up_clicked(self, *ignored):
        y_number = self._dataset2.get_indices(self._dim_2,[self._id_2])[0]
        if y_number+1 < self._dataset2.get_dimension_length(self._dim_2):
            self._id_2 = self._dataset2.get_identifiers_from_indices(self._dim_2,[y_number+1])[0]                        
        self.update()
                        
    def _down_clicked(self, *ignored):
        y_number = self._dataset2.get_indices(self._dim_2,[self._id_2])[0]
        if y_number > 0:
            self._id_2 = self._dataset2.get_identifiers_from_indices(self._dim_2,[y_number-1])[0]
        self.update()



class IndexPlot(Plot):
    """ Plots a vector against an Index 
    
    Has three modes: Line, Bar, Points.
    """
    def __init__(self, dataset,Line=False,Bar=True,Points=False):        
        self._shown = [Line, Bar, Points]
        self._use_axis = False
        self._use_grid = False
        self._use_objectnames = False
                   
        Plot.__init__(self, "IndexPlot, %s" %(dataset.get_name()))
        # setting various variables
        self._data = dataset

        # choose shortest dim to extract from
        dimn = list(dataset.shape).index(min(dataset.shape))               
        self._dim = dataset[dimn]
        self._id  = dataset[dimn,0]
                                                          
        self._fig = Figure(figsize=(5,4), dpi=72)
        self._canvas = FigureCanvas(self._fig)
        self._ax = self._fig.add_subplot(111)        
        self.add(self._canvas)
        self._canvas.show()
        
        self.selection = {}

        self._init_toolbar()         
        self._init_popupmenu()                                            
        self.update()
        
    def get_popupmenu(self):
        return self._popupmenu            
   
    def _init_popupmenu(self):
        self._popupmenu = CustomPopupMenu(self)
        self._popupmenu.init_from_toolbar(self.get_toolbar())
    
        self._popupmenu.add_togglebutton("Use Axis",self.toggle_axis,self._use_axis)
        self._popupmenu.add_togglebutton("Use Grid",self.switch_grid,self._use_grid)
        self._popupmenu.add_togglebutton("Named Objects",self.switch_named_objects,self._use_objectnames)
        self._popupmenu.add_separator()
        self._popupmenu.add_togglebutton("Line",self.toggle_line,self._shown[0])
        self._popupmenu.add_togglebutton("Bars",self.toggle_bar,self._shown[1])
        self._popupmenu.add_togglebutton("Points",self.toggle_points,self._shown[2])
    
    def toggle_axis(self, waa):
        self._use_axis = not self._use_axis
        self.update() 
               
    def toggle_line(self, waa):
        self._shown[0] = not self._shown[0]
        self.update()

    def toggle_bar(self, waa):
        self._shown[1] = not self._shown[1]
        self.update()

    def toggle_points(self,waa):
        self._shown[2] = not self._shown[2]
        self.update()

    def switch_grid(self,*args):
        self._use_grid = not self._use_grid
        self.update()
        
    def switch_named_objects(self,*args):
        self._use_objectnames = not self._use_objectnames
        self.update()                
                                                            
    def _init_toolbar(self):    
        # create toolbar
        self._toolbar = ScatterPlotToolbar(self._canvas, None) 
        self._toolbar.set_property('show-arrow', False)
             
        # adding buttons
        tbutton_back = self._toolbar.add_left_button()
        tbutton_switch = self._toolbar.add_switch_button()
        tbutton_forward = self._toolbar.add_right_button()
                                
        # connect to buttons                                                                
        tbutton_back.connect('clicked', self._back_clicked)
        tbutton_switch.connect('clicked',self._switch_clicked)
        tbutton_forward.connect('clicked', self._forward_clicked)
        
        self._toolbar.set_select_callback(self.rectangle_select_callback)        
    
    def _back_clicked(self, button):
        ids = self._data.get_identifiers(self._dim)
        index = ids.index(self._id)
        self._id = ids[index-1]
        self.update()
        
    def _switch_clicked(self, button):
        self._dim = self._data.get_dim_names()[self._data.get_dim_names().index(self._dim)-1]
        self._id = self._data.get_identifiers(self._dim)[0]
        self.update()        

    def _forward_clicked(self, button):
        ids = self._data.get_identifiers(self._dim)
        index = ids.index(self._id)
        if index+1>=len(ids):
            index = -1        
        self._id = ids[index+1]
        self.update()                    
            
    def get_toolbar(self):            
        return self._toolbar        
                
    def update(self):                
        self._ax.clear()
        
        # set names        
        self._ax.set_title(", ".join([self.get_title(),self._dim, self._id]))
        self._ax.set_xlabel(str([d for d in self._data.get_dim_names() if not d==self._dim][0]))      
        self._ax.set_ylabel("Amplitude")        
                             
        y = self._data.get_data_slice(self._dim, self._id)
        x = Numeric.array(range(len(y)))
        
        oppdim = [dim for dim in self._data.get_dim_names() if dim!=self._dim][0]   
                             
        # MAGIC NUMBER ALERT! 
        if (len(x)<73):
            self._ax.set_xticks(Numeric.array(x))
            self._ax.set_xticklabels(self._data.get_identifiers(oppdim))

        # draw text        
        if self._use_objectnames:
            for ii,id in enumerate(self._data.get_identifiers(oppdim)):
                self._ax.text(x[ii],y[ii],id)
                
        # enable grid?
        if self._use_grid:
            self._ax.grid(b=True)  

        if self._use_axis:
            self._ax.axhline(0)
            self._ax.axvline(0)              

        # draw actual data!
        self.draw(x,y)
        
        # handle selections
        draws = self.parse_selection(self.selection)
        for indices, ydata in draws.values():
            self._ax.plot(indices, ydata,'or', ms=3)            
        
        self._canvas.draw()                

    def draw(self, x, y):
        width = 0.8*sum(x-x[0])/sum(range(len(x)))
        if self._shown[0]:            
            self._ax.plot(x, y,'-g')
        if self._shown[2]:
            self._ax.plot(x, y,'ob', ms=3)                   
        if self._shown[1]:        
            self._ax.bar(Numeric.asarray(x)-width/2,y,width=width)
            self._ax.set_xlim(x[0]-width*0.8,x[-1]+width*0.8)

    def rectangle_select_callback(self, x1, y1, x2, y2):    
        # selections may start on the right/bottom and go up/leftwards
        x , y = [int(x1), int(x2)] , [y1, y2]
        x.sort(), y.sort()        
        ydata = self._data.get_data_slice(self._dim, self._id)
        xdata = Numeric.zeros(ydata.shape)
        xdata[x[0]:x[1]+1] = 1
        
        oppdim_num = list(self._data.get_dim_names()).index(self._dim) * -1 + 1
        oppdim = self._data[oppdim_num]
        # find indices of selected area
        index =scipy.nonzero((ydata>y[0]) & (ydata<y[1]) & (xdata!=0) )
        if type(index)==tuple:
                index = index[0]

        ids = self._data.get_identifiers(oppdim, index)     
        
        logger.log("debug",str(len(ids))+ " selected on dim: " + oppdim)
        
        self.selection_listener(oppdim, ids)

    def selection_changed(self, selection):
        if selection != self.selection:
            self.selection = selection.copy()
            self.update()               
                    
    def parse_selection(self, selection):
        result = {}
        for key in selection:        
            oppdim_num = list(self._data.get_dim_names()).index(self._dim) * -1 + 1
            oppdim = self._data[oppdim_num]
                    
            if key != oppdim:
                continue
                               
            ids1 = self._data.get_identifiers(oppdim)            
            ids1_union = [sel for sel in selection[key] if sel in ids1]
            indices = self._data.get_indices(oppdim, ids1_union)                
                
            # fetch data, filter it.
            ydata = self._data.get_data_slice(self._dim, self._id)                 
            ydata = scipy.take(ydata, indices)
            result[key] = (indices,ydata)
        return result            

class MultipleLinePlot(IndexPlot):
    
    def __init__(self,*stuff,**stuffer):
        self._colourscheme= "Green"
        IndexPlot.__init__(self,*stuff,**stuffer)
     
    def _init_toolbar(self):
        # We don't need back/forward buttons
            
        # create toolbar
        self._toolbar = ScatterPlotToolbar(self._canvas, None) 
        self._toolbar.set_property('show-arrow', False)
             
        # adding buttons        
        tbutton_switch = self._toolbar.add_switch_button()
                                
        # connect to buttons                                                                
        tbutton_switch.connect('clicked',self._switch_clicked)
                
        self._toolbar.set_select_callback(self.rectangle_select_callback)

    def _init_popupmenu(self):
        self._popupmenu = CustomPopupMenu(self)
        self._popupmenu.init_from_toolbar(self.get_toolbar())
    
        self._popupmenu.add_togglebutton("Use Axis",self.toggle_axis,self._use_axis)
        self._popupmenu.add_togglebutton("Use Grid",self.switch_grid,self._use_grid)
        self._popupmenu.add_togglebutton("Use Legend",self.switch_named_objects,self._use_objectnames)
        self._popupmenu.add_separator()
        colours = ['Green','Pinks','Random']        
        self._popupmenu.add_grouped_tooglebutton("Colour Scheme",colours, self.set_colour, self._colourscheme)      

    def set_colour(self, ignored, colour):
        self._colourscheme = colour
        self.update()
        
    def draw(self, x, y,names):
        c = (0,0,0)
        m = len(names)-1.0
        for ii, name in enumerate(names):
            if self._colourscheme == "Green": 
                c = (0.02,0.6,0.02)
            elif self._colourscheme == "Pinks":            
                c = (ii*1.0/m,(m*1.0-ii)/m,1)
                print c
            else:
                c = scipy.rand((3))              
                                                               
            self._ax.plot(x, y[:,ii],'-',label=name,c=c)

               
    def update(self):                
        self._ax.clear()
    
        # set names        
        self._ax.set_title(", ".join([self.get_title(),self._dim, self._id]))
        self._ax.set_xlabel(str([d for d in self._data.get_dim_names() if not d==self._dim][0]))      
        self._ax.set_ylabel("Amplitude")        
    
        if self._dim == self._data[0]:
            y = Numeric.transpose(self._data.asarray())
        else:            
            y = self._data.asarray()                                         

        x = Numeric.array(range(len(y)))
        
        oppdim = [dim for dim in self._data.get_dim_names() if dim!=self._dim][0]   
                             
        # MAGIC NUMBER ALERT! 
        if (len(x)<73):
            self._ax.set_xticks(Numeric.array(x))
            self._ax.set_xticklabels(self._data.get_identifiers(oppdim))
                
        # enable grid?
        if self._use_grid:
            self._ax.grid(b=True)  

        if self._use_axis:
            self._ax.axhline(0)
            self._ax.axvline(0)              

        # draw actual data!
        self.draw(x,y,self._data.get_identifiers(self._dim))

        # draw text        
        if self._use_objectnames:
            self._ax.legend()            
        
        # handle selections
        sel = self.parse_selection(self.selection)
        
        # plot selected lines
        if self._dim in sel:
            indices = self._data.get_indices(self._dim, sel[self._dim])                
            axisnum = self._data.get_dim_names().index(self._dim)                                                                                                                 
            ydata = scipy.take(self._data.asarray(), indices,axis=axisnum)
                    
            for jj in xrange(ydata.shape[0]):
                tp = Numeric.ravel(ydata[jj,])                            
                self._ax.plot(range(len(tp)), tp,'-b', ms=4)

        # plot points, only on first line though                
        if oppdim in sel:
            indices = self._data.get_indices(oppdim, sel[oppdim])
            axisnum = self._data.get_dim_names().index(oppdim)  
            xdata = scipy.take(self._data.asarray(), indices, axis=axisnum)                
            self._ax.plot(indices,Numeric.ravel(xdata[0,]),'or')
                                            
        self._canvas.draw()

    def parse_selection(self, selection):
        result = {}
        for key in selection:
            if key in self._data.get_dim_names():
                un = list(set(selection[key]).intersection(self._data.get_identifiers(key)))
                if un:
                    result[key] = un         
        return result
                           
    def rectangle_select_callback(self, x1, y1, x2, y2):    
        # selections may start on the right/bottom and go up/leftwards
        x , y = [int(x1), int(x2)] , [y1, y2]
        x.sort(), y.sort()
                
        ids = []
        for id in self._data.get_identifiers(self._dim):
            ydata = self._data.get_data_slice(self._dim, id)
            xdata = Numeric.zeros(ydata.shape)
            xdata[x[0]:x[1]+1] = 1
        
            oppdim_num = list(self._data.get_dim_names()).index(self._dim) * -1 + 1
            oppdim = self._data[oppdim_num]
            # find indices of selected area
            
            index =scipy.nonzero((ydata>y[0]) & (ydata<y[1]) & (xdata!=0) )
            
            if type(index)==tuple:
                index = index[0]
                
            if index.any():
                ids.append(id)
            
            #ids.extend(self._data.get_identifiers(oppdim, index))     
        
            logger.log("debug",str(len(ids))+ " selected on dim: " + oppdim)
        self.selection_listener(self._dim, list(set(ids)))
        
        selx = range(max(0,x[0]+1),min(x[1],self._data.shape[oppdim_num]))
        if selx:
            ids = self._data.get_identifiers(oppdim,indices=selx)
            self.selection_listener(oppdim,ids)  
            

class HistogramPlot(IndexPlot):
    """ Extension of IndexPlot. Creates a histogram of provided data."""
    
    def __init__(self,dataset):
        self._show_dist = True
        IndexPlot.__init__(self,dataset,Bar=True,Line=False,Points=False)        
        self.set_title("Histogram; %s" %(self._data.get_name()))
        #self.rawdata = []
                    
    def _init_popupmenu(self):
        IndexPlot._init_popupmenu(self)
        
        self._popupmenu.add_separator()
        self._popupmenu.add_togglebutton("Normal Distribution",self.toggle_dist,True)

    def toggle_dist(self, button):
        self._show_dist = not self._show_dist
        self.update()
                            
    def update(self):
        self.rawdata = self._data.get_data_slice(self._dim, self._id)
        rpy.r.assign("a",self.rawdata)
        rpy.r("h <- hist(a,plot=FALSE)")        
        self.y , self.x = Numeric.asarray(rpy.r("h$counts")) , Numeric.asarray(rpy.r("h$mids"))
        self._ax.clear()
        self.draw(self.x,self.y)
        if self.selection : 
            self.parse()
            
        min, max = self.x[0], self.x[-1]
        
        # draw normal distribution!
        if self._show_dist:
            norm = scipy.stats.norm
            std  = matplotlib.mlab.std(self.rawdata)            
            mean = matplotlib.mlab.mean(self.rawdata)
            tot  = sum(self.y)            
            step = sum(self.x-self.x[0])/sum(range(len(self.x)))               
            values = [(norm.cdf((sx+step/2-mean)/std) - norm.cdf((sx-step/2-mean)/std)) * tot for sx in self.x]                
            self._ax.plot(self.x,values,"r-")
                
        oppdim = [dim for dim in self._data.get_dim_names() if dim!=self._dim][0]
        self._ax.set_title("Histogram; " + self._dim + " : " + self._id)                                                       
        self._ax.set_xlabel("Number of " + oppdim + " at " + self._dim)        
                                                     
        self._canvas.draw() 
        
    # get data from a selected block in the histogram.
    # input data -> list, midpoint -> float
    # return a list where the first element is a list which contains data from the selected block
    def get_data(self, data, midpoint, width):
        sdata = Numeric.nonzero((data > midpoint-width) & (data < midpoint+width))
        if type(sdata) == tuple : sdata = sdata[0]
        return sdata
        
    def draw_selection(self, x, y):
        width = 0.8*sum(self.x-self.x[0])/sum(range(len(self.x)))
        if self._shown[0]:            
            self._ax.plot(x, y,'or')
        if self._shown[2]:
            self._ax.plot(x, y,'or', ms=3)                   
        if self._shown[1]:        
            self._ax.bar(Numeric.asarray(x)-width/2,y,width=width, color='red')
            self._ax.set_xlim(self.x[0]-width*0.8,self.x[-1]+width*0.8)
        pass
   
    # parse and draw a selection 
    def parse(self):
        data = self.rawdata
        sel = self.selection.values()
        val = []
        for i in range(len(sel[0])):
            val.append(int(sel[0][i]))
        sel = []
        for i in range(len(val)):
            sel.append(data[val[i]-1])
        if len(self.x) >= 2:
            blockwidth = (abs(self.x[1]-self.x[0]) / 2)
        tx = []
        ty = []
        # test if there is a selected point in the block of the histogram
        # block are self.x, data is sel[]
        for i in range(len(self.x)):
            if Numeric.less(sel, self.x[i]+blockwidth) & Numeric.greater(sel, self.x[i]-blockwidth):
                tx.append(self.x[i])
                ty.append(self.y[i])
        self.draw_selection(tx, ty)
        

            
    def rectangle_select_callback(self, x1, y1, x2, y2): #*stuff):
        # selections may start on the right/bottom and go up/leftwards
        x , y = [int(x1), int(x2)] , [y1, y2]
        x.sort(), y.sort()
        ids = []
        #x[0] = x[0] + 0.5
        x[1] = x[1] + 0.5

        data = self.rawdata       
        hist_y , hist_x = self.y, self.x 
        index = Numeric.nonzero((hist_x>=x[0]) & (hist_x<=x[1]))# & (xdata!=0) )
        if type(index)==tuple:
                index = index[0]
        if index :
            tempX = [] 
            tempY = []
            for i in range(len(index)):
                tempX.append(hist_x[index[i]])
                tempY.append(hist_y[index[i]])    
                #ids.extend(str(self.get_data(data, index[i])))
            if len(hist_x) >= 2 :
                blockwidth = (abs(hist_x[1]-hist_x[0]) / 2)
            else :
                pass
            test = []
            for i in range(len(tempX)):
                test.extend(self.get_data(data, float(tempX[i]), blockwidth))
 
            #test = self.get_data(data, blockwidth)
            id = self._data.get_dim_names()
            ids = self._data.get_identifiers(id[1], test)
            self.testx = array(tempX)
            self.testy = array(tempY)
            self.selection_listener(id[1], list(set(ids)))


    def selection_changed(self, selection): #*stuff):
        if selection != self.selection:
            self.selection = selection.copy()
            self.update()
            #self.draw_selection(self.testx, self.testy)
        # would be cool to show fractions here though
        #pass                

class DensityPlot(ScatterPlotBrowse):
    def __init__(self,*stuff,**stuffer):
        self.smoothing = '1'
        self.resolution = 64
        self.interpolation = 'nearest'
        self.colormap = cm.bone
        
        ScatterPlotBrowse.__init__(self,*stuff,**stuffer)
        
    def _init_popupmenu(self):
        ScatterPlotBrowse._init_popupmenu(self)
        resolutions = [str(2**x) for x in range(1,10)]

        options = ['nearest', 'bilinear', 'bicubic', 'spline16', 'spline36',
    'hanning', 'hamming', 'hermite', 'kaiser', 'quadric',
    'catrom', 'gaussian', 'bessel', 'mitchell', 'sinc',
    'lanczos', 'blackman']
        smoothing = [str(1.5**x) for x in xrange(6)]
        smoothing = ["1/"+smoothing[x] for x in range(5,0,-1)] + smoothing            
        colours = [color for color in dir(cm) if isinstance(getattr(cm,color), matplotlib.colors.LinearSegmentedColormap)]
       
        self._popupmenu.add_separator()
        self._popupmenu.add_grouped_tooglebutton("Smoothing",smoothing,self.set_smoothing,'1')                
        self._popupmenu.add_grouped_tooglebutton("Density Resolution",resolutions,self.set_resolution,"64")        
        self._popupmenu.add_grouped_tooglebutton("Colours",colours, self.set_colour, self.colormap)                
        self._popupmenu.add_grouped_tooglebutton("Interpolation",options,self.set_interpolation, 'nearest')        

    def set_smoothing(self,ignored, smoothing):
        self.smoothing = smoothing
        self.update()

    def set_interpolation(self, ignored, interpolation):
        self.interpolation = interpolation 
        self.update()

    def set_resolution(self,ignored,text):
        self.resolution = int(text)
        self.update()        

    def set_colour(self, ignored, colour):
        self.colormap = getattr(cm,colour)
        self.update()    

    def draw_plot(self,x,y):
        # ensure MASS
        rpy.r.library("MASS")
        rpy.r.assign("x",x)
        rpy.r.assign("y",y)
        rpy.r("band <- c(bandwidth.nrd(x),bandwidth.nrd(y)) * %s" % (dataset.make_number_scientific(self.smoothing),))
        res = rpy.r("kde2d(x,y,h=band,n=%s)" % (self.resolution,)) 
        xmin, xmax, ymin, ymax = res['x'][0], res['x'][-1], res['y'][0], res['y'][-1]        
        self._ax.imshow(Numeric.transpose(res['z']),origin="lower",extent=(xmin, xmax, ymin, ymax),cmap=self.colormap,interpolation=self.interpolation)
                                    
        # set ticks!
        self._ax.set_xscale("linear")                
        self._ax.set_yscale("linear")                        

class NavToolbar(NavigationToolbar2GTK):

    # overwriting normal Toolbar
    toolitems = (
        (None, None, None, None),
        ('Home', 'Reset original view', 'home.png', 'home'),
        ('Back', 'Back to  previous view','back.png', 'back'),
        ('Forward', 'Forward to next view','forward.png', 'forward'),
        (None, None, None, None))
    
    def __init__(self, *args):        
        NavigationToolbar2GTK.__init__(self, *args)        
        self._select_callback = None
        
        import matplotlib
        
        switch_buttons = (
        ('Pan', 'Pan axes with left mouse, zoom with right', 'move.png','pan',matplotlib.rcParams['datapath']),
        ('Zoom', 'Zoom to rectangle','zoom_to_rect.png', 'zoom',matplotlib.rcParams['datapath']),
        ('Select','Select within rectangle',"select.png",'select',os.path.join(os.path.dirname(sys.modules['system'].__file__),"..", "icons")))

        togglebuttons = []
        for text, tooltip_text, image_file, callback, basedir in switch_buttons:
            fname = os.path.join(basedir, image_file)
            image = gtk.Image()
            image.set_from_file(fname)
            tbutton = gtk.ToggleButton()
            tbutton.set_border_width(1)            
            tbutton.set_image(image)
            tbutton.set_relief(gtk.RELIEF_NONE)
            titem = gtk.ToolItem()
            titem.add(tbutton)
            self.insert(titem, 0)            
            titem.set_tooltip(self.tooltips, tooltip_text, 'Private')
            togglebuttons.append((tbutton,getattr(self,callback)))        

        for button, method in togglebuttons:
            button.connect('clicked', self.button_clicked, togglebuttons)

        self.show_all()

    def button_clicked(self,button,togglebuttons):
        # actual actiosn connected elsewhere, this only pops other selectbuttons        
        if button.get_active():
            # unpress everythign else
            [b.set_active(False) for b, method in togglebuttons if b != button]

        # call method
        [method() for b, method in togglebuttons if b == button]
        self.set_message(self.mode)

    def select(self, *args):
        """Selection mode selected handler."""
        if self._active == 'SELECT':
            self._active = None
        else:
            self._active = 'SELECT'

        if self._idPress is not None:
            self._idPress = self.canvas.mpl_disconnect(self._idPress)
            self.mode = ''

        if self._idRelease is not None:
            self._idRelease = self.canvas.mpl_disconnect(self._idRelease)
            self.mode = ''

        if self._active:
            self._idPress = self.canvas.mpl_connect('button_press_event',
                                                    self.press_select)
            self._idRelease = self.canvas.mpl_connect('button_release_event',
                                                      self.release_select)

    def set_message(self, s):
        """Set status in toolbar to string s.

        Overrided to make sure message can be updated even when
        drawing rubberband.
        """
        self.message.set_label(s)

    def press_select(self, event):
        """Mouse button pressed handler for selection mode."""
        if event.button == 1:
            x, y = event.x, event.y

            for i, a in enumerate(self.canvas.figure.get_axes()):
                if event.inaxes == a and event.inaxes.get_navigate():
                    xmin, xmax = a.get_xlim()
                    ymin, ymax = a.get_ylim()
                    lim = xmin, xmax, ymin, ymax
                    self._xypress = x, y, a, i, lim, a.transData.deepcopy()
                    break
        
        self.press(event)

    def release_select(self, event):
        """Mouse button released handler for selection mode."""
        if event.button == 1:
            if self._xypress:
                x, y = event.x, event.y
                lastx, lasty, a, ind, lim, trans = self._xypress
                lastx, lasty = a.transData.inverse_xy_tup( (lastx, lasty) )
                x, y = a.transData.inverse_xy_tup( (x, y) )

                if self._select_callback:
                    self._select_callback(lastx, lasty, x, y)
        
            self._xypress = None
            self.draw()
            self.release(event)

    def mouse_move(self, event):
        """Extend NavigationToolbar2GTK.mouse_move to provide selection support."""
        from matplotlib.backend_bases import cursors        

        # Only update the rubberband for selection mode when mouse is
        # within the plotting area.
        if event.inaxes and self._active == 'SELECT':
            if self._lastCursor != cursors.SELECT_REGION:
                self.set_cursor(cursors.SELECT_REGION)
                self._lastCursor = cursors.SELECT_REGION
            if self._xypress is not None:
                x, y = event.x, event.y
                lastx, lasty, a, ind, lim, trans = self._xypress
                self.draw_rubberband(event, x, y, lastx, lasty)

        NavigationToolbar2GTK.mouse_move(self, event)

    def set_select_callback(self, listener):
        """Allow plots to register a callback for selection events.

        The callback will be called as listener(x1, y1, x2, y2).  All
        coordinates are in the plot coordinate system, not pixels or
        widget coordinates.
        """
        logger.log("debug","Plotselection callback registered for: " + str(listener))
        self._select_callback = listener
        
class TablePlot(Plot):
        
    def __init__(self, data,only_selected=False,inverted=False):
        self._data = data
        self.only_selected = only_selected
        self.inverted      = inverted        
        
        Plot.__init__(self, "Table " + data.get_name())
        
        # settings for all widgets:
        # (could be settable from GUI)
        self.y_pixels = 20
        self.x_pixels = 40

        # selections        
        self.selection = {}
                
        # storing labels for faster redrawing
        self.created_widgets = []
        self.array_start_x = 0
        self.array_start_y = 0
        self.y_widgets = 0
        self.x_widgets = 0
        self.last_widgets = (0,0)
        self.last_size    = (0,0)
        self.x_ids = []
        self.y_ids = []
                
        self.init_gui()
        self.show_all() # running this before size-allocate is connected is a good idea 
        self.connect("size-allocate",self.on_resized)        
        self.set_selection_listener(self.selection_changed)                                                
                
    def init_gui(self):
        # find new name
        name = self._data.get_name()
        if self.only_selected:
            if self.inverted:
                name = "Not Selected: " + name
            else:
                name = "Selected: " + name
        self.set_name(name)

        # init basic widgets                                                
        label = gtk.Label(name)
        table = gtk.Table()        
        vb = gtk.VBox()
        commands = gtk.HBox()                  
        
        # place them
        vb.pack_start(label,False,False)        
        vb.pack_start(table, False,False)
        vb.pack_start(commands,False,False) 
        vb2 = gtk.VBox() # (trying to avoid resizing madness)   
        hb = gtk.HBox()             
        hb.pack_start(vb, False,False)
        vb2.pack_start(hb,False,False) 
        self.add(vb2)
        hb.expand = False
        
        self.colnames = gtk.HBox()
        table.attach(self.colnames,1,2,0,1,xoptions=0,yoptions=0)        
        self.rownames = gtk.VBox()
        table.attach(self.rownames,0,1,1,2,xoptions=0,yoptions=0)        
                
        self.data_table = gtk.Table()
        table.attach(self.data_table,1,2,1,2,xoptions=0,yoptions=0)
               
        # addings icons
        self._arrows = []
        icons = [gtk.STOCK_GO_DOWN,gtk.STOCK_GO_UP] 
                             
        for icon in icons:        
            a = gtk.Button()
            im = gtk.Image()
            im.set_from_stock(icon,gtk.ICON_SIZE_MENU)            
            a.set_image(im)
            a.set_size_request(self.x_pixels/2, self.y_pixels)
            a.xalign = 0
            a.yalign = 0
            commands.pack_start(a,False)
            self._arrows.append(a)
                                 
        commands = gtk.HBox()
        table.attach(commands,0,1,0,1)
        icons = [gtk.STOCK_GO_BACK,gtk.STOCK_GO_FORWARD]            
        for icon in icons:
            a = gtk.Button()            
            im = gtk.Image()
            im.set_from_stock(icon,gtk.ICON_SIZE_MENU)            
            a.set_image(im)
            a.set_size_request(self.x_pixels/2, self.y_pixels)
            a.xalign = 0
            a.yalign = 0
            commands.pack_start(a,False)
            self._arrows.append(a)

        methods = (self._down_clicked, self._up_clicked,self._left_clicked,self._right_clicked)
        for button, method in zip(self._arrows, methods):
            button.connect("clicked",method)


    def get_space_for_widgets(self,width,height):        
        # compute space (height/width-1) <- leave space for resizing (so table can be downsized)
        # -2 , -1 for lines containing headline / identifiers                                   
        y_widgets = max(0,min(((height-1) / self.y_pixels)-2 , self._data.shape[0]))
        x_widgets = max(0,min(((width-5)  / self.x_pixels)-1 , self._data.shape[1]))
        return [y_widgets, x_widgets]                    

    def selection_changed(self, selection):
        fisk = {}
        
        if self.inverted:            
            for dim in self._data.get_dim_names():   
                if dim in selection:
                    fisk[dim] = [id for id in self._data.get_identifiers(dim) if id not in selection[dim]]
                else:
                    fisk[dim] = set(self._data.get_identifiers(dim))
                self.selection = fisk
        else:
            self.selection = selection
                            
        if self.only_selected:
            # with need selection, max possible to show may have been changed
            self.resize_from(*self.last_size)
            self.update()        
         
    def _left_clicked(self,button):
        self.array_start_x -= 1
        self.update()
        
    def _right_clicked(self,button):
        self.array_start_x += 1
        self.update()
        
    def _up_clicked(self,button):
        self.array_start_y -= 1
        self.update()
        
    def _down_clicked(self,button):            
        self.array_start_y += 1
        self.update()                    
        
    def on_resized(self, frame, new_size):
        width, height = new_size.width, new_size.height
        self.resize_from(width,height)
        
    def resize_from(self, width, height):
        # compute space  
        self.last_size= (width,height)                                     
        self.y_widgets, self.x_widgets = self.get_space_for_widgets(width,height)
        self.update_ids_to_show()
        
        # did resize change max size?
        if self.last_widgets == (self.y_widgets, self.x_widgets):
            return
            
        self.last_widgets = (self.y_widgets, self.x_widgets)
        
        mins = [self.array_start_x,self.array_start_y]
        nums = [self.y_widgets,self.x_widgets]
        boxes = [self.rownames, self.colnames]

        # add new boxes
        for box in boxes:
            [box.remove(child) for child in box.get_children()] 
                         
        for box, dim, min_num, num in zip(boxes,self._data.get_dim_names(),mins,nums):            
            for id in self._data.get_identifiers(dim)[min_num:min_num+num]:
                label = gtk.Entry()
                label.set_editable(False)
                label.set_size_request(self.x_pixels, self.y_pixels)
                box.pack_start(label,expand=False)
                
        # add actual data!
        [self.data_table.remove(child) for child in self.data_table.get_children()]
    
        self.created_widgets = []
        for j,y in enumerate(xrange(self.array_start_y, self.array_start_y+self.y_widgets)):
            self.created_widgets.append([])
            for i,x in enumerate(xrange(self.array_start_x, self.array_start_x + self.x_widgets)):                
                a = gtk.Entry()
                a.set_text(str(self._data._array[y,x]))
                a.set_editable(False)
                a.set_size_request(self.x_pixels, self.y_pixels)
                self.created_widgets[-1].append(a)
                self.data_table.attach(a,i,i+1,j,j+1)
        self.show_all()                                                                                                            
        self.update()

    def update_ids_to_show(self):            
        # note: inverted selections are handled when set
        dims = self._data.get_dim_names()
        ids  = [self._data.get_identifiers(dim) for dim in dims]
        newids = [self._data.get_identifiers(dim) for dim in dims]
        
        # if using selection, take  of selected and available        
        if self.only_selected:            
            if dims[0] in self.selection:
                newids[0] = [id for id in ids[0] if id in self.selection[dims[0]]]
            else:
                newids[0] = []
            if dims[1] in self.selection:
                newids[1] = [id for id in ids[1] if id in self.selection[dims[1]]]
            else:
                newids[1] = []
                             
        # *Special Case*, if we have selections on one dimension and not the other, show everything                                 
        if newids[0] and not newids[1]:
            newids[1] = ids[1]
        if newids[1] and not newids[0]:
            newids[0] = ids[0]
               
        # make max/start values sane
        self.x_widgets = min(self.x_widgets,len(newids[1]))
        self.y_widgets = min(self.y_widgets,len(newids[0]))
        
        self.array_start_y = max(0,min(len(ids[0])-self.y_widgets, self.array_start_y))
        self.array_start_x = max(0,min(len(ids[1])-self.x_widgets, self.array_start_x))
        x_ids = newids[0][self.array_start_y:self.array_start_y+self.y_widgets]
        y_ids = newids[1][self.array_start_x:self.array_start_x+self.x_widgets]
        self.x_ids = [(ids[0].index(id),id) for id in x_ids]
        self.y_ids = [(ids[1].index(id),id) for id in y_ids]

        # update arrow buttons
        # set buttons     
        self._arrows[0].set_sensitive(self.array_start_y+self.y_widgets <len(newids[0]))
        self._arrows[1].set_sensitive(self.array_start_y)        
        self._arrows[2].set_sensitive(self.array_start_x)       
        self._arrows[3].set_sensitive(self.array_start_x+self.x_widgets < len(newids[1])) 
        
                                            
    def update(self):                       
        # add labels                                
        boxes = [self.rownames, self.colnames]
        dims = self._data.get_dim_names()
        self.update_ids_to_show()        

        if not self.x_ids or not self.y_ids:
            return

        for boxen, ids in zip(boxes,[self.x_ids,self.y_ids]):
            for (pos,id), box in zip(ids,boxen.get_children()):
                box.set_text(id)
                         
        for j,(x,id) in enumerate(self.x_ids):
            for i,(y,id) in enumerate(self.y_ids):             
                self.created_widgets[j][i].set_text(str(self._data._array[x,y]))                                                                                                     
        
    
    
    def get_description(self):
            dims = zip(self._data.get_dim_names(), self._data.shape)
            dim_text = ", ".join(["%s (%d)" % dim for dim in dims])                             
            text = """\
    <span weight="bold">Data:</span> %s
    <span weight="bold">Dimensions:</span> %s""" % (self._data.get_name(), dim_text)  
            return text

class ImagePlot(Plot):
    def __init__(self, data):
        Plot.__init__(self, data.get_name())
        self._data = data
        
        self._show_colorbar = True
        self.interpolation = 'blackman'
        self.colormap = cm.bone
        
        self.selection = None
        
        self._fig = Figure(figsize=(5,4), dpi=72)
        self._canvas = FigureCanvas(self._fig)                
        self.add(self._canvas)
        self._canvas.show()
        self.axes = []
        self.update_axis()
                
        self._toolbar = ScatterPlotToolbar(self._canvas, None)
        self._init_popupmenu()
                                                    
        self.update()
    
    def get_toolbar(self):
        return self._toolbar
            
    def get_popupmenu(self):
        return self._popupmenu            
       
    def _init_popupmenu(self):
        self._popupmenu = CustomPopupMenu(self)        
        self._popupmenu.init_from_toolbar(self.get_toolbar())
        options = ['nearest', 'bilinear', 'bicubic', 'spline16', 'spline36',
    'hanning', 'hamming', 'hermite', 'kaiser', 'quadric',
    'catrom', 'gaussian', 'bessel', 'mitchell', 'sinc',
    'lanczos', 'blackman']
        
        colours = [color for color in dir(cm) if isinstance(getattr(cm,color), matplotlib.colors.LinearSegmentedColormap)]
        
        self._popupmenu.add_grouped_tooglebutton("Colours",colours, self.set_colour, self.colormap)                
        self._popupmenu.add_grouped_tooglebutton("Interpolation",options,self.set_interpolation, 'bicubic')        
        self._popupmenu.add_separator()
        self._popupmenu.add_togglebutton("Colorbar",self._toggle_colorbar,self._show_colorbar)

        
    def set_colour(self, ignored, colour):
        self.colormap = getattr(cm,colour)
        self.update_axis()
        self.update()    

    def _toggle_colorbar(self,ignored):
        self._show_colorbar = not self._show_colorbar        
        self.update_axis()
        self.update()
    
    def update_axis(self):
        
        map(self._fig.delaxes, self.axes)
        if self._show_colorbar:
            ax = matplotlib.axes.Axes(self._fig, [0.1,0.1,0.75,0.85])
            ax = self._fig.add_axes(ax)
            ax2 = matplotlib.axes.Axes(self._fig, [0.90,0.1,0.04,0.85])
            ax2 = self._fig.add_axes(ax2)
            self.axes = [ax, ax2]            
        else:
            ax = matplotlib.axes.Axes(self._fig, [0.1,0.05,0.80,0.90])
            ax = self._fig.add_axes(ax)
            self.axes = [ax]  
                               
    def update(self):
        
        [ax.clear for ax in self.axes]
        
        arr = self._data.asarray().copy()
        mi = min(min(arr))
        if self.selection:
            d1, d2 = self._data.get_dim_names()
                    
            if d1 in self.selection:
                ids = self._data.get_identifiers(d1)
                for id in list(set(self.selection[d1]).union(ids)):
                    arr[ids.index(id),:] = mi
                    
            if d2 in self.selection:
                ids = self._data.get_identifiers(d2)
                for id in list(set(self.selection[d1]).union(ids)):
                    arr[:,ids.index(id)] = mi                                
                                                                                
        image = self.axes[0].imshow(self._data.asarray(),cmap=self.colormap,interpolation=self.interpolation)
        
        # force update of axes
        self.axes[0].set_xscale("linear")                
        self.axes[0].set_yscale("linear")
                                      
        if self._show_colorbar:       
            ax = self._fig.colorbar(image,cax=self.axes[1])
            ax.set_yscale("linear")
            ax.set_xscale("linear")

        self._canvas.draw()                                      
            
    def set_interpolation(self, ignored, interpolation):
        self.interpolation = interpolation 
        self.update()

    def selection_changed(self, selection):
        if selection != self.selection:
            self.selection = selection.copy() 
            self.update()
            
class ImagePlots(ImagePlot): 
    """ Plot showing several different imageplots"""
    def __init__(self, data):
        self.range = (0,data.shape[2]-1)
        self.data = data
        self.slice = 0 # -1 to force update, see valinit in Slider for actual init value
        self.v = min(Numeric.ravel(data.asarray())) , max(Numeric.ravel(data.asarray()))
        self.scplot = None
        
        ImagePlot.__init__(self, data)
                                            
        self.ax2 = matplotlib.axes.Axes(self._fig, [0.3,0.05,0.60,0.05])
        self.ax2 = self._fig.add_axes(self.ax2)

        # Name, min, max and valinit=start_position , valfmt='%i' makes widget display only ints
        self.slider = Slider(self.ax2, "Choose Slice", self.range[0], self.range[1], valinit=0, valfmt='%i'  )
        # register callback for slider changed
        self.slider.on_changed(self.update_slice)                        
        self._canvas.mpl_connect('button_press_event',self.on_mouseclick)
          
    def set_scatterplot(self, plot):
        self.scplot = plot

    def update_slice(self, val):
        val = int(val)
        if val != self.slice:
            self.slice = val
            self.update()
                                        
    def update(self):
        
        [ax.clear() for ax in self.axes]
        image = self.axes[0].imshow(self._data.asarray()[:,:,self.slice],cmap=self.colormap,interpolation=self.interpolation, vmin = self.v[0], vmax=self.v[1])
        
        # force update of axes
        self.axes[0].set_xscale("linear")                
        self.axes[0].set_yscale("linear")
                                      
        if self._show_colorbar:       
            ax = self._fig.colorbar(image,cax=self.axes[1])
            ax.set_yscale("linear")
            ax.set_xscale("linear")

        self._canvas.draw()
                                                                                                           
    def on_mouseclick(self,event):
        if event.inaxes == self.axes[0]:
            if self.scplot:
                self.scplot._data = dataset.Dataset([self.v[1]]+list(self.data.asarray()[int(self.data.shape[0]-event.ydata),int(event.xdata),:]))
                self.scplot._dim = self.scplot._data[0]
                self.scplot._id  = self.scplot._data[0,0]
                self.scplot.update()              
            
class ScatterPlotToolbar(NavToolbar):
    def __init__(self, *args,**kwargs):
        NavToolbar.__init__(self, *args)
                      
    def add_switch_button(self):    
        fname = os.path.join(os.path.dirname(sys.modules['system'].__file__), "..", "icons", "switch_var.png")
        image = gtk.Image()
        image.set_from_file(fname)
        tbutton_switch_var = gtk.ToolButton(image,"Use Grid")        
        num = self.get_n_items() 
        self.insert(tbutton_switch_var, num-2)
        self.show_all()
        return tbutton_switch_var
        
    def add_spin_button(self, max_elements, initial,label="Comps:" ):    
        spinbutton = gtk.SpinButton(climb_rate=1)
        spinbutton.set_range(1, max_elements)
        spinbutton.set_value(initial)
        spinbutton.set_increments(1,5)        
        hbox = gtk.HBox()
        label_ = gtk.Label(label)
        toolitem = gtk.ToolItem()                          
        toolitem.set_expand(False)
        toolitem.set_border_width(3)
        toolitem.add(hbox)        
        hbox.pack_start(label_)        
        hbox.pack_start(spinbutton)        
        num = self.get_n_items() 
        self.insert(toolitem, num-2)
        toolitem.set_tooltip(self.tooltips, "Number of components used for calculating leverage")
        self.show_all()
        return spinbutton

    def add_legend_button(self):
        tbutton_legend = gtk.ToolButton(gtk.STOCK_INFO)       
        num = self.get_n_items() 
        self.insert(tbutton_legend, num-2)               
        tbutton_legend.set_tooltip(self.tooltips, 'Toggle Legend')
        self.show_all()
        return tbutton_legend
        
    def add_up_button(self,tooltip = 'Browse down on y-axis' ):
        tbutton_up = gtk.ToolButton(gtk.STOCK_GO_UP)
        num = self.get_n_items() 
        self.insert(tbutton_up, num-2)
        tbutton_up.set_tooltip(self.tooltips, tooltip)
        self.show_all()
        return tbutton_up
                
    def add_down_button(self,tooltip='Browse up on y-axis'):
        tbutton_down = gtk.ToolButton(gtk.STOCK_GO_DOWN)
        num = self.get_n_items() 
        self.insert(tbutton_down, num-2)
        tbutton_down.set_tooltip(self.tooltips, tooltip)
        self.show_all()
        return tbutton_down
                        
    def add_left_button(self, tooltip='Browse back on x-axis'):
        tbutton_back = gtk.ToolButton(gtk.STOCK_GO_BACK)
        num = self.get_n_items() 
        self.insert(tbutton_back, num-2)
        tbutton_back.set_tooltip(self.tooltips, tooltip)
        self.show_all()
        return tbutton_back
                    
    def add_right_button(self,tooltip='Browse forward on x-axis'):
        tbutton_forward = gtk.ToolButton(gtk.STOCK_GO_FORWARD)    
        num = self.get_n_items() 
        self.insert(tbutton_forward, num-2)        
        tbutton_forward.set_tooltip(self.tooltips,)
        self.show_all()        
        return tbutton_forward
    
    def add_arrows_button(self,tooltip="Set x and y-axis",pos=-2):
        fname = os.path.join(os.path.dirname(sys.modules['system'].__file__), "..", "icons", "arrows.png")
        image = gtk.Image()
        image.set_from_file(fname)
        tbutton_arrows = gtk.ToolButton(image, 'Arrows')
        num = self.get_n_items() 
        self.insert(tbutton_arrows, num+pos)        
        tbutton_arrows.set_tooltip(self.tooltips,)
        self.show_all()        
        return tbutton_arrows
      
    def add_combobox(self, elements,label="Shown:",tooltip='Select Number of Elements'):                                        
        toolitem = gtk.ToolItem()
        combobox = gtk.combo_box_entry_new_text()
        for choice in elements:
            combobox.append_text(str(choice))       
        combobox.set_active(len(elements)-1)                        
        hbox = gtk.HBox()                        
        toolitem.add(hbox)
        hbox.pack_start(gtk.Label(label))
        hbox.pack_start(combobox)
        toolitem.set_expand(False)
        toolitem.set_border_width(7)
        combobox.set_size_request(80,15)        
        toolitem.set_tooltip(self.tooltips,tooltip,'private')
        num = self.get_n_items()                                                          
        self.insert(toolitem, num-2)        
        self.show_all()
        return combobox     
        
class CustomPopupMenu:
    def __init__(self,widget):
        self._items  = []  # save figure, 
        self._toggle_items = []  # 
        self._newwindow_callback = None   # objects concerning the plot / workflow
        widget.connect("button-release-event", self.button_released_event)
        self.generate_menu()
        
    def generate_menu(self):
        menu  = gtk.Menu()               
        widgets = []
        
        if self._newwindow_callback:            
            # add callback
            callback, params = self._newwindow_callback
            item = gtk.MenuItem(label="Move to New Window")
            menu.append(item)
            item.connect("activate",callback,*params)
            item.show()            
            
            # add separator
            sep = gtk.SeparatorMenuItem()
            menu.append(sep)
            sep.show()
                                                                                       
        for text, callback in self._items:                                            
            item = gtk.MenuItem(label=text)            
            menu.append(item)
            item.connect("activate", callback)
            item.show()
                    
        sep = gtk.SeparatorMenuItem()
        menu.append(sep)
        sep.show()
    
        for text, callback,state in self._toggle_items:
            if not text:
                item = gtk.SeparatorMenuItem()
            else:
                if isinstance(text,str) and state!=None:
                    item = gtk.CheckMenuItem(label=text)
                    item.set_active(state)
                    item.connect("toggled", callback)
                elif isinstance(text,str):
                    item = gtk.MenuItem(label=text)
                    item.connect("activate",callback)                    
                else:
                    item = gtk.MenuItem(text[0])
                    submenu = gtk.Menu()
                    item.set_submenu(submenu)
                    
                    items  = [None]                                                                                                    
                    for t in text[1:]:
                        items.append(gtk.RadioMenuItem(items[-1],t))                    
                        if t==state:
                            items[-1].set_active(True)                        
                        items[-1].connect("toggled",callback,t)
                    items.remove(None)
                        
                    for r in items:
                        submenu.append(r)
                        
                                                                                            
                    submenu.show_all()
                                                                                

                                                                                                                
            menu.append(item)
            item.show()        
                        
        self.menu = menu
                           
    def init_from_toolbar(self, toolbar):
        # adds save figure and configure subplots menu items taken from NavToolbar
        self._items.append(("Configure",toolbar.configure_subplots))
        self._items.append(("Save Figure", toolbar.save_figure))
        self.generate_menu()

    # callback called with button as first param.
    def add_togglebutton(self,text,callback,state):
        state = bool(state)                        
        self._toggle_items.append((text,callback,state))
        self.generate_menu()

    # callback called with button,text as parameters            
    def add_grouped_tooglebutton(self, name, texts, callback,selected):
        self._toggle_items.append(([name]+texts,callback,selected))
        self.generate_menu()

    def add_button(self,text,callback):
        self._toggle_items.append((text,callback,None))                                                    
        self.generate_menu()
        
    def add_separator(self):
        self._toggle_items.append((None,None,None))
        self.generate_menu()                
                  
    def set_show_in_new_window_callback(self,callback,params):
        self._newwindow_callback = callback, params
        self.generate_menu()                
                            
    def button_released_event(self, list, event):
        """Show context menu on rightclick."""            

        # only show menu on right click
        if event.button != 3:
            return
                                                                    
        self.menu.popup(None, None, None, 0, 0)      
