import os
import sys
import pygtk
import gtk
import gtk.gdk
import gtk.glade
import gobject
import scipy
import pango
import traceback
from exceptions import IndexError
from system import project, workflow, dataset, logger, plots, navigator, views
from system import dialogs, workflowimport, config, filehandlers
import webbrowser
import cPickle


DATADIR = os.path.dirname(sys.modules['system'].__file__)
GLADEFILENAME = os.path.join(DATADIR, 'scicraft.glade')


class _PreferencesDialog:
    def __init__(self, widget_tree):
        self._widget_tree = widget_tree
        self._dialog = widget_tree.get_widget('prefsdialog')
        self._pathview = widget_tree.get_widget('workflow_path_view')

        self._store = gtk.ListStore(gobject.TYPE_STRING)
        for path in config.user_workflow_paths:
            self._store.insert_before(None, (path,))
        self._pathview.set_model(self._store)

        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Blarg")
        column.pack_start(cell)
        column.set_attributes(cell, text=0)
        self._pathview.insert_column(column, -1)

        widget_tree.signal_autoconnect({'on_add':self.on_add, 'on_remove':self.on_remove})

    def on_add(self, *rest):
        chooser = gtk.FileChooserDialog(title="Select additional workflow path...",
                                        parent=None,
                                        action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                                        buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                                 gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        chooser.set_default_response(gtk.RESPONSE_OK)

        if chooser.run() == gtk.RESPONSE_OK:
            path = chooser.get_filename()
            if path not in self._current_paths():
                self._store.insert_before(None, (path,))
        chooser.destroy()

    def _current_paths(self):
        """Get all strings currently in path list store."""
        all_paths = []
        current = self._store.get_iter_first()
        while current:
            all_paths.append(self._store.get_value(current, 0))
            current = self._store.iter_next(current)
        return all_paths

    def on_remove(self, *rest):
        selection = self._pathview.get_selection()
        model, treeiter = selection.get_selected()
        if treeiter:
            model.remove(treeiter)

    def run(self):
        if self._dialog.run() == gtk.RESPONSE_OK:
            config.user_workflow_paths = self._current_paths()
            config.write_config()
        self._dialog.destroy()


class SciCraftApp:

    def __init__(self, wf):
        # Application variables
        self.project = None
        self._last_view = None
        self._plot_toolbar = None

        self._workflow = wf(self)

        # widgets to be assigned during xml-loading
        self._main_view = None
        self._navigator_view = None
        self._selection_view = None
        self._wf_menu = None
        self._widget_tree = None

        # load, create and connect widgets
        self._init_gui()    
               
    def set_project(self, project):
        self.project = project
        self._navigator_view.add_project(self.project)
        self._selection_view.add_project(self.project)

    def set_workflow(self, workflow):
        self._workflow = workflow
        self._wf_menu.set_workflow(self._workflow)
    
    def set_version(self, version):
        self.version = version
        self._widget_tree.get_widget('appwindow').set_title("SciCraft " + version)

    def custom_object_factory(self, glade, function_name, widget_name,\
                              str1, str2, int1, int2):
        def create_logview():
            return logger.LogView(logger.logger)

        def create_main_view():
            self._main_view = views.MainView()
            return self._main_view

        def create_small_view():
            return plots.SmallView()
        
        def create_large_view():
            return plots.LargeView()

        def create_navigator_view():
            self._navigator_view = navigator.NavigatorView(GLADEFILENAME)
            return self._navigator_view

        def create_selection_view():
            self._selection_view = navigator.SelectionView(GLADEFILENAME) 
            return self._selection_view             

        def create_workflow_view():
            self._wf_menu = workflow.WorkflowView(self, self._workflow)
            return self._wf_menu

        def create_console_view():
            # creates console view, copies stdout, stderr to console
            console_text = gtk.TextView()
            console_text.set_editable(False)
            console_buffer = console_text.get_buffer()
            sys.stdout = std_sink(console_buffer, sys.stdout)
            sys.stderr = std_sink(console_buffer, sys.stderr)
            return console_text

        def create_information_view():
            logger.information.set_markup("Nothing selected.")
            return logger.information           
            
        handler = locals()[function_name]
        return handler()
        
    def _connect_logger_autojump(self):
        """Set logger to be shown on errors/warnings."""
        def row_change_handler(store, path, iter):
            value = store.get_value(iter, 0)
            if value in ("error", "warning"):
                self._widget_tree.get_widget('notebook2').set_current_page(1)

        logger.logger.store.connect("row-changed", row_change_handler)

    def on_quit(self, *rest):
        if self.project.is_dirty():
            qres = dialogs.get_confirm_quit("Closing SciCraft...", "Save project before quit?")
            if qres == "QUIT":
                gtk.main_quit()
            elif qres =="SAVE":
                self.on_save(self)
                # we only quit if user actually saved in last step
                if not self.project.is_dirty():
                    gtk.main_quit()
        else: 
            gtk.main_quit()
        return True 
        
    def _init_gui(self):
        def zoom_in(button):
            self._main_view.goto_large()

        def zoom_out(button):
            self._main_view.goto_small()

        def on_view_changed(widget, vf):
            self._update_toolbar(vf.get_view())

        # point gtk in the direction of our factory methods so it'll
        # know how to create custom widgets in the glade file
        gtk.glade.set_custom_handler(self.custom_object_factory)
        
        self._widget_tree = gtk.glade.XML(GLADEFILENAME, 'appwindow')
        
        # dependency injection after creation
        self._wf_menu.set_statusbar(self._widget_tree.get_widget('statusbar'))
        self._wf_menu.set_gdk_window(self._widget_tree.get_widget('appwindow').window)
        self._navigator_view.set_plot_main_view(self._main_view)
        
        # sets logger to grab_focus on errors/warnings
        self._connect_logger_autojump()                        
        self['appwindow'].set_size_request(800, 600)

        # connect buttons
        self['zoom_in_button'].connect("clicked", zoom_in)
        self['zoom_out_button'].connect("clicked", zoom_out)
        self['new_button'].connect("clicked", self.on_create_project)
        self['save_button'].connect("clicked", self.on_save)
        self['open_button'].connect("clicked", self.on_open)
        self['workflow_refresh_button'].connect("clicked", self.on_workflow_refresh_clicked)
        
        # connect export/import buttons
        self['delimited_text1'].connect("activate",self.on_import_clicked)
        self['r2'].connect("activate",self.on_import_clicked)
        self['matlab2'].connect("activate",self.on_import_clicked)
        self['excel_read'].connect("activate",self.on_import_clicked) 
        
        # if not available, disable Microsoft Excel button
        if not filehandlers.ensure_excel_available(show_warnings = False):
            self['excel_read'].set_sensitive(False)                  
        
        # connect menuitems
        self['menu_new'].connect("activate", self.on_create_project)
        self['menu_help_manual'].connect("activate", self.on_help_index)
        self['microsoft_excel_interaction1'].connect("activate", self.on_excel_help)
        self['menu_help_getting_started'].connect("activate", self.on_help_getting_started)
        self['menu_help_report_bug'].connect("activate", self.on_help_report_bug)
        self['menu_help_about'].connect("activate", self.on_help_about)
        self['menu_quit'].connect("activate", self.on_quit)
        self['menu_save'].connect("activate", self.on_save)
        self['menu_save_as'].connect("activate", self.on_save_as)
        self['menu_open'].connect("activate", self.on_open)
        self['menu_preferences'].connect("activate", self.on_preferences)
        self['download_and_update_now1'].connect("activate", self.on_update)

        # connect other events
        self['appwindow'].connect("delete_event", self.on_quit)
        self['workflow_combo'].connect("changed", self.on_change_workflow)
        self['main_view'].connect('view-changed', on_view_changed)                        

        self['appwindow'].show_all()

        self._update_workflow_combo()
        # Log that we've set up the app now
        logger.log('debug', 'Program started')
        
        # set correct page on logger
        self['notebook2'].set_current_page(0)
                     
        # revolving labels code
        def turn_label(widget, event,label):
            if event.button == 2:
                for x in xrange(360):
                    label.set_angle(x)
                    if gtk.events_pending():
                        gtk.main_iteration_do()
                label.set_angle(0)
        stuff = [self['appwindow'].get_children()]
        while(stuff):
            item = stuff.pop()
            if isinstance(item, gtk.Label):
                item.connect                                        
        self['eventbox2'].connect("button-press-event", turn_label,self['label65'])
        self['eventbox1'].connect("button-press-event", turn_label,self['label10'])                            
        self['eventbox3'].connect("button-press-event", turn_label,self['label11'])
        self['eventbox4'].connect("button-press-event", turn_label,self['label15'])
        self['eventbox5'].connect("button-press-event", turn_label,self['label64'])
        self['eventbox6'].connect("button-press-event", turn_label,self['label9'])
        self['eventbox7'].connect("button-press-event", turn_label,self['label12'])

    def _update_workflow_combo(self):
        combo = self['workflow_combo']
        store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
        index = None
        workflows = dict([(wf.name, wf) for wf in workflowimport.workflow_list()])
        names = workflows.keys()
        names.sort()
        for i, name in enumerate(names):
            wf = workflows[name]
            # make sure current workflow is still selected if
            # available
            if self._workflow.__class__.name == name:
                index = i
            store.insert_before(None, (wf.name, wf))

        cell = gtk.CellRendererText()
        combo.set_model(store)
        if combo.get_text_column() == -1:
            # still unset
            combo.set_text_column(0)
        combo.child.set_editable(False)
        combo.set_active(0)
        if index != None:
            combo.set_active(index)

    def on_change_workflow(self, combo):
        it = combo.get_active_iter()
        wf = combo.get_model().get_value(it, 1)
        self.set_workflow(wf(self))

    def on_open(self, *rest):
        if not self.project.is_dirty() or dialogs.get_yes_no("Open project...",
                                                             "Really close existing project without saving?"):
            filename = dialogs.get_open_filename("Open project...",
                                                 ("SciCraft Project", "scp"))

            if filename:
                p = project.Project(filename)
                p.import_data_tree(cPickle.load(open(filename, "rb")))
                self.set_project(p)
                self._main_view.clear_plots()

    def on_preferences(self, *rest):
        widget_tree = gtk.glade.XML(GLADEFILENAME, 'prefsdialog')
        prefs = _PreferencesDialog(widget_tree)
        prefs.run()
        self._update_workflow_combo()

    def on_save(self, *rest):
        filename = self.project.get_filename()
        if not filename:
            filename = dialogs.get_save_filename("Save project...", "Project1.scp",
                                                 ("SciCraft Project", "scp"))
        if not filename:
            return

        cPickle.dump(self.project.extract_data_tree(), open(filename, "wb"),
                     protocol=2)
        self.project.set_filename(filename)
        logger.log("notice", "Saved project to %r." % filename)

    def on_save_as(self, *rest):
        name = self.project.get_filename() or "Project1.scp"
        filename = dialogs.get_save_filename("Save project...", name,
                                             ("SciCraft Project", "scp"))
        if not filename:
            return

        cPickle.dump(self.project.extract_data_tree(), open(filename, "wb"),
                     protocol=2)
        self.project.set_filename(filename)
        logger.log("notice", "Saved project to %r." % filename)
    
    def __getitem__(self, key):
        return self._widget_tree.get_widget(key)

    def on_create_project(self, *rest):
        if not self.project.is_dirty() or dialogs.get_yes_no("New project...",
                                                             "Really close existing project without saving?"):
            self.set_project(project.Project())
            self._main_view.clear_plots()
        
    def on_help_about(self, *rest):
        widget_tree = gtk.glade.XML(GLADEFILENAME, 'aboutdialog')
        about = widget_tree.get_widget('aboutdialog')
        about.set_version(self.version)
        about.run()

    def on_help_index(self, *ignored):
        webbrowser.open_new('https://dev.scicraft.org/trac/wiki/SciCraftManual')

    def on_help_getting_started(self, *ignored):
        webbrowser.open_new('https://dev.scicraft.org/trac/wiki/GettingStarted')

    def on_help_report_bug(self, *ignored):
        webbrowser.open_new('https://dev.scicraft.org/trac/newticket?version='+self.version)
    
    def on_excel_help(self,*ignored):
        webbrowser.open_new('https://dev.scicraft.org/trac/wiki/ExcelCommunication')

    def on_workflow_refresh_clicked(self, *ignored):
        try:
            workflows = dict([(wf.__name__, wf) for wf in workflowimport.workflow_list()])
            self.set_workflow(workflows[self._workflow.__class__.__name__](self))
        except Exception, e:
            etype, value, tb = sys.exc_info()
            failed = "".join(traceback.format_exception(etype, value, tb, None))
            logger.log('error', 'workflow failed to load - %s: %s' % (etype, str(value).strip()))
            logger.log('debug', failed)
        else:
            logger.log('notice', 'Successfully reloaded workflow')

    def on_update(self, button):
        
        if os.name=='nt':
            basedir = os.path.abspath(os.curdir)        
            program = "\"" + os.path.abspath(os.curdir +"\\..") + "\\subversion\\bin\\svn.exe" + "\""        
        else:                       
            program = 'svn'
            
        cmd     = " update "        
        os.system(program + cmd)    
        self.on_quit()                                 
                        

    def _update_toolbar(self, view):
        logger.log("debug", "view changed to %s" % view)
                    
        # update toolbar        
        self._last_view = view
        
        if not view.parent.has_toolbar_dock():
            window = self._widget_tree.get_widget('plot_toolbar_dock')
        else:
            window = view.parent.get_toolbar_dock()

        # return if no change needed
        if self._last_view == view and window == view.parent:        
            return
                
        if self._plot_toolbar and self._plot_toolbar.parent:
            self._plot_toolbar.parent.remove(self._plot_toolbar)
            
        self._plot_toolbar = view.get_toolbar()
        
        if self._plot_toolbar:
            window.add(self._plot_toolbar)

        # update right-click meny
        menu = view.get_popupmenu()
        
        if menu:
            def call_new_plot(button,object,view):
                object.new_plot_window(view)
                            
            menu.set_show_in_new_window_callback(call_new_plot,[self._main_view,view])                  

    def on_import_clicked(self, button):
        if button.get_name()=="matlab2":
            filehandlers.MatFileLoadFunction().insert_into(self.project)
        elif button.get_name()=="r2":
            filehandlers.RLoadFunction().insert_into(self.project)
        elif button.get_name()=="delimited_text1":
            filehandlers.TextFileReader().insert_into(self.project)
        elif button.get_name()=="excel_read":
            filehandlers.ExcelReader().insert_into(self.project)
        
    


class std_sink:
    def __init__(self, textbuffer, alternate):
        self.textbuffer = textbuffer
        self.alternate  = alternate
        
    def write(self, text):
        self.alternate.write(text)
        self.textbuffer.insert(self.textbuffer.get_end_iter(), text)
    
    def flush(self):
        self.alternate.flush()
