import gtk, gobject
import scipy, Numeric
import copy
import os
import sys
from system import logger, dataset, plots
import matplotlib
from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
from matplotlib.backends.backend_gtkagg import NavigationToolbar2GTK
from matplotlib.figure import Figure
from matplotlib.numerix import arange, sin, pi
from matplotlib import cm

DATADIR = os.path.dirname(sys.modules['system'].__file__)
GLADEFILENAME = os.path.join(DATADIR, 'scicraft.glade')

class ObjectTable:
    """A 2D table of elements."""

    def __init__(self, xsize=0, ysize=0, creator=None):
        self._elements = []
        self._creator = creator or (lambda : None)
        self.xsize = xsize
        self.ysize = ysize
        self.resize(xsize, ysize)

    def resize(self, xsize, ysize):
        """Resizes the table by removing and creating elements as required."""
        # Delete or append new rows
        del self._elements[ysize:]
        new_rows = ysize - len(self._elements)
        self._elements.extend([list() for i in range(new_rows)])

        # Delete or append new columns
        for row in self._elements:
            del row[xsize:]
            new_elems = xsize - len(row)
            row.extend([self._creator() for i in range(new_elems)])

    def __getitem__(self, (x, y)):
        return self._elements[y][x]

    def __setitem__(self, (x, y), value):
        self._elements[y][x] = value

    def as_list(self):
        retlist = []
        [retlist.extend(items) for items in self._elements]
        return retlist


class ViewFrame (gtk.Frame):
    """A ViewFrame is a gtk bin that contains a view.
    
    The ViewFrame is either active or inactive, and belongs to a group
    of ViewFrames of which only one can be active at any time.
    """

    def __init__(self, view_frames):
        gtk.Frame.__init__(self)
        self.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.focused = False
        self.view_frames = view_frames
        
        if not view_frames:
            # first viewframe created!
            self.focus()
        
        self.empty_view = plots.EmptyPlot()
        self._button_event = None
        self._key_press_event = None
        self._key_release_event = None
        
        view_frames.append(self)
        
        # Get dropped views
        self.drag_dest_set(gtk.DEST_DEFAULT_ALL,
                           [("GTK_TREE_MODEL_ROW", gtk.TARGET_SAME_APP, 7)],
                           gtk.gdk.ACTION_LINK)
        self.connect("drag-data-received", self.on_drag_data_received)
        
        # Set view
        self._view = self.empty_view
        self._view.connect("button-press-event", self.on_button_press_event)
        self.add(self._view)
        self._view.show()
        self.show()
        
    def focus(self):
        """Gets focus and ensures that no other window is in focus."""
        self.emit('focus-changed', self, True)

        self.on_mouse_entered(self)
    
        if self.focused:
            return self
                
        for frame in self.view_frames:
            frame.unfocus()
        
        parent = self
        while(parent!=None and not isinstance(parent, gtk.Window)):
            parent = parent.get_parent()
        if parent:
            self._key_press_event = parent.connect("key-press-event",self._view.on_key_press)
            self._key_release_event = parent.connect("key-release-event",self._view.on_key_release)
                
        self.set_shadow_type(gtk.SHADOW_IN)
        self.focused = True

        
    def on_mouse_entered(self, widget):
        if hasattr(self,"_view"):
            logger.information.set_markup(self._view.get_description())

    def unfocus(self):
        """Removes focus from the ViewFrame. Does nothing if unfocused."""
        if not self.focused:
            return
            
        parent = self
        while(parent!=None and not isinstance(parent, gtk.Window)):
            parent = parent.get_parent()
        if parent:            
            if self._key_press_event:
                parent.disconnect(self._key_press_event)
            if self._key_release_event:
                parent.disconnect(self._key_release_event)        
            
        self.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.focused = False       
        self.emit('focus-changed', self, False)


    def set_view(self, view):
        """Set view to view or to empty view if parameter is None"""

        # if None is passed, use empty view
        if view == None:
            view = self.empty_view

        # do nothing if the view is already there
        if view == self._view:
            return
       
        # detach view from current parent
        view_parent = view.get_parent()
        if view_parent:
            view_parent.set_view(None)
        
        # switch which widget we are listening to
        if self._button_event:
            self._view.disconnect(self._button_event)

        self._button_event = view.connect("button-press-event",
                                          self.on_button_press_event)                                                                                            
                                          
        # remove old view, set new view
        self._view.hide()
        self.remove(self._view)
        self.add(view)
        view.show()
                
        self._view = view

    def get_view(self):
        """Returns current view."""
        return self._view

    def on_button_press_event(self, widget, event):
        if not self.focused:
            self.focus()
            
    def on_drag_data_received(self, widget, drag_context, x, y, selection,
                              info, timestamp):
        treestore, path = selection.tree_get_row_drag_data()
        obj = treestore.get_object_at_iter(treestore.get_iter(path))
        if isinstance(obj, plots.Plot):
            self.set_view(obj)
            self.focus()
        elif isinstance(obj, dataset.Dataset):
            self.set_view(plots.get_table(obj)[0])
            self.focus()
    
    def has_toolbar_dock(self):
        """ Does view have seperate space for placing a toolbar in?"""
        return False

    def get_toolbar_dock(self):
        return None

class WiewFrameWindowed(ViewFrame):
    def __init__(self,frames,dock):
        self._dock = dock
        ViewFrame.__init__(self,frames)
            
    def has_toolbar_dock(self):
        return True

    def get_toolbar_dock(self):
        return self._dock        
        
    
                                

class MainView (gtk.Notebook):
    def __init__(self):
        gtk.Notebook.__init__(self)
        self.set_show_tabs(False)
        self.set_show_border(False)        
        # Add a multiple pane view and a single pane view.
        self._view_frames = []
        self._views = ObjectTable(2, 2, lambda : ViewFrame(self._view_frames))
        self._small_views = gtk.Table(2, 2, True)
        self._small_views.set_col_spacings(4)
        self._small_views.set_row_spacings(4)
        self._large_view = ViewFrame(list())
         
        self.update_small_views()
        
        for vf in self._view_frames:
            vf.connect('focus-changed', self.on_view_focus_changed)
        self._large_view.connect('focus-changed', self.on_view_focus_changed)
        
        self.append_page(self._small_views)
        self.append_page(self._large_view)
        self.show()
        self.set_current_page(0)
    
    # fetch a viewframe from the smallviews
    def __getitem__(self, x, y):
        return self._views[x, y]

    # iter over a all viewframes in plot
    def __iter__(self):
        viewframes = [self._large_view]
        for view in self._views.as_list():  
            viewframes.append(view)
        return iter(viewframes)
        
    def update_small_views(self):
        for x in range(self._views.xsize):
            for y in range(self._views.ysize):
                child = self._views[x, y]
                self._small_views.attach(child, x, x+1, y, y+1)

    def get_active_small_view(self):
        for vf in self._view_frames:
            if vf.focused:
                return vf
        return None
    
    def goto_large(self):
        if self.get_current_page() == 1:
            return

        vf = self.get_active_small_view()
        view = vf.get_view()
        vf.set_view(None)
        
        # EmptyvPlot shouldn't be moved
        if not isinstance(view, plots.EmptyPlot):
            self._large_view.set_view(view)
        else:
            self._large_view.set_view(None)
        self.set_current_page(1)

    def goto_small(self):
        if self.get_current_page() == 0:
            return

        vf = self.get_active_small_view()
        view = self._large_view.get_view()
        self._large_view.set_view(None)
        
        # Emptyview shouldn't be moved
        if not isinstance(view, plots.EmptyPlot):
            vf.set_view(view)
        else:
            vf.set_view(None)
        self.set_current_page(0)

    def insert_view(self, view):
        if self.get_current_page() == 0:
            vf = self.get_active_small_view()
        else:
            vf = self._large_view
        
        vf.unfocus()
        vf.set_view(view)
        vf.focus()
        
    def show(self):
        for vf in self._view_frames:
            vf.show()
        self._small_views.show()
        gtk.Notebook.show(self)
        
    def on_view_focus_changed(self, widget, vf, focused):
        if focused:
            self.emit('view-changed', vf)    
    
    def clear_plots(self):
        self._large_view.set_view(None)
        for view in self._views.as_list():
            view.set_view(None)
        
    def remove_plot(self, plot):
        for viewframe in self:
            if viewframe.get_view() == plot:
                viewframe.set_view(None)

    def new_plot_window(self, plot):
        window = gtk.Window()
        window.set_title(plot.get_name())
        vbox = gtk.VBox()
        window.add(vbox)
        handle = gtk.HandleBox()
        vbox.pack_start(handle)                               
                               
        vf = WiewFrameWindowed(self._view_frames,handle)        
        vf.connect('focus-changed', self.on_view_focus_changed)        
        vf.set_size_request(300,400)
        vf.set_view(plot)

        vbox.pack_start(vf)

        window.connect("delete-event", self.on_plot_window_delete, vf,vbox)
        vf.focus()
        
        window.show()
        vbox.show_all()
        
    
    def on_plot_window_delete(self, widget, event, viewframe,vbox):
        viewframe.set_view(None)
        
        # save toolbar from deletion by removing it
        for child in vbox.children():
            vbox.remove(child)
                    
        return False


                       
# Create a view-changed signal that should be emitted every time
# the active view changes.
gobject.signal_new('view-changed', MainView, gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE, 
                   (gobject.TYPE_PYOBJECT,))

# Create focus-changed signal
gobject.signal_new('focus-changed', ViewFrame, gobject.SIGNAL_RUN_LAST,
                   gobject.TYPE_NONE, 
                   (gobject.TYPE_PYOBJECT, gobject.TYPE_BOOLEAN,))
