import gtk
import sys
import os
import inspect
import traceback
from system import logger, config


class Workflow:
    """Defines a workflow that contains a set of analysis stages.

    A Workflow is a set of analysis stages for a certain type of analysis.
    Each stage contains some possible operations to do accomplish that 
    task.
    """

    name = "Workflow"
    ident = None
    description = "Workflow Description"

    def __init__(self, app):
        self.stages = []
        self.stages_by_id = {}
        self.app = app

    def add_stage(self, stage):
        self.stages.append(stage)
        self.stages_by_id[stage.id] = stage

    def print_tree(self):
        print self.name
        for stage in self.stages:
            print '   %s' % stage.name
            for fun in stage.functions:
                print '        %s' % fun.name


class EmptyWorkflow(Workflow):
    name = 'Empty Workflow'

    def __init__(self, app):
        Workflow.__init__(self, None)
        
    
class Stage:
    """A stage is a part of the data analysis process.

    Each stage contains a set of functions that can be used to
    accomplish the task. A typical early stage is 'preprocessing', which
    can be done in several ways, each represented by a function.
    """

    def __init__(self, id, name):
        self.id = id
        self.name = name
        self.functions = []
        self.functions_by_id = {}

    def add_function(self, fun):
        self.functions.append(fun)
        self.functions_by_id[fun.id] = fun


class Function:
    """A Function object encapsulates a function on a data set.

    Each Function instance encapsulates some function that can be applied
    to one or more types of data.
    """

    def __init__(self, id, name):
        self.id = id
        self.name = name

    def run(self):
        pass


class WorkflowView(gtk.VBox):
    def __init__(self, app, wf):
        gtk.VBox.__init__(self)
        self.set_workflow(wf)
        self._app = app
        # target for mouse pointer changes
        self._gdk_window = None
        
    def set_workflow(self, workflow):
        """Show new workflow with functions in view."""
        # remove old workflow-widgets
        for c in self.get_children():
            self.remove(c)

        # Add stage in the process
        for stage in workflow.stages:
            exp = gtk.Expander(stage.name)
            btn_box = gtk.VBox()
            btn_box.show()
            exp.add(btn_box)

            # Add functions in each stage 
            for fun in stage.functions:
                btn = gtk.Button(fun.name)
                btn.connect('clicked',
                            lambda button, f=fun : self.run_function(f))
                btn.connect('enter', lambda button, f=fun : self.inform_about_function(f))
                btn.connect('leave', lambda button, f=fun : self._statusbar.pop(self._context_id))
                btn_box.add(btn)
                btn.show()
            
            exp.show()
            self.pack_start(exp, expand=False, fill=False)
   
    def set_statusbar(self, statusbar):
        """Gives the WorkflowView a reference to the statusbar."""
        self._statusbar = statusbar
        self._context_id = statusbar.get_context_id("wfv")

    def set_gdk_window(self, window):
        """Set gtk.gdk.Window instance used for mouse pointer changes."""
        self._gdk_window = window
 
    def inform_about_function(self, function):
        """Gives the user info about the input of a function.

        Called when the mouse pointer is over the button of a function.
        """
        args, varargs, varkw, defaults = inspect.getargspec(function.run)
        # if we have variable arguments in the end, we show them as
        # and ellipsis (...) argument
        if varargs:
            args.append("...")

        # remove 'self' and 'selection', we only show datasets needed
        args = args[1:]
        if args and args[0] == 'selection':
            args.remove('selection')

        status_message = "%s (%s)" % (function.name, ", ".join(args))
        self.message_id = self._statusbar.push(self._context_id, status_message)

        if function.__doc__:
            logger.information.set_markup(function.__doc__)
        else:
            logger.information.set_markup("No doc-string available.")
            
    def run_function(self, function):
        logger.log('debug', 'Starting function: %s' % function.name)
        project = self._app.project
        parent_data = project.get_current_data()

        args, varargs, varkw, defaults = inspect.getargspec(function.run)

        # first argument is 'self' and second should be the selection
        # and we don't care about those...
        args.remove('self')
        if args and args[0] == "selection":
            pass_selection = True
            args.remove('selection')
        else:
            if "selection" in args:
                logger.log('warning', "selection specified as not-first argument in function, will be treated as data.")
            pass_selection = False


        if defaults:
            min_args = len(args) - len(defaults)
        else:
            min_args = len(args)
            
        args_provided = len(parent_data)
        
        # do we have right number of arguments?
        if args_provided < min_args:
            # too few
            if min_args == 1:
                logger.log('warning', "Function requires at least one dataset selected.")
            else:
                logger.log('warning', "Function requires at least %d datasets selected." % min_args)
            return
        elif not varargs and args and args_provided > len(args):
            # too many (but required is more than 0)
            if len(args) == 1:
                logger.log('warning', "Function requires maximum one dataset selected.")
            else:
                logger.log('warning', "Function requires a maximum of %d datasets selected." % len(args))
            return

        if not args and not varargs:
            # we allow functions requiring no data to be run even if a
            # dataset is is selected
            data = []
        else:
            data = parent_data

        self._gdk_window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))

        try:
            try:
                if pass_selection:
                    # if the function has a 'selection' argument, we pass in
                    # the selection
                    new_data = function.run(project.get_selection(), *data)
                else:
                    new_data = function.run(*data)
            except:
                etype, value, tb = sys.exc_info()
                failed = "".join(traceback.format_exception(etype, value, tb, None))
                logger.log('error', '%s failed to run - %s: %s' % (function.name , etype, str(value).strip()))
                logger.log('debug', failed)
                return
        finally:
            self._gdk_window.set_cursor(None)
       
        if new_data != None:
            if min_args:
                project.add_data(parent_data, new_data, fun=function.name)
            else:
                project.add_data(parent_data, new_data, fun=function.name,
                                 place_at_bottom=True)
        
        logger.log('debug', 'Function ended: %s' % function.name)
