import gtk,  gobject, gtk.glade
import os , sys
import scipy , random, math
import Numeric, pylab, matplotlib
import webbrowser
import rpy

from scipy import randn, array, transpose, zeros, io
from StringIO import StringIO
from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
from matplotlib.collections import LineCollection
from matplotlib.ticker import NullFormatter
from matplotlib.ticker import NullLocator
from types import StringTypes
from string import capitalize
from exceptions import ValueError

from system import dataset, logger, plots, workflow, dialogs, filehandlers
from system.plots import ScatterPlot, ScatterPlotToolbar, NavToolbar, ScatterPlotBrowse
from system.octpy import matfile
from system.project import Notice
from workflows import affyworkflow


def r_path(path):
    from string import split
    split_path = path.split("\\")
    new_path = split_path.pop(0)
    for i in split_path:
        new_path += "/" + i
    return new_path 

def ensure_r_libraries(libs):
    """Ensure packages in list libs are installed locally for user.

    Returns True on successful install or already installed.  Will
    show GUI to user to confirm install if not already installed.
    """
    # check if we have a working rpy
    try:
        import rpy
        rpy.r
    except Exception, e:
        logger.log("error", "RPy not properly installed.")
        return False
     
    lib_paths = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r._libPaths)()
    if os.name=="posix":
        path = os.path.join(os.getenv("HOME"), ".scicraft-ng", "rlibs")
        rpy.r._libPaths([path] + lib_paths)
    else:
        # we assume directory is writable
        pass
    
    install_packages = []
    # do we have libraries?
    for lib in libs:
        try:
            rpy.r.library(lib)
            # yes
        except: 
            # no
            install_packages.append(lib)

    if install_packages == []:
        # all libraries are installed
        return True

    # for unixes we make sure path to install in is writable
    if os.name == "posix":
        lib_paths = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r._libPaths)()
        path = lib_paths[0]
        if not os.access(path, os.R_OK | os.W_OK | os.X_OK):
            if os.name == "posix":                      
                path = os.path.join(os.getenv("HOME"), ".scicraft-ng", "rlibs")
            else:
                path = os.path.join(os.getenv("USERPROFILE"), "SciCraft-NG",
                                    "rlibs")
            if not os.path.exists(path):
                os.mkdir(path)
            rpy.r._libPaths([path] + lib_paths)
    
    message = ""
    for i in install_packages:    
        message += "'" + i + "', "
    message = message[:-2]
    install_path = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r._libPaths)()[0]
    if not dialogs.get_yes_no("Installing R packages", """\
                                The following package(s) are missing: %s
                                Do you wish to install into '%s'?

                                If you do not install the package(s), the function will not be run.
                                """  % (message, install_path)):
        logger.log("warning",
                   "Not installing missing packages, aborting function.")
        return False
                              
    logger.log("notice", "Installing package(s): %s into '%s'" % (message,
                                                                  install_path))
    logger.log("notice", "Please be patient while installation is running.")
    silent_eval = rpy.with_mode(rpy.NO_CONVERSION, rpy.r)

    while(gtk.events_pending()):
        gtk.main_iteration()
    from system.config import r_CRAN_mirror 
    for package in install_packages:
        silent_eval("install.packages('%s',repos='%s', lib='%s')" % \
                    (package, r_CRAN_mirror,install_path))
    return True


class ChemometricsWorkflow(workflow.Workflow):
    name = 'Chemometrics Workflow'
    ident = 'chemometrics'
    description = 'Chemometrics workflow.' 
    def __init__(self, app):
        workflow.Workflow.__init__(self, app)

        test = workflow.Stage('testdata','Generate Testdata')
        test.add_function(CreateExampleDataset())        
        test.add_function(testdata())
        test.add_function(CreateAmp("amp","ampicillin_small.mat"))
        self.add_stage(test)
        
        preprocess = workflow.Stage('preprocess', 'Preprocessing')
        preprocess.add_function(AddNoise())
        self.add_stage(preprocess)
                        
        supervised = workflow.Stage('supervised', 'Supervised analysis')
        supervised.add_function(PLSFunction(self))
        supervised.add_function(PredictPLSFunction(self))
        supervised.add_function(KNN())        
        self.add_stage(supervised)

        unsupervised = workflow.Stage('unsupervised', 'Unsupervised analysis')
        unsupervised.add_function(affyworkflow.PCAFunction(self))
        unsupervised.add_function(HCAFunction(self))
        unsupervised.add_function(Kmeansc())
        unsupervised.add_function(kmeans_comp("kmeans_comp","Apply Kmeans"))
        unsupervised.add_function(SOM())
        self.add_stage(unsupervised)

        signal = workflow.Stage("signal_proc","Signal Processing")
        signal.add_function(CreateWavelet('waveletbasis','Wavelet Transform'))
        signal.add_function(CompareWavelets('compwav','Compare Basis Functions'))
        signal.add_function(TimeScalePlotFunction('tsc',"Time Scale Plot"))
        signal.add_function(InvTransform('invertwavelet',"Invert Wavelet Transform"))
        signal.add_function(Fourier(self,'fourier','Fourier Transform'))
        signal.add_function(InvFourier('inv_fourier','Inv. Fourier Transform'))
        self.add_stage(signal)
        
        help = workflow.Stage('help', 'Help')        
        help.add_function(ShowExampleFunction())
        help.add_function(ShowHelpFunction())
        self.add_stage(help)


class ShowHelpFunction(workflow.Function):
    """ Shows documentation for this workflow from the internet."""
    
    def __init__(self):
        workflow.Function.__init__(self, 'show_help', 'Workflow information')

    def run(self):
        webbrowser.open_new('https://dev.scicraft.org/trac/wiki/ChemometricsWorkflow')

class ShowExampleFunction(workflow.Function):
    """ Shows example for this workflow from the internet."""
    
    def __init__(self):
        workflow.Function.__init__(self, 'show_help', 'Show Example Description')

    def run(self):
        webbrowser.open_new('https://dev.scicraft.org/trac/wiki/ChemometricsWorkflowExample')
                                                                                                                                                
class Kmeansc(workflow.Function):
    """ <b>Kmeansc</b>: Perform k-means clustering on a data matrix.
    
Input:
X: A numeric matrix of data

Outputs:
Cluster: A vector of integers indicating the cluster to which each point is allocated.
Centers: A matrix of clustercentre co-ordinates.
Combined X : Combined Rawdata and Centers into one matrix.
PCA Scores and Loadings: Scores and Loadings from PCA run on combined centers and rawdata
PCA Cluster Plott: Score plot, showing groups by colour/shape and centers/objects by size.
    """
    def __init__(self):
        workflow.Function.__init__(self, 'kmeansc','Kmeans')
            
    def run(self,data):
        #K-Means Clustering: Perform k-means clustering on a data matrix.
        import rpy
        rpy.r.assign("data",data.asarray())
                   
        WORKFLOWDIR = os.path.split(__file__)[0]
        xml = gtk.glade.XML(os.path.join(WORKFLOWDIR, "classification.glade"),'kmeans_dialog')
        dialog = xml.get_widget('kmeans_dialog')
        spin1  = xml.get_widget('spinbutton1')
        spin2  = xml.get_widget('spinbutton2')
        try:
            response = dialog.run()
        finally:
            dialog.destroy()        
        
        if (response != gtk.RESPONSE_OK):            
            return
        
        # compute KMeans and run PCA                                   
        rpy.r.assign("iters", spin2.get_value())
        rpy.r.assign("centers",spin1.get_value()) 
        rpy.r("result <- kmeans(data,centers,iters)")
        rpy.r("pca <- prcomp(rbind(data,result$centers))")
        scores = rpy.r("pca$x")
        loadings = rpy.r("pca$rotation")
        
        # create all identifiers needed
        variables = data[1],data.get_identifiers(data[1])
        samples   = data[0],data.get_identifiers(data[0])
        centers = ("Centers",["Center "+str(num+1) for num in range(spin1.get_value())])
        samples_center = (samples[0], samples[1]+ centers[1])        
        pca = ("PCA",["PCA"+str(num+1) for num in range(scores.shape[1])])
        nones = (None,None)

        # plot over centers             
        centers = dataset.Dataset(rpy.r("result$centers"),[centers,variables],name="centers")
        clusters = dataset.Dataset(scipy.transpose(scipy.array(rpy.r("result$cluster"))),[("Cluster",["Cluster"]),samples],name="clusters")
        
        # make new dataset with new dim names                        
        pcascores = dataset.Dataset(scores,[samples_center,pca],name="PCA Scores")
        pcaloadings = dataset.Dataset(loadings,[variables,pca],name="PCA Loadings")


                               
        colour_plot = colourPlot(pcascores, clusters, samples[1],centers[1])        
                                 
        return [centers, clusters, pcascores,pcaloadings,colour_plot, ]

class kmeans_comp(workflow.Function):
    """<b>Apply Kmeans</b>: Group data according to found centers. Or, more specifically, 
    for each sample in X, find the sample in centers with lowest Ecludian distance.
    
Accepts:
X - Data to be classified
centers - The centers matrix

Outputs:
Applied Clusters 
"""      
     
    def run(self, X, centers):
        if X.shape[1] != centers.shape[1]:
            logger.log("warning","Second argument didn't have %s samples. Aborting" % (centers.shape[1],))
            return       
            
        results = []            
        for sample in xrange(X.shape[0]):        
            c = X.asarray()[sample]
            dist = centers.asarray() - c                        
            dist = dist * dist
            sums = list(sum(Numeric.transpose(dist)))
            results.append(sums.index(min(sums))+1)
                    
        return [dataset.Dataset(results,[(X[0],X.get_identifiers(X[0]))],name="Applied Clusters")]                   

class colourPlot(ScatterPlotBrowse):

    def __init__(self, data, membership, sample_ids, center_ids):
        self.membership = Numeric.ravel(membership.asarray())
        self.membership_types = Numeric.array(list(set(self.membership)))
        
        self.center_ids = center_ids
        
        dim = data.get_dim_names()[1]
        id1, id2 = data.get_identifiers(dim)[0:2]
        
        self._use_legend = False
                        
        ScatterPlotBrowse.__init__(self,data,data,(dim,id1),(dim,id2),name="Score Plot with Clustering")

    def draw_plot(self, data_to_plot1, data_to_plot2):
                      
        symbols =['o','s','d','p','^','v','+','1','2','3','4','h','H','.',','] 
        colors  =['b','g','c','m','y','k','w']

        num_clusters = len(self.membership_types)
                
        for num,cluster in enumerate(self.membership_types):
            indices = Numeric.nonzero(self.membership == cluster)
            data1 = Numeric.take(data_to_plot1, indices)
            data2 = Numeric.take(data_to_plot2, indices)
            s = symbols[int(num)/7]
            c = colors[num%7]
                        
            self._ax.plot(data1,data2,s+c,label="Objects " + str(num))

            # last num_clusters are 
            indice = len(data_to_plot1) - num_clusters + num                                                        
            self._ax.plot([data_to_plot1[indice]],[data_to_plot2[indice]],s+c,label="Center " + str(num),markersize=15)        
            
        if self._use_legend:            
            self._ax.legend()

    def _init_toolbar(self):
        ScatterPlotBrowse._init_toolbar(self)
        tbutton_legend = self._toolbar.add_legend_button() 
        tbutton_legend.connect('clicked', self.switch_legend)

    def switch_legend(self,*ignored):
        self._use_legend = not self._use_legend
        self.update()    
                          
        
class testdata(workflow.Function):
    """<b>Create Example Dataset</b>: WheatSpectra """ 
    def __init__(self):
        workflow.Function.__init__(self, 'wheat_spectra.mat', 'WheatSpectra.mat')

    def run(self):
        
        from system.octpy import matfile
        #Get wheat_spectra data set
        WORKFLOWDIR = os.path.split(__file__)[0]
        path = WORKFLOWDIR + os.sep + os.sep.join(["datasets","wheat.mat"])
        
        a = filehandlers.MatFileLoadFunction().run(str(path))
        return a                
        


class CreateExampleDataset(workflow.Function):
    """<b>Create Example Dataset</b>: NIR spectra and density measurements of PET yarns.
    
A training set consisting of 21 NIR spectra of PET yarns, measured at 268 wavelengths, and 21
corresponding densities. A test set of 7 samples is also provided. Many thanks to Erik Swierenga. 
    
Outputs:
X, X-Testset
Y, Y-Testset""" 
    def __init__(self):
        workflow.Function.__init__(self, 'create_example', 'PET yarn')    
    
    def run(self):
        import rpy
                
        if not ensure_r_libraries(["pls"]):
            return None

        rpy.r("library(pls)") 

      	try:
    	    rpy.r("data(NIR)")
            x = rpy.r.NIR['X']
            y = rpy.r.NIR['y']
    	    t = rpy.r.NIR['train']
    	except Exception:
    	    # renamed as of R library pls 2.0-0
    	    rpy.r("data(yarn)")
    	    x = rpy.r.yarn['NIR']
    	    y = rpy.r.yarn['density']
    	    t = rpy.r.yarn['train']

        test_indices = scipy.array([i for i, test in enumerate(t) if not test])
        cali_indices  = scipy.array([i for i, test in enumerate(t) if test])

        test_data_x = Numeric.take(x,test_indices)
        cali_data_x = Numeric.take(x,cali_indices)
        test_data_y = Numeric.take(Numeric.reshape(y,(28,1)),test_indices-1)
        cali_data_y = Numeric.take(Numeric.reshape(y,(28,1)),cali_indices-1)

        dataset_x_cali = dataset.Dataset(cali_data_x,[("Samples",map(str,cali_indices)),("Wavelengths",None)],name="X")
        dataset_y_cali = dataset.Dataset(cali_data_y,[("Samples",map(str,cali_indices)),("Concentrations",None)],name="Y")
        dataset_x_test = dataset.Dataset(test_data_x,[("Samples",map(str,test_indices)),("Wavelengths",None)],name="X - Testset")
        dataset_y_test = dataset.Dataset(test_data_y,[("Samples",map(str,test_indices)),("Concentrations",None)],name="Y - Testset")                                

        return [dataset_x_cali,dataset_y_cali,dataset_x_test,dataset_y_test]


class CreateAmp(workflow.Function):
    """<b>Create Example Dataset</b>: Octave """ 

    def run(self):
        
        from system.octpy import matfile
        #Get wheat_spectra data set
        WORKFLOWDIR = os.path.split(__file__)[0]
        path = WORKFLOWDIR + os.sep + os.sep.join(["datasets","ampicillin_small.mat"])
        
        aaaa = filehandlers.MatFileLoadFunction().run(str(path))
        
        aaaa[3].change_dim_name(aaaa[3][0],aaaa[0][0])
        aaaa[2].change_dim_name(aaaa[2][0],aaaa[1][0])
        return aaaa
       

        
        
       
class AddNoise(workflow.Function):
    """ <b>Add Noise</b>: Adds noise to a data set

Input:
    X: A numeric matrix of data

Output:
    Y: A numeric matrix of the input data with gaussian noise with selected standard diviation """
    def __init__(self):
        workflow.Function.__init__(self, 'noise', 'Add Noise')

    def run(self, data):
        #n = scipy.stats.norm()
        dialog = gtk.Dialog(title="Standard Deviation",buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                      gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
                      
        hbox = gtk.HBox()
        infoframe = gtk.Frame(label=" " + data.get_name())
        selframe = gtk.Frame(label=" Normal Distributed Noise: ")
        label = gtk.Label("Standard deviation: ")
        infoframe.set_border_width(5)
        selframe.set_border_width(5)
                
        var = scipy.std(scipy.ravel(data.asarray()))
        
        table = gtk.Table()
        table.attach(gtk.Label("Current standard deviation: "),0,1,0,1)
        table.attach(gtk.Label(str(var)[0:5]),1,2,0,1)
                 
        adjust = gtk.Adjustment()
        adjust.value = var/16      # MAGIC NUMBER ALERT HERE! 
        spin_button = gtk.SpinButton(adjust, climb_rate=0.1, digits=2)
        spin_button.set_range(-100000000000, 100000000000)

        spin_button.set_increments(0.1, 0.1)
        infoframe.add(table)
        hbox.pack_start(label)
        hbox.pack_start(spin_button)
        selframe.add(hbox)
        dialog.vbox.pack_start(infoframe)
        dialog.vbox.pack_start(selframe)

        dialog.show_all()
        
        try:
            if dialog.run()!= gtk.RESPONSE_ACCEPT:
                return
        finally:
            dialog.destroy()
        noise = ( scipy.randn(*data.shape) * spin_button.get_value())             
        newdata = data.asarray().copy() + noise
         
        ids = [(dim, data.get_identifiers(dim))
                for dim in data.get_dim_names()]
        data_wrapped = dataset.Dataset(newdata, name = data.get_name(), identifiers = ids)
        noise = dataset.Dataset(noise,name="Noise", identifiers=ids)
        return [data_wrapped,noise, plots.IndexPlot(data_wrapped), plots.IndexPlot(noise)]

class PLSFunction(workflow.Function):
    """<b>PLS</b>:  Perform partial least squares regression (PLSR)
    
Accepts: 
X: XxN , terms
Y: XxM , response variables

Outputs:
Loadings, Scores
Various Plots"""

    def __init__(self, wf):
        workflow.Function.__init__(self, 'pls', 'PLS')
        self._workflow = wf
        
    #To select the two datasets, press Ctrl and click on the datasets,
    #the order must be: Ctrl + <click on X input> + <click on Y input>
    def run(self,dataX,dataY):
        import rpy
       
        if not ensure_r_libraries(["pls","MASS"]):
            return None        

        dimX_1, dimX_2 = dataX.get_dim_names()
        dimX_1_ids = dataX.get_identifiers(dimX_1)
        dimX_2_ids = dataX.get_identifiers(dimX_2)
            
        dimY_1, dimY_2 = dataY.get_dim_names()
        dimY_1_ids = dataY.get_identifiers(dimY_1)
        dimY_2_ids = dataY.get_identifiers(dimY_2)
        ydims = ((dimY_1,dimY_1_ids),(dimY_2,dimY_2_ids)) # god, clean this up!       

        silent_eval = rpy.with_mode(rpy.NO_CONVERSION, rpy.r)
        silent_set = rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)
        vector_eval = rpy.with_mode(rpy.VECTOR_CONVERSION, rpy.r)

        #Load the pls script
        WORKFLOWDIR = os.path.split(__file__)[0]
        r_file = r_path(WORKFLOWDIR + "/ChemometricsFunctions/PLS.r")
        silent_eval("source('%s')" % r_file)
       
        silent_set("Y",dataY.asarray())
        silent_set("X",dataX.asarray())
        #If Y data only has one row we have to transpose:     

        xml = gtk.glade.XML(os.path.join(WORKFLOWDIR, "plssettings.glade"),
                            'dialog')
        window = xml.get_widget('dialog')
                
        self.num_comp = xml.get_widget('NumOfComp')
        self.val_combo = xml.get_widget('Val')
        self.val_combo.set_active(2)
        self.val_seg = xml.get_widget('ValSeg')
        self.jk_combo = xml.get_widget('JK')
        self.jk_combo.set_active(0)
        self.f_comp_combo = xml.get_widget('FComp')
        self.f_comp_combo.set_active(0)
        self.fdr_combo = xml.get_widget('FDR')
        self.fdr_combo.set_active(0)
        self.mod_t_test = xml.get_widget('ModTtest')
        self.permutations = xml.get_widget('perm')
        self.meth_combo = xml.get_widget('method')
        self.meth_combo.set_active(2)
        self.fdr_combo.connect('changed', self._fdr_combo_changed)
        self.jk_combo.connect('changed', self._jk_combo_changed)
        self.val_combo.connect('changed', self._val_combo_changed)
        res = run = window.run()

        if res==gtk.RESPONSE_DELETE_EVENT:
            window.destroy()
            return None
        
        num_of_comp = self.num_comp.get_value_as_int()
        val = self.val_combo.get_active_text()
        val_seg = self.val_seg.get_value_as_int()
        jk = self.jk_combo.get_active_text()
        force_components = self.f_comp_combo.get_active_text()
        fdr = self.fdr_combo.get_active_text()
        mod_t_test = self.mod_t_test.get_value_as_int()
        perm = self.permutations.get_value_as_int()
        meth = self.meth_combo.get_active_text()     

        if val == "none":
            val_seg = 0
            jk = "none"
            force_components = "Use NumberOfComponents"
            fdr = "none"
            mod_t_test = 0
            perm = 0
        elif jk == "none":
            fdr = "none"
            mod_t_test = 0
            perm = 0
        elif fdr == "none":
            perm = 0
 
        window.destroy()
       
        silent_set("NumberOfComponents",num_of_comp)
        silent_set("Validation",val)
        silent_set("ValidationSegments",val_seg)
        silent_set("JackKnife",jk)
        silent_set("ForceComponents",force_components)
        silent_set("FDR",fdr)
        silent_set("ModifiedTtest",mod_t_test)
        silent_set("permutations",perm)
        silent_set("method",meth)       
        silent_eval("result = PLS(X, Y, NumberOfComponents, Validation, ValidationSegments, JackKnife, ForceComponents, FDR, ModifiedTtest, permutations, method)")
     
        comp_dim = "comp"
        loadings = dataset.Dataset(rpy.r("result$plsModel$loadings"),
                                   ((dimX_2,dimX_2_ids), (comp_dim,None)),
                                   name="Loadings")
        comp_ids = loadings.get_identifiers(comp_dim)                                   
        
        

        scores = dataset.Dataset(rpy.r("result$plsModel$scores"),
                                 ((dimX_1,dimX_1_ids),(comp_dim,comp_ids)),
                                 name="Scores")                                                                  
                            
        residuals = dataset.Dataset(rpy.r("result$plsModel$residuals"),
                                    ((dimX_1,dimX_1_ids),(dimY_2,dimY_2_ids),
                                     (comp_dim,comp_ids)),
                                    name="Residuals")        
                                   
        y_pred_opt = dataset.Dataset(rpy.r("result$diagnostics$Ypred"),
                                   ((dimX_1,dimX_1_ids),
                                    (dimY_2,dimY_2_ids)),name="Predicted Y")   
        
        #plot leverage:
        lev_plot = LeveragePlot(scores,loadings)
                                                                                                                                                          
        #regression coefficients:
        coeff = []
        for ncomp in range(rpy.r("result$plsModel$ncomp")):
            coeff.append(rpy.r("coef(result$plsModel, %s, intercept=TRUE)" % (ncomp+1,)))
        coeff = Numeric.reshape(Numeric.transpose(coeff),(-1,dataY.shape[1],ncomp+1))
        coefficients = dataset.Dataset(coeff,name="Coefficients")                                                                               
        # Explained variance for x :               
        xvar = dataset.Dataset(rpy.r("as.matrix(result$plsModel$Xvar)"),
                            ((comp_dim, comp_ids),(None, None)),                               
                               name="Explained Variance for X")                                                               
                        
        # Explained variance for Y:
        
        #Explained Yvar on the calibration set:        
        silent_eval('yvar_cal <- R2(result$plsModel,"train")')
        Yvar_cal_dim =  rpy.r("yvar_cal$val").shape
        Yvar_calibration = dataset.Dataset(100-rpy.r("yvar_cal$val*100")[0,:,1:],((dimY_2,dimY_2_ids),(comp_dim,comp_ids)),name= "Explained Variance_Y_cal")
        
        #Explained Yvar on validation       
        msep = rpy.r("result$plsModel$validation$MSEP")        
        msep0 = rpy.r("result$plsModel$validation$MSEP0")
        names = rpy.r("names(result$plsModel$validation$MSEP0)")
        if msep and msep0:
            msep0 = Numeric.array([msep0[key] for key in names])                
            fisk = Numeric.zeros((msep.shape[0],msep.shape[1]+1),msep.typecode())
            for num, x in enumerate(msep0):
                fisk[num,1:] = (1 - msep[num] / x)

            fisk = fisk * 100                
            Explained_y_variance = dataset.Dataset(100-fisk[:,1:],((dimY_2,dimY_2_ids),(comp_dim,comp_ids)),name="Explained Variance_Y")
    
            # Create Explained Variance Plot for Y 
            dim1 = Explained_y_variance.get_dim_names()[0]
            id1 = Explained_y_variance.get_identifiers(dim1)[0]
            dim2 = Yvar_calibration.get_dim_names()[0]
            id2 = Yvar_calibration.get_identifiers(dim2)[0]                                
            yvariance_plot = PlotPLSExplainedVarY(Explained_y_variance,Yvar_calibration)  
        else:
            dim = Yvar_calibration.get_dim_names()[0]
            id  = Yvar_calibration.get_identifiers(dim)[0]            
            yvariance_plot = ExplainedVariancePlot(Yvar_calibration,dim,id,name="Residual Variance Y",xaxis_label="Residual Variance Calibration")
                                                            
        textlist = {}
        for num, id in enumerate(dimY_2_ids):                                     
            #plot predicted values vs. the observed values, we add some extra information into the plot: correlation, slope and offset.
            silent_eval("data <- as.data.frame(cbind(Y[,%s],result$diagnostics$Ypred[,%s]))" % (num+1,num+1))	
            silent_eval('colnames(data) <- c("Y","Y_pred")')
            silent_eval("lm_reg <- lm(Y~ Y_pred,data)")
    
            correlation = rpy.r("r <- round(cor(Y[,%s],result$diagnostics$Ypred[,%s]),4)" %(num+1,num+1)) 
            correlation_squared = rpy.r("round(r*r,4)")
            offset = rpy.r("round(lm_reg$coef[1],4)").popitem()[1]
            slope = rpy.r("round(lm_reg$coef[2],4)").popitem()[1]
        
            textlist[id] = ([("R: ", correlation),        
                             ("R^2: ", correlation_squared),
                             ("offset: ", offset),
                             ("slope: ", slope)])
                                                 
        y_pred_dim2 = y_pred_opt.get_dim_names()[1]
        
        # ValX with constant dim                
        silent_eval("Xbound = cbind(rep(1,dim(X)[1]),as.matrix(X))")                
        ValX = dataset.Dataset(rpy.r("Xbound"), [(dimX_1, dataX.get_identifiers(dimX_1)),
                                          (dimX_2, ["Constant"] + dataX.get_identifiers(dimX_2))],
                               dataX.get_name())
         
        # removed due to not making sense at all.
        #predicted_vs_observed = PredPLSScatterPlotWithUpDownLegend(coefficients,ValX,dataY)
                                                                                  
        #plot explained variance:       
        explained_variance = []
        temp = xvar.asarray()
        for i in range(1, len(temp) + 1):
            explained_variance.append(sum(temp[0:i])[0])

        explained_variance = dataset.Dataset(explained_variance,[(comp_dim, comp_ids)])
        explained_variance._array = 100 - explained_variance._array 
        
        dim = explained_variance.get_dim_names()[0]
        id  = explained_variance.get_identifiers(dim)[0]
        xvar_plot =  ExplainedVariancePlot(explained_variance,dim,id)
        
        textlist = {}
            
        for num, id in enumerate(dimY_2_ids):                                     
            #plot predicted values vs. the observed values, we add some extra information into the plot: correlation, slope and offset.
            silent_eval("data <- as.data.frame(cbind(Y[,%s],result$diagnostics$Ypred[,%s]))" % (num+1,num+1))   
            silent_eval('colnames(data) <- c("Y","Y_pred")')
            silent_eval("lm_reg <- lm(Y~ Y_pred,data)")  
            correlation = rpy.r("cor <- round(cor(Y[,%s],result$diagnostics$Ypred[,%s]),4)" %(num+1,num+1))
            cor_squared = rpy.r("round(cor*cor,4)") 
            offset = rpy.r("round(lm_reg$coef[1],4)").popitem()[1]
            slope = rpy.r("round(lm_reg$coef[2],4)").popitem()[1]
        
            textlist[id] = ([("r: ", rpy.r("cor")),
                             ("r^2: ", cor_squared),        
                             ("offset: ", offset),
                             ("slope: ", slope)])                    
            y_pred_dim2 = y_pred_opt.get_dim_names()[1]
            
        datasets = []
        shape = rpy.r("dim(result$plsModel$validation$pred)")
        if shape:
            for ii in range(1,1+shape[2]):
                datasets.append(dataset.Dataset(rpy.r("as.matrix(result$plsModel$validation$pred[,,%s])" % (ii,)),ydims))                                                
            validated = ValidatedScatterPlotWithUpDownLegend(datasets,dataY)                         
        else:   
            validated = None
            
        yloadings = dataset.Dataset(rpy.r("result$plsModel$Yloadings"),[(dimY_2,dimY_2_ids),(comp_dim,comp_ids)],name="YLoadings")
        	
        #Loadings and Score plots:
        load_dim = loadings.get_dim_names()[1]
        load_id1 , load_id2 = loadings.get_identifiers(load_dim)[0:2]
        loading_plot = PLSLoadingsPlot(yloadings,loadings,loadings,
                                        (load_dim, load_id1), 
                                        (load_dim, load_id2),
                                        name="Loadings",
                                        xaxis_label="PLS component: %s ",
                                        yaxis_label="PLS component: %s ")
        
        score_dim = scores.get_dim_names()[1]
        score_id1, score_id2 = scores.get_identifiers(score_dim)[0:2]
        score_plot = ScatterPlotBrowse(scores, scores, (score_dim, score_id1),
                                    (score_dim, score_id2),name="Scores",
                                    xaxis_label="PLS component nr: %s",
                                    yaxis_label="PLS component nr: %s")
                                    
        influenceplot = InfluencePlot(scores,loadings,dataX,dataset.Dataset(Numeric.ravel(rpy.r("as.matrix(result$plsModel$Xmeans)"))))

        biplot = Biplot(scores,loadings,load_dim,score_dim,score_id1,score_id2)                                                                        
        
        Q = dataset.Dataset(rpy.r("result$plsModel$Yloadings[]"),((dimY_2,dimY_2_ids),(comp_dim,comp_ids)), name="Loadings matrix for Y")
        if rpy.r("result$plsModel$loading.weights"):        
            W = dataset.Dataset(rpy.r("result$plsModel$loading.weights"),((dimX_2,dimX_2_ids),(comp_dim,comp_ids)), name="covariance matrix for X,Y")                                  
        else:
            W = Notice("Covariance Matrix","Covariance Matrix not available after simPLS")
                                                                                                                   
        #clean up:
        rpy.r("rm(list=ls())")             
                       
        ret = [loadings,scores,Q,W,
        residuals,xvar, coefficients, loading_plot, score_plot, influenceplot,biplot, xvar_plot, 
        yvariance_plot,lev_plot, validated]
        while(None in ret):
            ret.remove(None)           
            
        return ret            

    def _fdr_combo_changed(self, *args):
        value = self.fdr_combo.get_active_text()
        if value == "none":
            self.permutations.set_sensitive(False)
        else:
            self.permutations.set_sensitive(True)

    def _val_combo_changed(self, *args):
        sens = True
        if self.val_combo.get_active_text() == "none":
            sens = False
        self.val_seg.set_sensitive(sens)
        self.jk_combo.set_sensitive(sens)
        self.f_comp_combo.set_sensitive(sens)
        if self.jk_combo.get_active_text() != "none":
            self.fdr_combo.set_sensitive(sens)
            self.mod_t_test.set_sensitive(sens)
        if self.fdr_combo.get_active_text() != "none":
            self.permutations.set_sensitive(sens)
        
    def _jk_combo_changed(self, *args):
        sens = True
        if self.jk_combo.get_active_text() == "none":
            sens = False
        self.fdr_combo.set_sensitive(sens)
        self.mod_t_test.set_sensitive(sens)
        if self.fdr_combo.get_active_text() != "none":
            self.permutations.set_sensitive(sens)
         


class PredictPLSFunction(workflow.Function):
    """Predict PLS: Estimate new responsevariables from coefficients matrix. Please note the order of the arguments!
    
Accepts:
coefficients (XxYxN),X (XxN) , Y (XxY)    
Obviously, the last Y/responsevariabel arguments is optional, and only needed for verification plot. 

Outputs:
Predicted Y
Predicted vs True Plot, if response variable was provided."""    
    
    def __init__(self,wf):
        workflow.Function.__init__(self,'predpls', 'PredictPLS')
        self._workflow = wf

    def run(self, coeffs, X, Y=None):
        import rpy
        
        # adding 1's for intercept
        xc = Numeric.ones((X.shape[0],X.shape[1]+1),X.asarray().typecode())
        xc[:,1:] = X.asarray()
        
        numberOfComp = coeffs.shape[2]
                                                          
        # -- POPUP BOX --                                               
        WORKFLOWDIR = os.path.join(os.path.dirname(sys.modules['system'].__file__),
            "..", "workflows")
        xml = gtk.glade.XML(os.path.join(WORKFLOWDIR, "predplssettings.glade"), 'dialog1')
        window = xml.get_widget('dialog1')
        self.spinbutton = xml.get_widget('spinbutton1')        
        self.spinbutton.set_range(1, numberOfComp)
        self.spinbutton.set_value(numberOfComp)
        
        res = window.run()
        if res != gtk.RESPONSE_OK:
           window.destroy()
           return None
                                   
        numberOfComp = self.spinbutton.get_value_as_int()                                                
        window.destroy()
                
        pred_Y = Numeric.matrixmultiply(xc,coeffs.asarray()[:,:,numberOfComp-1])
                                             
        if not Y:                                       
            pred_Y = dataset.Dataset(pred_Y,
                                ((X[0],X.get_identifiers(X[0])),
                                (None,None)),name="Predicted Y")                                
            return [pred_Y]
                                                                                               
        else:
        
            pred_Y = dataset.Dataset(pred_Y,
                                ((X[0],X.get_identifiers(X[0])),
                                (Y[1],Y.get_identifiers(Y[1]))),name="Predicted Y")                    
                                                                                                                                            
            PredTrue = PredPLSScatterPlotWithUpDownLegend(coeffs,X,Y,numberOfComp)
            
            PredTrue.set_title("Predicted vs True")  
            PredTrue._xaxis_label = "True"
            PredTrue._yaxis_label = "Predicted" 
            return [pred_Y,PredTrue]                                                           
                           




                

            
class HCAFunction(workflow.Function):
    """<b>Hierchial Cluster Analysis</b>: performs a cluster analysis.

Accepts:
I really don't know. semi-magical distance-matrix is created under some circumstances. 

Outputs:
order, merge, height: datasets used to create dendrogram
Dendrogram"""
    def __init__(self,wf):
        workflow.Function.__init__(self, 'hca','HCA')
        self._workflow = wf 
        
    def run(self,data):
        import rpy
        dim_2, dim_1 = data.get_dim_names()
        ids_2, ids_1 = map(data.get_identifiers, [dim_2, dim_1])
        #Set different abbreviations:
        silent_eval = rpy.with_mode(rpy.NO_CONVERSION, rpy.r)
        silent_set = rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)
        
        silent_set("matrix", data.asarray())
        
        #Get access to the R Function written by Endre:
        WORKFLOWDIR = os.path.split(__file__)[0]
        r_file = r_path(WORKFLOWDIR + "/ChemometricsFunctions/HierarchicalClusterAnalysis.r")
        silent_eval("source('%s')" % r_file)


        xml = gtk.glade.XML(os.path.join(WORKFLOWDIR, "hcasettings.glade"), 'dialog')
        window = xml.get_widget('dialog')
        
        xml.get_widget('metric').set_active(1)
        xml.get_widget('method').set_active(0)
        xml.get_widget('direction').set_active(0)
        xml.get_widget('exponent').set_value(2)
             
        run = window.run()
        if run==gtk.RESPONSE_DELETE_EVENT:
            # Window was closed/deleted. Don't take any other actions.
            return

        metric = xml.get_widget('metric').get_active_text()
        method = xml.get_widget('method').get_active_text()
        rows = xml.get_widget('direction').get_active_text()
        p = xml.get_widget('exponent').get_value_as_int()

        window.destroy()
            
        #Assign input parameters as R Objects
        silent_set("metric",metric)
        silent_set("method",method)
        silent_set("rows",rows)
        silent_set("p",p)

        #Do HCA:        
        current_dir = os.getcwd()
        try:
            os.chdir(WORKFLOWDIR)
            silent_eval("result = HierarchicalClusterAnalysis(matrix,metric,method,rows,p)")
        finally:
            os.chdir(current_dir)
        
        order = dataset.Dataset(rpy.r("as.matrix(result$Clustering$order)"),(("rows",rpy.r("as.character(1:length(result$Clustering$order))")),("cols","1")),name="order")
        height = dataset.Dataset(rpy.r("as.matrix(result$Clustering$height)"),(("rows",rpy.r("as.character(1:length(result$Clustering$height))")),("cols","1")),name="height")
        mergeids = (("rows",rpy.r("as.character(1:nrow(result$Clustering$merge))")),("cols",rpy.r("as.character(1:ncol(result$Clustering$merge))")))
        merge = dataset.Dataset(rpy.r("result$Clustering$merge"),mergeids,name="merge")              
 
        order_names  = rpy.r("result$Clustering$order.lab")
        
        new_height = []
        for n in height.asarray():
            new_height.append(n[0])
    
        #clean up:
        rpy.r("rm(list=ls())")
        
        if method == "divisive":
            method = capitalize(method) + " clustering "
        else:
            method = capitalize(method) + " linkage "            

        xlabel = "Method: " + method + " Metric: " + capitalize(metric)
            
        plot = DendrogramPlot(merge.asarray(), new_height, order_names, xlabel, data.get_dim_names()[0],data.get_identifiers(data.get_dim_names()[0]))
                 
        return [order,height,merge, plot]

class DendrogramPlot(plots.Plot):
    def __init__(self, merge, height, order, xlabel, dim_name, ids, name="Dendrogram"):
        """Create a new dendrogram from R tree data """
        
        plots.Plot.__init__(self, name)
        self._dim_name = dim_name   # for selections
        self._ids = ids
        
        self._fig = pylab.Figure(figsize=(5,4), dpi=72)
        self._canvas = plots.FigureCanvas(self._fig)
        self._merge = merge
        self._height = height
        self._order = order
        self._sensitives = []
        self._selected = []
        
        # MAGIC CONSTANTS
        self._shown_branch_weight = 5   
        self._hidden_branch_weight = 3  
        self._clickable_height = 0.3   # Magic constant defining height of clickable area
        
        self._hidden_junctions = []
        self._x_weight =  [] 
        self._x_pos = []
        self._update_x_pos()        
    
        self._xlabel_string = xlabel
        self.ylim = (0,0)
        self._ax = self._fig.add_subplot(111)             
        
        # variable to contain a sequence of lines, improves the speed of drawing the plot
        self._lines = []
        self._canvas.show()    
        self._init_toolbar()
        self._init_popupmenu()        
        self.add(self._canvas)         
        self.update()
        self._set_axes_properties()      
        self.connect("button_press_event", self._click)        
                                       
    def get_toolbar(self):
        return self._toolbar
        
    def get_popupmenu(self):
        return self._popupmenu
               
    def _init_popupmenu(self):
        self._popupmenu = plots.CustomPopupMenu(self)
        self._popupmenu.init_from_toolbar(self.get_toolbar())

    def _init_toolbar(self):
        self._toolbar = plots.NavToolbar(self._canvas, None)
        self._toolbar.set_select_callback(self.rectangle_select_callback)                                                

    def _set_axes_properties(self):
        self._ax.set_ylabel("Height")
        self._ax.set_xlabel(self._xlabel_string)
        self._ax.set_title("Hierarchical cluster analysis")
        self._ax.xaxis.set_major_formatter(NullFormatter())
        self._ax.xaxis.set_minor_formatter(NullFormatter())
        self._ax.xaxis.set_major_locator(NullLocator())
        self._ax.xaxis.set_minor_locator(NullLocator())                      
        self._ax.set_ylim(self.ylim)
        self._ax.set_xlim((-2,self._x_pos[-1]*1.05))
        
    def update(self):
        def get_x_pos_for_bottom(child):
            return self._x_pos[self._order.index(str(abs(child)))]
            
        def get_x_pos(child):
            if child<0:
                return get_x_pos_for_bottom(child) 
            else:
                return x_pos[child-1]
                 
        # find x_positions for all junctions
        x_pos = []
        for child1, child2 in self._merge:
            x_pos.append((get_x_pos(child1) +get_x_pos(child2)) / 2.0)

        start_pos = [0] * len(self._merge)         
        child_pos = []
        junctions = [len(self._merge)-1]        
        self._sensitives = []
        self._lines = []                # drawn in green
        self._selected_lines = []       # drawn in red        
        triangles = []    

        ch = self._clickable_height                                             
        while(junctions):

            num = junctions.pop(0)            
            c1,c2 = self._merge[num]     
            h = start_pos[num] + self._height[num]            
            self._lines.append(((x_pos[num],start_pos[num]),(x_pos[num],h)))
                                    
            if not num in self._hidden_junctions:
                # draw horizontal line
                x1,x2 = (get_x_pos(c1), get_x_pos(c2))
                self._lines.append(((x1,h),(x2,h)))
                                                                                                                        
                # register childrens            
                if c1>=0:
                    start_pos[c1-1] = h
                    junctions.append(c1-1)
                else:
                    child_pos.append((h,x1,str(abs(c1))))
                if c2>=0:                
                    start_pos[c2-1] = h
                    junctions.append(c2-1)
                else:
                    child_pos.append((h,x2,str(abs(c2))))                                    
                
                # register callback for switching to triangle
                self._sensitives.append((x1,x2,h-ch,h+ch,self._switch_junction,[num]))
            else:                                
                # find left and right coords.
                while not c1<0:
                    c1 = self._merge[c1-1][0]
                while not c2<0:
                    c2 = self._merge[c2-1][1]
                x1,x2 = get_x_pos_for_bottom(c1),get_x_pos_for_bottom(c2)
                
                # update how
                start_pos[num] = h
                                
                # draw triangle
                triangles.append((x1,x_pos[num],x2,h))
                
                # register callback for switching back
                self._sensitives.append((x1,x2,h-ch,h+ch,self._switch_junction,[num]))                                                                        
        
        # only setting ylim first time            
        if self.ylim == (0,0):
            low = max([y for y,x,name in child_pos]+start_pos)*1.05+1                            
            self.ylim = (low*1.15,0-low*0.05)
        else:
            low = self.ylim[0]/1.15

        # connect lines down            
        for y,x,name in child_pos:
            if name in self._selected:
                self._selected_lines.append(((x,y),(x,low)))
            else:                
                self._lines.append(((x,y),(x,low)))
                    
        # draw triangles
        for x1,x,x2,y in triangles:
            self._lines.append(((x1,low),(x,y)))
            self._lines.append(((x2,low),(x,y)))
            self._lines.append(((x1,low),(x2,low)))
                    
        # clear plot, and start drawing stuff
        self._ax.clear() 
            
        # draw text labels
        for pos, name in zip(self._x_pos,self._order):
            t = self._ax.text(pos-0.12, low*1.01,self._ids[int(name)-1], horizontalalignment="center",
	               verticalalignment="top", rotation=-90)        
            if name in self._selected:
                t.set_color("r")
                               

        # set properties	                         
        self._set_axes_properties()
        
        # draw lines
        lc = LineCollection(self._lines, linewidths=(1,))
        lc.set_color('green')
        ic2 = LineCollection(self._selected_lines, linewidths=(2,))
        ic2.set_color('black')
        self._ax.add_collection(lc)
        self._ax.add_collection(ic2)
        self._canvas.draw()
        
    def _switch_junction(self,junction):

        if junction in self._hidden_junctions:
            self._hidden_junctions.remove(junction)
        else:
            self._hidden_junctions.append(junction)
        self._update_x_pos()
        self.update()
             
    def _update_x_pos(self): 
        self._x_weight = [self._shown_branch_weight] * len(self._order)
        a = list(self._hidden_junctions)

        while a:
            j = a.pop()
            c1,c2 = self._merge[j]
            if c1<0:
                self._x_weight[self._order.index(str(abs(c1)))] = self._hidden_branch_weight
            else:
                a.append(c1-1)
            if c2<0:
                self._x_weight[self._order.index(str(abs(c2)))] = self._hidden_branch_weight
            else:
                a.append(c2-1)
        self._x_pos = [self._x_weight[0]]
        [self._x_pos.append(self._x_weight[num]+self._x_weight[num+1]+self._x_pos[-1]) for num in range(len(self._x_weight)-1)]

    def _click(self, _obj , event):            
        x, y = event.x, event.y # global co-ords
        if self._ax.in_axes(x,y):
            x,y = self._ax.transData.inverse_xy_tup((x,y))  # local coords
            y = self.ylim[0] - y + self.ylim[1]     # converting y from screen coords to data coords
                                              
            for x1,x2,y1,y2,callback,params in self._sensitives:
                if x1 < x < x2 and y1 < y < y2:
                    callback(*params)
                                        
    def rectangle_select_callback(self,x1,y1,x2,y2):
        ids = [self._order[num] for num in range(len(self._x_pos)) if x1 < self._x_pos[num] < x2]
        ids = [self._ids[int(num)-1] for num in ids]
        self.selection_listener(self._dim_name, ids)
        
    def selection_changed(self,selection):
        if self._dim_name in selection:
            self._selected = [str(self._ids.index(id)+1) for id in selection[self._dim_name] if id in self._ids]
            self.update()                                         
                                        
class ScatterPlotFunction(workflow.Function):
    def __init__(self):
        workflow.Function.__init__(self, '_scatter_plot_function',
                                   'Scatter Plot')

    def run(self, data1, data2=None):
        if len(data1.shape)!=2:
            logger.log("warning","Scatter Plot does not support dataset with more than two dimensions")
            return    
        # trying to find default parameters
        if not data2:
            data2 = data1

        # min(union(data1.shape & data2.shape)
        length = min([num for num in data1.shape if num in data2.shape])        
        dim1 = data1.get_dim_names()[list(data1.shape).index(length)]
        dim2 = data2.get_dim_names()[list(data2.shape).index(length)]
        
        if data1==data2 and length>1:
            id1 = data1.get_identifiers(dim1)[0]
            id2 = data2.get_identifiers(dim1)[1]
            return [ScatterPlotBrowse(data1,data2,(dim1,id1),(dim2,id2))]
        elif data1 != data2:
            id1 = data1.get_identifiers(dim1)[0]
            id2 = data2.get_identifiers(dim2)[1]
            a = ScatterPlotBrowse(data1,data2,(dim1,id1),(dim2,id2))
            return [a]                
        elif length==1:
            # IndexPlot!
            id1 = data1.get_identifiers(dim1)[0]
            return [IndexPlot(data1,dim1,id1,name="Index Plot: " +data1.get_name())]
        else:
            # hrmp!
            return
    
class LinePlotFunction(workflow.Function):

    def __init__(self):
        workflow.Function.__init__(self,'line_plot_function','Line Plot')

    def run(self, data1):
        if len(data1.shape)!=2:
            logger.log("warning","Line Plot does not support dataset with more than two dimensions")
            return    
        # trying to find default parameters
        data2 = data1

        # min(union(data1.shape & data2.shape)
        length = min([num for num in data1.shape if num in data2.shape])        
        dim1 = data1.get_dim_names()[list(data1.shape).index(length)]
                
        id1 = data1.get_identifiers(dim1)[0]
        lineplot = LinePlotWithSelector(data1,dim1,id1) 
        return [lineplot]

class ExplainedVariancePlot(ScatterPlot):
    """ Written for displaying ExplainedVariancePlot, may be used for whatever 
    though. Shows percentages, and allows one to only view the X first items.
    Usefull for showing cumulative datasets which converge early    
    """

    def __init__(self,dataset,dim,id,name="Residual Variance Plot X",
                                    xaxis_label="Component",yaxis_label="Residual Variance",
                                    named_xticks=True,use_grid=False,use_objectnames=False):
        
        self.original_dataset = (dataset,dim,id)
        self.use_grid = use_grid
        
        # generating indexset
        index2, dim2, id2 = create_matching_index_dataset(dataset,dim,id)
        
        # Setting up variables for customized toolbar
        max_elements = index2.get_data_slice(dim2,id2).shape[0]        
        self.alternative_cuts = range(0,max_elements,max(1,min(max_elements/4,5)))[1:] 
        if not max_elements in self.alternative_cuts:
            self.alternative_cuts.append(max_elements)
        
        ScatterPlot.__init__(self,index2, dataset,(dim2,id2),(dim,id),
                    name=name,xaxis_label=xaxis_label, yaxis_label=yaxis_label,use_grid=use_grid,
                    use_objectnames=use_objectnames)       
                                                
    def _init_toolbar(self):
        ScatterPlot._init_toolbar(self)
        combobox = self._toolbar.add_combobox(self.alternative_cuts)
        combobox.connect("changed",self.update_dataset)                                                             
        
    def update_dataset(self,combobox):
        def create_dataset(ds,dim,id,num):
            ds,dim,id = self.original_dataset[0:3]
            dim_to_shorten = ds.get_dim_names().index(dim)* -1 + 1
            new_ids = ds.get_identifiers(ds.get_dim_names()[dim_to_shorten])[0:num]
            new_dim_id_pairs = [[dim_,ds.get_identifiers(dim_)] for dim_ in ds.get_dim_names()]
            new_dim_id_pairs[dim_to_shorten][1] = new_ids
            indices_to_keep = range(num)                       
            new_array = Numeric.take(ds.asarray(),indices_to_keep, axis=dim_to_shorten)
            return dataset.Dataset(new_array,new_dim_id_pairs,name=ds.get_name())

        # called on combobox changed
        text = combobox.get_active_text()
        try:
            num = int(text)
        except ValueError:
            return                                
        ds1,dim1,id1 = self.original_dataset
        self._dataset2 = create_dataset(ds1,dim1,id1,num)   
        self._dataset1, self._dim_1, self._id_1 = create_matching_index_dataset(self._dataset2,self._dim_2,self._id_2)                         
        self.update()

    def _ensure_draw_mode(self):
        # overrides normal scatterplot plotting  
        pass

    def draw_plot(self,data_to_plot1, data_to_plot2):       
        self._ensure_draw_mode() 
        self._ax.plot(data_to_plot1, data_to_plot2,'ob',data_to_plot1, data_to_plot2,'k')
        
    def draw_selection(self, data_to_plot1, data_to_plot2):
        self._ensure_draw_mode()
        self._ax.plot(data_to_plot1, data_to_plot2,'or')
        
class PlotPLSExplainedVarY(ScatterPlot):
    def __init__(self,dataset1,dataset2):
        self._use_legend = True
        self.original_dataset = dataset1, dataset2
        max_elements = dataset1.shape[1]        
        self.alternative_cuts = range(1,max_elements,max(1,min(max_elements/4,5)))[1:] 
        if not max_elements in self.alternative_cuts:
            self.alternative_cuts.append(max_elements)            
        self.active_id = 0
        self.max_id = dataset1.shape[0]-1            
        ScatterPlot.__init__(self,dataset1, dataset2, (dataset1[0],dataset1[0,0]),
                        (dataset2[0],dataset2[0,0]),name="Residual Variance Plot: Y",
                                    xaxis_label="Component",yaxis_label="Residual Variance",
                                    use_objectnames=False)            
        self.update_dataset(self.combobox)
        
    def _init_toolbar(self):
        ScatterPlot._init_toolbar(self)
        # adding 
        tbutton_up = self._toolbar.add_up_button()
        tbutton_down = self._toolbar.add_down_button()
        tbutton_legend = self._toolbar.add_legend_button()
        self.combobox = self._toolbar.add_combobox(self.alternative_cuts)       
        # connecting signals
        tbutton_up.connect('clicked', self._up_clicked)
        tbutton_down.connect('clicked', self._down_clicked)
        tbutton_legend.connect("clicked", self._legend_clicked)                             
        self.combobox.connect("changed",self.update_dataset)

    def _legend_clicked(self,*ign):
        self._use_legend = not self._use_legend
        self.update()
            
    def _up_clicked(self,*stuff):
        if self.active_id < self.max_id:
            self.active_id += 1
            self._id_2 = self._dataset2.get_identifiers(self._dim_2)[self.active_id]
            self._id_1 = self._dataset1.get_identifiers(self._dim_1)[self.active_id]
            self.update()    
    
    def _down_clicked(self,*stuff):
        if self.active_id > 0:
            self.active_id -= 1
            self._id_2 = self._dataset2.get_identifiers(self._dim_2)[self.active_id]
            self._id_1 = self._dataset1.get_identifiers(self._dim_1)[self.active_id]            
            self.update()
                
    def update_dataset(self,combobox):
        def create_dataset(ds,dim,id,num):
            dim_to_shorten = ds.get_dim_names().index(dim)* -1 + 1
            new_ids = ds.get_identifiers(ds.get_dim_names()[dim_to_shorten])[0:num]
            new_dim_id_pairs = [[dim_,ds.get_identifiers(dim_)] for dim_ in ds.get_dim_names()]
            new_dim_id_pairs[dim_to_shorten][1] = new_ids
            indices_to_keep = range(num)                       
            new_array = Numeric.take(ds.asarray(),indices_to_keep, axis=dim_to_shorten)
            return dataset.Dataset(new_array,new_dim_id_pairs,name=ds.get_name())

        # called on combobox changed
        text = combobox.get_active_text()
        try:
            num = int(text)
        except ValueError:
            return                        
        ds1,ds2 = self.original_dataset
        self._dataset1 = create_dataset(ds1,ds1[0],ds1[0,self.active_id],num)
        self._dataset2 = create_dataset(ds2,ds2[0],ds2[0,self.active_id],num)          
        self.update()    
    
               
    def _ensure_draw_mode(self):
        # overrides normal scatterplot plotting  
         pass   
        
    def draw_plot(self,*args,**kwargs):
        self._ax.plot(range(1,len(args[0])+1),args[0],'sg',label='_nolegend_')
        self._ax.plot(range(1,len(args[0])+1),args[0],'-g',label='Validation')
        self._ax.plot(range(1,len(args[1])+1),args[1],'ob',label='_nolegend_')
        self._ax.plot(range(1,len(args[1])+1),args[1],'-b',label='Calibration')
        if self._use_legend:
            self._ax.legend()
                              
    def draw_selection(self, *args, **kwargs):
        for arg in args:
            self._ax.plot(range(1,len(arg)+1),arg,'or',label="_nolegend_")
            
class ScatterPlotWithUpDownLegend(ScatterPlot):
    def __init__(self, textlist,*args,**kwargs):
        self._legend_active = True
        self._textdict = textlist
        ScatterPlot.__init__(self,*args,**kwargs)
        

    def _init_toolbar(self):
        ScatterPlot._init_toolbar(self)
        tbutton_up = self._toolbar.add_up_button()
        tbutton_down = self._toolbar.add_down_button()
        tbutton_legend = self._toolbar.add_legend_button()
                
        tbutton_up.connect('clicked', self._up_clicked)
        tbutton_down.connect('clicked', self._down_clicked)
        tbutton_legend.connect("clicked", self._legend_clicked)
        
    def draw_plot(self,*stuff):
        ScatterPlot.draw_plot(self,*stuff)        
        # find new wanted scale before inserting line
        self._find_scale()                                
        if self._legend_active:
            text = []
            for t in self._textdict[self._id_1]:
                text.append(t[0] + str(t[1]))
            linelist = [None] * len(text)
            self._ax.legend(linelist, text)                    
            self._ax.plot(*self._find_line(*self._textdict[self._id_1][2:4]))
        self.ensure_scale()
        
    def _find_line(self,(ignored,offset),(ignored2,slope)):
        y_start = offset + slope * self._xscale[0]
        y_end   = offset + slope * self._xscale[1]        
        return (self._xscale, [y_start,y_end])
        
    def _find_scale(self):
        ymin,ymax = self._ax.get_ylim()
        increase = (ymax-ymin)*0.05
        self._yscale = (ymin-increase,ymax+increase)
        
        xmin,xmax = self._ax.get_xlim()
        increase = (xmax-xmin)*0.05
        self._xscale = (xmin-increase,xmax+increase)
        
    def ensure_scale(self):
        # ensures sane scales are set by adding by expanding limits by 10%           
        self._ax.set_ylim(*self._yscale)       
        self._ax.set_xlim(*self._xscale)
        
    def _legend_clicked(self, *ignored):
        self._legend_active = not self._legend_active
        self.update()
        
    def _up_clicked(self, *ignored):
        y_number = self._dataset2.get_indices(self._dim_2,[self._id_2])[0]
        if y_number + 1 < self._dataset2.get_dimension_length(self._dim_2):
            self._id_2 = self._dataset2.get_identifiers_from_indices(self._dim_2,[y_number+1])[0]            
        x_number = self._dataset1.get_indices(self._dim_1,[self._id_1])[0]
        if x_number + 1 < self._dataset1.get_dimension_length(self._dim_1):
            self._id_1 = self._dataset1.get_identifiers_from_indices(self._dim_1,[x_number+1])[0]
        self.update()            
            
    def _down_clicked(self, *ignored):
        y_number = self._dataset2.get_indices(self._dim_2,[self._id_2])[0]
        if y_number > 0:
            self._id_2 = self._dataset2.get_identifiers_from_indices(self._dim_2,[y_number-1])[0]
        x_number = self._dataset1.get_indices(self._dim_1,[self._id_1])[0]
        if x_number > 0:
            self._id_1 = self._dataset1.get_identifiers_from_indices(self._dim_1,[x_number-1])[0]
        self.update()                
                                                                  
        
class PredPLSScatterPlotWithUpDownLegend(ScatterPlotWithUpDownLegend):
                
    def __init__(self,coefficients,X,Y,max_comp=-1):
        self._legend_active = True
        
        if max_comp == -1:                
            self.max_comps = coefficients.shape[2]
        else:
            self.max_comps = max_comp
            
        self.pred_params   = [coefficients, X, Y, self.max_comps]
        yhat, self._textdict = self.predict(*self.pred_params)
        
        dim1,dim2 = yhat.get_dim_names()[1] , Y.get_dim_names()[1]
        id1, id2 = yhat.get_identifiers(dim1)[0], Y.get_identifiers(dim2)[0]        
        ScatterPlot.__init__(self,yhat,Y,(dim1,id1),(dim2,id2))
        self.update()                    
                       
    def _init_toolbar(self):        
        ScatterPlotWithUpDownLegend._init_toolbar(self)        
        spinbox = self._toolbar.add_spin_button(self.max_comps,self.max_comps)                                
        # connect to buttons                                                                                
        spinbox.connect("changed",self.spinbox_changed)
                        
    def spinbox_changed(self,spinbox):
        self.pred_params[-1] = spinbox.get_value_as_int()
        yhat, textlist = self.predict(*self.pred_params)
        
        self._dataset1 = yhat
        self._textdict = textlist
        self.update()

    def update(self):
        ScatterPlotWithUpDownLegend.update(self)    
        self.plot_title = "Predicted vs True, " +  str(self.pred_params[-1]) + " comps. used."
             
    def predict(self, coeffs, X, Y, comps):
        xc = Numeric.ones((X.shape[0],X.shape[1]+1),X.asarray().typecode())
        xc[:,1:] = X.asarray()
        pred_Y = Numeric.matrixmultiply(xc,coeffs.asarray()[:,:,comps-1])
        pred_Y = dataset.Dataset(pred_Y,
                                ((X[0],X.get_identifiers(X[0])),
                                (Y[1],Y.get_identifiers(Y[1]))),name="Predicted Y")                               
        inf = {}
        import stats
        for i,id in enumerate(Y.get_identifiers(Y[1])):
            slope, offset = stats.linregress(Y.asarray()[:,i],pred_Y.asarray()[:,i],1)[0:2]
            corr = scipy.corrcoef(Y.asarray()[:,i],pred_Y.asarray()[:,i])[0,1]                                                                                                      
            inf[id] = [("R: " , corr),
                    ("R^2: ", corr*corr),
                    ("offset: " , offset),
                    ("slope: " , slope)]  
        return [pred_Y, inf]                      
        
class ValidatedScatterPlotWithUpDownLegend(ScatterPlotWithUpDownLegend):

    def __init__(self,datasets,Y):
        self._legend_active = True    
        self._datasets = datasets            
        self.max_comps = len(datasets)
        self.num = self.max_comps
        yhat = datasets[-1]         
                
        dim1,dim2 = yhat.get_dim_names()[1] , Y.get_dim_names()[1]
        id1, id2 = yhat.get_identifiers(dim1)[0], Y.get_identifiers(dim2)[0]        
        ScatterPlot.__init__(self,yhat,Y,(dim1,id1),(dim2,id2),xaxis_label="True %s",
                    yaxis_label="Validated Prediction %s",name="Predicted vs True Crossvalidated")
        self.update()                    
                       
    def _init_toolbar(self):        
        ScatterPlotWithUpDownLegend._init_toolbar(self)        
        spinbox = self._toolbar.add_spin_button(self.max_comps,self.max_comps)
        
        self.spinbox_changed(spinbox) # urg, ugly. force spinbox changed to update                                
        # connect to buttons                                                                                
        spinbox.connect("changed",self.spinbox_changed)
                        
    def spinbox_changed(self,spinbox):
        self._dataset1 = self._datasets[spinbox.get_value_as_int()-1]
        self.num = spinbox.get_value_as_int()
        silent_eval = rpy.with_mode(rpy.NO_CONVERSION, rpy.r)
        silent_set = rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)
        
        textlist = {}
        silent_set("Y",self._dataset2.asarray())
        silent_set("yhat", self._dataset1.asarray())                        
        
        y_ids2 = self._dataset2.get_identifiers(self._dataset2.get_dim_names()[1])
        for x in range(1,self._dataset2.shape[1]+1):
            silent_eval("y <- as.matrix(Y[,%s])" % (x,))
            silent_eval("yfhat <- as.matrix(yhat[,%s])" % (x,))
            silent_eval("data_set <- as.data.frame(cbind(y,yfhat))")
            silent_eval('colnames(data_set) <- c("Y","Y_pred")')
            silent_eval("lm_regr <- lm(Y~ Y_pred,data_set)")
    
            correlation = rpy.r("round(cor(y,yhat),4)")[0,0]
            correlation_squared = rpy.r("round((cor(y,yhat))^2,4)")[0,0]
            offset = rpy.r("round(lm_regr$coef[1],4)").popitem()[1]
            slope = rpy.r("round(lm_regr$coef[2],4)").popitem()[1]
                
            textlist[y_ids2[x-1]] = (("R:", correlation),
                    ("R^2: " , correlation_squared),
                    ("offset: " , offset),
                    ("slope: " , slope))        
                        
        self._textdict = textlist
        self.update()

    def update(self):
        ScatterPlotWithUpDownLegend.update(self)                                                 
        
class ScatterPlotWithUpDown(ScatterPlot): 

    def _init_toolbar(self):
        ScatterPlot._init_toolbar(self)
        tbutton_up = self._toolbar.add_up_button()
        tbutton_down = self._toolbar.add_down_button()                        
        # connect to buttons                                                                
        tbutton_up.connect('clicked', self._up_clicked)
        tbutton_down.connect('clicked', self._down_clicked)

    def _up_clicked(self, *ignored):
        y_number = self._dataset2.get_indices(self._dim_2,[self._id_2])[0]
        if y_number + 1 < self._dataset2.get_dimension_length(self._dim_2):
            self._id_2 = self._dataset2.get_identifiers_from_indices(self._dim_2,[y_number+1])[0]            
        x_number = self._dataset1.get_indices(self._dim_1,[self._id_1])[0]
        if x_number + 1 < self._dataset1.get_dimension_length(self._dim_1):
            self._id_1 = self._dataset1.get_identifiers_from_indices(self._dim_1,[x_number+1])[0]
        self.update()            
            
    def _down_clicked(self, *ignored):
        y_number = self._dataset2.get_indices(self._dim_2,[self._id_2])[0]
        if y_number > 0:
            self._id_2 = self._dataset2.get_identifiers_from_indices(self._dim_2,[y_number-1])[0]
        x_number = self._dataset1.get_indices(self._dim_1,[self._id_1])[0]
        if x_number > 0:
            self._id_1 = self._dataset1.get_identifiers_from_indices(self._dim_1,[x_number-1])[0]
        self.update()                
                                                  
                                     
class LeveragePlot(ScatterPlot):
    def __init__(self, t_data,p_data, **kwargs):
        if not 'name' in kwargs:
            kwargs['name'] = "Leverage"
        self._original_title = "Leverage: %s : %s components used"        
        self.MODE_VARIABLES = False
        self.t_data = t_data
        self.p_data = p_data
        self.x_dim_1 = self.t_data.get_dim_names()[0]
        self.x_ids_1 = self.t_data.get_identifiers(self.x_dim_1)
        self.x_dim_2 = self.p_data.get_dim_names()[0]
        self.x_ids_2 = self.p_data.get_identifiers(self.x_dim_2)
        self.num_components = int(self.t_data.asarray().shape[1])
        self.calculate_leverage()
        if not self.leverage_object:
            return None
        dim = self.leverage_object.get_dim_names()[1]
        id = self.leverage_object.get_identifiers(dim)[0]
        data2, dim2, id2 = create_matching_index_dataset(self.leverage_object, dim, id)
        ScatterPlot.__init__(self, data2, self.leverage_object, (dim2, id2),
                                    (dim, id), **kwargs)                
        self.changed(self.spinbutton)

    def _init_toolbar(self):
        ScatterPlot._init_toolbar(self)
        self.spinbutton = self._toolbar.add_spin_button(self.num_components,self.num_components)
        button_switch_var = self._toolbar.add_switch_button()
        self.spinbutton.connect("value-changed", self.changed)
        button_switch_var.connect("clicked",self.mode_changed)        

    def mode_changed(self,*ignored):
        self.MODE_VARIABLES = not self.MODE_VARIABLES
        self.changed(self.spinbutton)
   
    def calculate_leverage(self):
        if not self.MODE_VARIABLES:
            rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("T", self.t_data.asarray()[:,0:self.num_components])
            try:
                lev = rpy.r("as.matrix(diag(T%*%solve(crossprod(T))%*%t(T)))")
                self.leverage_object = dataset.Dataset(lev,[(self.x_dim_1,self.x_ids_1),(None,None)])
            except rpy.RException:
                # singular matrix
                self.num_components -= 1
                return self.calculate_leverage()
        else:                
            rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("P", self.p_data.asarray()[:,0:self.num_components])
            try:
                lev = rpy.r("as.matrix(diag(P%*%t(P)))")            
                self.leverage_object = dataset.Dataset(lev,[(self.x_dim_2,self.x_ids_2),(None,None)])         
            except rpy.RException:
                # singular matrix
                self.num_components -= 1
                return self.calculate_leverage()

    def changed(self, spinbutton):
        self.num_components = spinbutton.get_value_as_int()
        self.calculate_leverage()
        dim = self.leverage_object.get_dim_names()[1]
        id = self.leverage_object.get_identifiers(dim)[0]
        data2, dim2, id2 = create_matching_index_dataset(self.leverage_object, dim, id)
        self._dataset2 = self.leverage_object
        self._dataset1 = data2
        self._dim_2 = dim
        self._dim_1 = dim2
        self._id_2 = id
        self._id_1 = id2
        if self.MODE_VARIABLES:       
            self.plot_title = self._original_title % ("Variables",self.num_components)
            self._xaxis_label = "Variables"
        else:
            self.plot_title = self._original_title % ("Samples",self.num_components)
            self._xaxis_label = "Samples"        
        self.update()
        
        

class IndexPlot(ScatterPlot):
    def __init__(self,data,dim,id,draw_lines=False,**kwargs):
        self._use_lines = draw_lines
        data2, dim2, id2 = create_matching_index_dataset(data,dim,id)
        if not "xaxis_label" in kwargs:
            kwargs['xaxis_label'] = "Index"
        if not "yaxis_label" in kwargs:
            kwargs['yaxis_label'] = "%s"
        ScatterPlot.__init__(self,data2,data,(dim2,id2),(dim,id),**kwargs)

    def draw_plot(self,data_to_plot1,data_to_plot2):
        """ Draws the actual plot """         
        self._ax.plot(data_to_plot1, data_to_plot2,'ob')
        if self._use_lines:
            self._ax.plot(data_to_plot1, data_to_plot2,'_g')

class LinePlotWithSelector(ScatterPlotBrowse):
    def __init__(self,data,dim,id):
        
        data2, dim2, id2 = create_matching_index_dataset(data,dim,id)
        ScatterPlotBrowse.__init__(self,data2,data,(dim2,id2),(dim,id),
                name="Line Plot",run_selector=False)

    def draw_plot(self,data_to_plot1,data_to_plot2):
        """ Draws the actual plot """         
        self._ax.plot(data_to_plot1, data_to_plot2,'ob', ms=3)
        self._ax.plot(data_to_plot1, data_to_plot2,'-g')

class LinePlotWithBoundary(LinePlotWithSelector):
    def __init__(self,data,dim,id,line_height):
        self._line_height = line_height
        LinePlotWithSelector.__init__(self,data,dim,id)

    def draw_plot(self,data_to_plot1, data_to_plot2):
        self._ax.axhline(y=self._line_height)
        LinePlotWithSelector.draw_plot(data_to_plot1, data_to_plot2)                                                                                               

class Biplot(plots.Plot):
    def __init__(self,scores, loading, score_dim, loading_dim, loading_id1, loading_id2,use_objectnames=True):    
        plots.Plot.__init__(self,"Biplot")
                            
        self._use_grid = True
        self._use_objectnames = use_objectnames
        self._use_legend = True
        self._selections = []
                
        # plot init
        fig = pylab.Figure(figsize=(5,4), dpi=72)
        self._canvas = FigureCanvas(fig)
        self._ax = fig.add_subplot(111)        
        self.add(self._canvas)
        self._canvas.show()                    
        self.datasets = scores, loading
        self.dims     = score_dim, loading_dim
        self.ids      = [None, None, loading_id1, loading_id2]
        self._update_score_ids_from_loading()
                
        self._init_toolbar()
        self._init_popupmenu()        
        self.update()  
    
    def _init_toolbar(self):
        # create toolbar
        self._toolbar = ScatterPlotToolbar(self._canvas, None)         
        # add buttons
        tbutton_up = self._toolbar.add_up_button()
        tbutton_down = self._toolbar.add_down_button()
        tbutton_back = self._toolbar.add_left_button()
        tbutton_forward = self._toolbar.add_right_button() 
        tbutton_legend = self._toolbar.add_legend_button()                       
        
        # connect to buttons                                                                  
        tbutton_up.connect('clicked', self._up_clicked)
        tbutton_down.connect('clicked', self._down_clicked)
        tbutton_back.connect('clicked', self._back_clicked)
        tbutton_forward.connect('clicked', self._forward_clicked)
        tbutton_legend.connect('clicked', self.switch_legend)
        
        # connect various
        self._toolbar.show_all()
        self._toolbar.set_select_callback(self.rectangle_select_callback)
        
    def get_popupmenu(self):
        return self._popupmenu
   
    def _init_popupmenu(self):
        self._popupmenu = plots.CustomPopupMenu(self)
        self._popupmenu.init_from_toolbar(self.get_toolbar())           
        self._popupmenu.add_togglebutton("Use Grid",self.switch_grid,self._use_grid)
        self._popupmenu.add_togglebutton("Named Objects",self.switch_named_objects,self._use_objectnames)
       
    def get_toolbar(self):
        return self._toolbar      
            
    #Slots for GUI handling when selecting what data to display
    def _forward_clicked(self, *ignored):
        x_number = self.datasets[1].get_indices(self.dims[1],[self.ids[2]])[0]
        if x_number + 1 < self.datasets[1].get_dimension_length(self.dims[1]):
            self.ids[2]= self.datasets[1].get_identifiers_from_indices(self.dims[1],[x_number+1])[0]
            self._update_score_ids_from_loading()
            self.update()
            
    def _back_clicked(self, *ignored):
        x_number = self.datasets[1].get_indices(self.dims[1],[self.ids[2]])[0]
        if x_number > 0:
            self.ids[2] = self.datasets[1].get_identifiers_from_indices(self.dims[1],[x_number-1])[0]
            self._update_score_ids_from_loading()
            self.update()
            
    def _up_clicked(self, *ignored):
        y_number = self.datasets[1].get_indices(self.dims[1],[self.ids[3]])[0]
        if y_number + 1 < self.datasets[1].get_dimension_length(self.dims[1]):
            self.ids[3] = self.datasets[1].get_identifiers_from_indices(self.dims[1],[y_number+1])[0]
            self._update_score_ids_from_loading()
            self.update()
            
    def _down_clicked(self, *ignored):
        y_number = self.datasets[1].get_indices(self.dims[1],[self.ids[3]])[0]
        if y_number > 0:
            self.ids[3] = self.datasets[1].get_identifiers_from_indices(self.dims[1],[y_number-1])[0]
            self._update_score_ids_from_loading()
            self.update()
    
    def _get_selected_combo_text(self):
        liststore = self.display_mode_combobox.get_model()
        iter = liststore.get_iter(self.display_mode_combobox.get_active())
        key  = (liststore.get_value(iter,0) , liststore.get_value(iter,1))
        return key 
    
    def switch_legend(self,*ignored):
        self._use_legend = not self._use_legend
        self.update()
                              
    def switch_grid(self,*ignored):
        self._use_grid = not self._use_grid
        self.update()
    
    def switch_named_objects(self,*ignored):
        self._use_objectnames = not self._use_objectnames
        self.update()                    

    def _update_score_ids_from_loading(self):
        ids = self.datasets[1].get_identifiers(self.dims[1])
        num1, num2 = ids.index(self.ids[2]) , ids.index(self.ids[3])
        ids = self.datasets[0].get_identifiers(self.dims[0])
        self.ids[0] = ids[num1]
        self.ids[1] = ids[num2]                                    
        
    def _scale(self,li):
            rpy.r.assign("li",li)
            return rpy.r("scale(li)")
                                                
    def update(self,selections=None):
        """Refreshes the plot according to the new parameters."""
            
        self._ax.clear()
        self._ax.set_title(self.get_title())
        
        self._ax.set_xlabel("Component %s" % (self.ids[2]))
        self._ax.set_ylabel("Component %s" % (self.ids[3]))
        
        # enable grid?
        if self._use_grid:
            self._ax.grid(b=True)
        
        for num, dataset in enumerate(self.datasets):                                     
            data_to_plot1 = Numeric.ravel(self._scale(dataset.get_data_slice(self.dims[num],self.ids[num*2])))
            data_to_plot2 = Numeric.ravel(self._scale(dataset.get_data_slice(self.dims[num],self.ids[num*2+1])))            

            # set object names
            opposite_dim2 = [dim for dim in dataset.get_dim_names() if dim != self.dims[num]][0]
        
            # draw text
            if self._use_objectnames:
                for x,id in enumerate(dataset.get_identifiers(opposite_dim2)):
                    self._ax.text(data_to_plot1[x],data_to_plot2[x],id)                
            if data_to_plot1.shape != data_to_plot2.shape:
                logger.log('warning', "Error trying to plot vectors of different sizes")
            else:          
                if num == 0:
                    self.draw_plot(data_to_plot1,data_to_plot2,mode='o',label="Scores",ms=5,mec='black',mfc='darkblue')
                else:
                    self.draw_plot(data_to_plot1,data_to_plot2,mode='s',label="Loadings",ms=8,mec='white',mfc='lightgreen')
                                    
        if self._selections:
            for selection in self._selections:
                self.draw_selection(*selection)
                
        # enable legend?                
        if self._use_legend:            
            self._ax.legend()                
        self.ensure_scale()                                                       
        self._canvas.draw()
            
    def ensure_scale(self):
        # ensures sane scales are set by adding by expanding limits by 10%
        ymin,ymax = self._ax.get_ylim()
        increase = (ymax-ymin)*0.05           
        self._ax.set_ylim(ymin-increase,ymax+increase)
        xmin,xmax = self._ax.get_xlim()
        increase = (xmax-xmin)*0.05           
        self._ax.set_xlim(xmin-increase,xmax+increase)
    
    def draw_plot(self,data_to_plot1,data_to_plot2,mode="o",**kwargs):
        """ Draws the actual plot """         
        self._ax.plot(data_to_plot1, data_to_plot2,mode,**kwargs)        
            
    def draw_selection(self,data_to_plot1, data_to_plot2,mode='or',label='_nolegend_'):
        """ Redraw marked items with different colours """    
        self._ax.plot(data_to_plot1, data_to_plot2,mode,label=label)
               
    def rectangle_select_callback(self, x1, y1, x2, y2):
        selections = []
        for num, dataset in enumerate(self.datasets):
                    
            # convert id, dim to numbers
            id_number1  = dataset.get_identifiers(self.dims[num]).index(self.ids[num*2]) 
            id_number2  = dataset.get_identifiers(self.dims[num]).index(self.ids[num*2+1])
            dim_number1 = dataset.get_dim_names().index(self.dims[num])            
        
            # when comparing ids we want dimension data runs *along*, not dimension it's selected *from*
            opposite_dim1 = dataset.get_dim_names()[dim_number1 * -1 + 1]
                    
            # fetch autoscaled data, filter out relevant                    
            xdata = Numeric.ravel(self._scale(dataset.get_data_slice(self.dims[num],self.ids[num*2])))
            ydata = Numeric.ravel(self._scale(dataset.get_data_slice(self.dims[num],self.ids[num*2+1])))
        
            # selections may start on the right/bottom and go up/leftwards
            x , y = [x1, x2] , [y1, y2]
            x.sort(), y.sort()
        
            # find indices of selected area
            index =scipy.nonzero((xdata>x[0]) & (xdata<x[1]) & (ydata>y[0]) &
                                     (ydata<y[1]))                                                 
            ids1 = dataset.get_identifiers(opposite_dim1, index)            
            selections.append((opposite_dim1, ids1))
            
        # make selectionlist unique
        import copy        
        [selections.remove(selection) for selection in copy.copy(selections) if selections.count(selection)!=1]
        for dim, ids in selections:
            self.selection_listener(dim,ids)                                                 
        
    def selection_changed(self, selection):
        selections = []
        for key in selection:
            for num, dataset in enumerate(self.datasets):
                # getting dim data runs *along*
                opposite_dim1 = [dim for dim in dataset.get_dim_names() if dim != self.dims[num]][0]        
                # is this selection relevant?
                if key != opposite_dim1:
                    continue                                   
                indices = []            
                available_ids1 = dataset.get_identifiers(opposite_dim1)
                ids1_union = [sel for sel in selection[key] if sel in available_ids1]
                indices.extend(dataset.get_indices(opposite_dim1, ids1_union))
                                                
                # fetch auto-scaled data, filter it.
                data_to_plot1 = Numeric.ravel(self._scale(dataset.get_data_slice(self.dims[num],self.ids[num*2])))                      
                data_to_plot1 = scipy.take(data_to_plot1, indices)
                data_to_plot2 = Numeric.ravel(self._scale(dataset.get_data_slice(self.dims[num],self.ids[num*2+1])))
                data_to_plot2 = scipy.take(data_to_plot2, indices)
                if num==0:
                    selections.append((data_to_plot1,data_to_plot2,'or'))
                else:
                    selections.append((data_to_plot1,data_to_plot2,'sr'))
                
        self._selections = selections
        self.update()
        


    
class InfluencePlot(ScatterPlot):
    def __init__(self, t_data,p_data, x_data, center, name="Influence Plot", plotname="Influence Plot: %s : %s components"):
        self.t_data = t_data
        self.p_data = p_data
        self.MODE_VARIABLES = False        
        x = Numeric.zeros(x_data.shape,Numeric.Float)
        for num in range(x_data.shape[1]):
            x[:,num] = x_data.asarray()[:,num] - float(center.asarray()[0,num])        
        self.X = x        
        
        self._original_title = plotname        
        self.x_dim_2 = x_data.get_dim_names()[1]
        self.x_ids_2 = x_data.get_identifiers(self.x_dim_2)
        self.x_dim_1 = x_data.get_dim_names()[0]
        self.x_ids_1 = x_data.get_identifiers(self.x_dim_1)                
        self.num_components = int(self.t_data.asarray().shape[1])      
        self.calculate_leverage()
        self.calculate_residuals()        
            
        dim = self.leverage_object.get_dim_names()[1]
        id = self.leverage_object.get_identifiers(dim)[0]
        data2, dim2, id2 = create_matching_index_dataset(self.leverage_object, dim, id)

        ScatterPlot.__init__(self, self.leverage_object,data2,
                                    (dim, id), (dim2, id2), name=name, yaxis_label="Residuals",xaxis_label="Leverage")                              
        self.changed(self.spinbutton)
    
    def mode_changed(self,*ignored):
        self.MODE_VARIABLES = not self.MODE_VARIABLES
        self.changed(self.spinbutton)
    
    def _init_toolbar(self):
        ScatterPlot._init_toolbar(self)
        self.spinbutton = self._toolbar.add_spin_button(self.num_components,max(1,int(self.num_components*0.75)))
        tbutton_switch_var = self._toolbar.add_switch_button()        
        self.spinbutton.connect("value-changed", self.changed)
        tbutton_switch_var.connect("clicked",self.mode_changed)
        
   
    def calculate_leverage(self):
        if not self.MODE_VARIABLES:
            rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("T", self.t_data.asarray()[:,0:self.num_components])
            try:
                lev = rpy.r("as.matrix(diag(T%*%solve(crossprod(T))%*%t(T)))")
                self.leverage_object = dataset.Dataset(lev,[(self.x_dim_1,self.x_ids_1),(None,None)])
            except rpy.RException:
                # singular matrix
                self.num_components -= 1
                return self.calculate_leverage()
        else:                
            rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("P", self.p_data.asarray()[:,0:self.num_components])
            try:
                lev = rpy.r("as.matrix(diag(P%*%t(P)))")            
                self.leverage_object = dataset.Dataset(lev,[(self.x_dim_2,self.x_ids_2),(None,None)])         
            except rpy.RException:
                # singular matrix
                self.num_components -= 1
                return self.calculate_leverage()
                    
    def calculate_residuals(self):
        rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("T", self.t_data.asarray()[:,0:self.num_components])        
        rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("P", self.p_data.asarray()[:,0:self.num_components])
        rpy.with_mode(rpy.NO_CONVERSION, rpy.r.assign)("m", self.X)
        if not self.MODE_VARIABLES:
            try:
                residuals = rpy.r("as.matrix(apply(abs(m-T%*%t(P)),1,sum))")
                self.residuals = dataset.Dataset(residuals,[(self.x_dim_1,self.x_ids_1),(None,None)])
            except rpy.RException:
                self.residuals = None
        else: 
            try:
                residuals = rpy.r("as.matrix(apply(abs(m-T%*%t(P)),2,sum))")
                self.residuals = dataset.Dataset(residuals,[(self.x_dim_2,self.x_ids_2),(None,None)])
            except rpy.RException:
                self.residuals = None                                                                                   

    def changed(self, spinbutton):
        self.num_components = spinbutton.get_value_as_int()        
        self.calculate_leverage()
        self.calculate_residuals()                  
        self._dataset2 = self.leverage_object
        self._dataset1 = self.residuals
        self._dim_2 = self.leverage_object.get_dim_names()[1]
        self._dim_1 = self.residuals.get_dim_names()[1]
        self._id_2 = self.leverage_object.get_identifiers(self._dim_2)[0]
        self._id_1 = self.residuals.get_identifiers(self._dim_1)[0]
                
        if self.MODE_VARIABLES:       
            self.plot_title = self._original_title % ("Variables",self.num_components)
        else:
            self.plot_title = self._original_title % ("Samples",self.num_components)        
        self.update()
        
class PLSLoadingsPlot(ScatterPlotBrowse):
    def __init__(self,loadings2,*args,**kwargs):
        ScatterPlotBrowse.__init__(self,*args,**kwargs)
        tbutton_switch = self._toolbar.add_switch_button()
        tbutton_switch.connect("clicked",self.changed)                 
        self.plotnames = ["W Loadings for X","Q Loadings for Y"]
        self.datasets  = [args[0],loadings2]
        self.plot_in_use = 0
        self.plot_title = self.plotnames[self.plot_in_use]
        self.update()
        
    def changed(self,*args):
        self.plot_in_use = self.plot_in_use * -1 + 1        
        self.plot_title = self.plotnames[self.plot_in_use]        
        self._dataset1 = self.datasets[self.plot_in_use]         
        self._dataset2 = self.datasets[self.plot_in_use]
        self._dim_1 = self._dataset1.get_dim_names()[1]
        self._dim_2 = self._dim_1
        self._id_1 = self._dataset1.get_identifiers(self._dim_1)[0]
        self._id_2 = self._dataset1.get_identifiers(self._dim_1)[1]
        self.update()

def create_matching_index_dataset(dataset_,dim,id):
    shape_ = dataset_.get_data_slice(dim,id).shape
    indexset = dataset.Dataset(Numeric.reshape(range(1,1+Numeric.product(shape_))
                                    ,shape_),[("index",None),(None,None)])
    first_dim = indexset.get_dim_names()[0] 
    first_id  = indexset.get_identifiers(first_dim)[0]
    return (indexset, first_dim, first_id)      


class Fourier(workflow.Function):
    """ <b>Fourier Transform</b> """
    
    def __init__(self,wf,*stuff):
        workflow.Function.__init__(self,*stuff)
        self.wf = wf
        
    
    def run(self,data):
        arr = data.asarray()
        res = []
        for x in xrange(arr.shape[0]):
            rpy.r.assign("x",arr[x])
            res.append(rpy.r("fft(x)"))
        # remember to add proper labels!
        arr = dataset.Dataset(res,[(data[0],data.get_identifiers(data[0])),("Wavelengths",None)],name=data.get_name() + " Fourier")
        arr.add_to_description("<b>Fourier Transformed from:</b> "+data.get_name())
        halved = dataset.Dataset(abs(arr.asarray()[:,0:int((arr.shape[1]+0.5)/2)].astype('f')),[(arr[0],arr.get_identifiers(data[0])),("Wavelengths",None)],name=arr.get_name())
                    
        plot = plots.IndexPlot(halved,Bar=False,Line=True,Points=False)
        plot.set_title("Powerspectra, " + halved.get_name())
        plot2 = InvFourierPlot(arr,self.wf.app.project,data)
        plot2.set_title("Inverted")
        plot2.set_description("Inverted Fourier data: Selected items on Powerspectra set to zero.")
            
        return arr, plot, plot2
        
  
class InvFourier(workflow.Function):
    """ <b> Inverse Fourier Transform </b>"""
    def run(self,data):
        arr = data.asarray()
        res = []
        for x in xrange(arr.shape[0]):
            rpy.r.assign("x",arr[x])
            res.append(rpy.r("as.real(fft(x,inverse=TRUE))/length(x)"))

        # remember to add proper labels!
        arr = dataset.Dataset(res)        
        plot = plots.IndexPlot(arr,Bar=False,Line=True,Points=False)         
        return arr, plot
        
class InvFourierPlot(plots.IndexPlot):
    def __init__(self, data, project, parent, *stuff,**stuffer):
    
        self.org_array = data._array.copy()
        self._project = project
        self._parent = parent
        
        dims = [(dim,data.get_identifiers(dim)) for dim in data.get_dim_names()]
        plots.IndexPlot.__init__(self,dataset.Dataset(self.org_array,dims,name=data.get_name()),*stuff,**stuffer)

    def _init_popupmenu(self):
        plots.IndexPlot._init_popupmenu(self)
        self._popupmenu.add_separator()
        self._popupmenu.add_button("Export Current",self.export_current)

    def export_current(self,button):
        dims = [(dim,self._data.get_identifiers(dim)) for dim in self._data.get_dim_names()]
        data = dataset.Dataset(self._data._array,dims,name=self._data.get_name())
    
        self._project.add_data([self._parent],[data],fun=None)       
                                            
    def selection_changed(self, selection):
        self._data._array = self.org_array.copy()
        sel = self.parse_selection(selection)
        if self._data[1] in sel and sel[self._data[1]]:
            for ind in sel[self._data[1]][0]:            
                self._data._array[:,ind] = 0
                self._data._array[:,-ind-1] = 0  
        self.update()

    def draw(self, x, y):
        rpy.r.assign("y",y)
        y = rpy.r("as.real(fft(y,inverse=TRUE))/length(y)")
    
        width = 0.8*sum(x-x[0])/sum(range(len(x)))
        if self._shown[0]:            
            self._ax.plot(x, y,'-g')
        if self._shown[2]:
            self._ax.plot(x, y,'ob', ms=3)                   
        if self._shown[1]:        
            self._ax.bar(Numeric.asarray(x)-width/2,y,width=width)
            self._ax.set_xlim(x[0]-width*0.8,x[-1]+width*0.8)                                

class CompareWavelets(workflow.Function):
    """ <b>Find Best Wavelet</b>

Finds residuals for every waveletbasis on first wavelength sample, only using first two wavelet levels."""
    def run(self,data):
        ensure_r_libraries(["waveslim"])
        rpy.r.library("waveslim")    
        row = Numeric.ravel(data.asarray()[0])
        row = row[0:2**int(math.log(len(row),2))]
        
        rpy.r.assign("r",row)
        wavelets = ['haar', 'd4', 'mb4', 'bs3.1', 'fk4', 'd6', 'fk6', 'd8', 'fk8', 'la8', 'bl14',
         'fk14', 'd16', 'la16', 'mb16', 'la20', 'bl20', 'fk22', 'mb24']
        result = {} 
        for wavelet in wavelets:            
            result[wavelet] = abs(sum(row-rpy.r("idwt(dwt(r,wf='%s',n.levels=1))" % (wavelet,))))
        wavelets.sort(cmp=lambda x,y : cmp(result[x],result[y]))        
                    
        results = dataset.Dataset([result[wav] for wav in wavelets],[(None,wavelets)],name="Residuals")             
            
        return [results, plots.IndexPlot(results)]
            
class CreateWavelet(workflow.Function):
    """<b>Wavelet Transform</b>: 1D Vector transform pr row.
        
Accepts: 
data: any dataset 
"""

    def run(self, data):
        ensure_r_libraries(["waveslim"])
        rpy.r.library("waveslim")
                
        text = get_new_wavelet_name(orgdata=data) 
        
        # store wavelet-basis used and make a plot of it    
        wav = dataset.Dataset(rpy.r("wavelet.filter(\"%s\")" % (text,)),name=text)
        plot = plots. IndexPlot(wav,Bar=False,Line=True,Points=False) 
        #plot.toggle_axis(False) 
        plot.set_title("Wavelet Basis")
        
        # find max levels
        new_shape = 2**int(math.log(data.shape[1],2))
        levels = int(math.log(new_shape,2))
                
        # generate wavelets
        rows = []   # for creating total matrix, vectors merged
        cube = []   # for creating timescaleplot
                                                        
        for x in range(data.shape[0]):
            rpy.r.assign("w",data.asarray()[x,0:new_shape])
            rpy.r("a <- dwt(w,wf=\"%s\",n.levels=%s)" % (text,levels))
            rows.append(rpy.r("c(a$"+",a$".join(rpy.r("names(a)"))+")"))

                       
        transformed = dataset.Dataset(rows,name="Wavelet Data")
        transformed.add_to_description("<b>Wavelet Used</b>: "+ text)
        
        sc = plots.IndexPlot(dataset.Dataset([1,1,1],name="kanin"))            
        tsp = TimeScalePlot(transformed)
        tsp.set_wavelet(text)
        tsp.set_scatterplot(sc)
        tsp.set_title("Timescale Plot")
        sc.set_title("Timescale Plot: Reverse Transformed")                               
        return [wav,transformed,MultipleLinePlot(data,name="Original " + data.get_name()),plot,tsp,sc]



class TimeScalePlotFunction(workflow.Function):
    def run(self,data):    
        text = search_wavelet_description(data)

        if not text:
            text = get_new_wavelet_name()    
    
        sc = plots.IndexPlot(dataset.Dataset([1,1,1],name="kanin"))            
        tsp = TimeScalePlot(data)
        tsp.set_wavelet(text)
        tsp.set_scatterplot(sc)
        tsp.set_title("Timescale Plot")
        sc.set_title("Timescale Plot: Reverse Transformed")
        return [tsp,sc]                                       

class InvTransform(workflow.Function):
    def run(self, data):
        # ensure stuff
        wav = search_wavelet_description(data)

        if not wav:
            wav = get_new_wavelet_name()
            if not wav:
                return
        result = []
                                                                                                                             
        for x in range(data.shape[0]):
            l = 0
            levels = math.log(data.shape[1],2)
            rpy.r.assign("t",data.asarray()[x]) 
            rpy.r("fisk <- dwt(t,wf=\"%s\",n.levels=%s)" % (wav, levels))# dummy object!
            
            for y in range(levels):
                li = data.asarray()[x,l:l+(data.shape[1]+1)/2**(y+1)]             
                l += (data.shape[1]+1)/2**(y+1)
                rpy.r.assign("t",li) 
                rpy.r("fisk$d"+str(y+1)+"<- t")
            rpy.r.assign("t",data.asarray()[x,-1])                
            rpy.r("fisk$s"+str(y+1)+" <- t")
                                                  
            result.append(rpy.r("idwt(fisk)"))
                
        return [dataset.Dataset(result,name="Inv")]

## wavelet helper functions
def get_new_wavelet_name(orgdata=None):
        text = None
        wavelets = ['haar', 'd4', 'mb4', 'bs3.1', 'fk4', 'd6', 'fk6', 'd8', 'fk8', 'la8', 'bl14',
         'fk14', 'd16', 'la16', 'mb16', 'la20', 'bl20', 'fk22', 'mb24'] 

        window = gtk.Dialog("Select Wavelet Basis",
                     None,gtk.DIALOG_MODAL,(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                      gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        
        combobox = gtk.combo_box_new_text()
        window.vbox.pack_start(gtk.Label("Please selected wanted basis:"))           
        window.vbox.pack_start(combobox)
        
        if orgdata:
            import math
            new_shape = 2**int(math.log(orgdata.shape[1],2))
            if new_shape != orgdata.shape[1]:
                window.vbox.pack_start(gtk.Label("WARNING!:\nWavelength will be cropped down to %s)" % (new_shape,))) 
        
        for text in wavelets:
            combobox.append_text(text)
            
        combobox.set_active(8)
        window.show_all()
        try:
            if window.run() != gtk.RESPONSE_ACCEPT:
                return
            text = combobox.get_active_text()                        
        finally:
            window.destroy()
        return text

def search_wavelet_description(data):
        desc = data.get_description()
        wav = None
        for line in desc.split("\n"):
            if "Wavelet Used" in line:
                wav = line.split(":")[1].strip()                 
        return wav                 

class TimeScalePlot(plots.ImagePlot):
    def __init__(self,ds):
        self.wavelet = None
        self._last_slice = 0
        self._offsets = 0        
        self._vectors = ds        
                                              
        plots.ImagePlot.__init__(self,dataset.Dataset(self.get_timescale_matrix(ds.asarray()[0])))
        self.reset_vals() # init offsets
         
        self.interpolation = "nearest"
        self._canvas.mpl_connect('button_press_event',self.on_mouseclick)

        self.ax2 = matplotlib.axes.Axes(self._fig, [0.3,0.05,0.60,0.05])
        self.ax2 = self._fig.add_axes(self.ax2)

        # Name, min, max and valinit=start_position , valfmt='%i' makes widget display only ints
        self.slider = pylab.Slider(self.ax2, "Choose Slice", 0, ds.shape[0], valinit=0, valfmt='%i'  )
        # register callback for slider changed
        self.slider.on_changed(self.update_slice)          
        self.update()
        
              
    def update_axis(self):
        # create more space in the bottom than super        
        map(self._fig.delaxes, self.axes)
        if self._show_colorbar:
            ax = matplotlib.axes.Axes(self._fig, [0.1,0.2,0.75,0.75])
            ax = self._fig.add_axes(ax)
            ax2 = matplotlib.axes.Axes(self._fig, [0.90,0.2,0.04,0.75])
            ax2 = self._fig.add_axes(ax2)
            self.axes = [ax, ax2]            
        else:
            ax = matplotlib.axes.Axes(self._fig, [0.1,0.2,0.80,0.75])
            ax = self._fig.add_axes(ax)
            self.axes = [ax]  
        
    def update_slice(self,val):
        self.reset_vals()
        arr = self.get_timescale_matrix(self._vectors.asarray()[int(val)])
        self._last_slice = int(val)
        self._data = dataset.Dataset(arr)                
        self.update()

    def on_mouseclick(self,event):   
        if event.inaxes == self.axes[0] and event.button == 1:
        
            # popup box asking for changes!
            dia = gtk.Dialog(flags=gtk.DIALOG_MODAL,buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                      gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
            
            # find various needed values
            s = self._data.shape[0]
            sizes = [2**x for x in xrange(s)]
            time = int(event.ydata)
            pos = int(event.xdata) / (2**(s-1-time))
            vec = sum(sizes[time+1:])+pos
            ori_value  = self._vectors.asarray()[self._last_slice,vec]
            cur_offset = self._offsets[s-1-time,pos*2**(s-time-1)] 

            # create GUI
            ovalue_label = gtk.Label("Original Value: " + str(ori_value))                                              
            dia.vbox.pack_start(ovalue_label)
            
            hbox = gtk.HBox()
            offset_label = gtk.Label("Select Offset: ")
            spin = gtk.SpinButton(adjustment=gtk.Adjustment(value=cur_offset, lower=-1000, upper=1000, step_incr=0.1, page_incr=1),climb_rate=0.5)
            
            hbox.pack_start(offset_label)
            hbox.pack_start(spin)            
            dia.vbox.pack_start(hbox)
            dia.show_all()
            
            try:            
                dia.run()
            finally:                
                dia.destroy()
            self._offsets[s-1-time,pos*2**(s-1-time):(pos+1)*2**(s-1-time)] = spin.get_value()
                                                                                
            # update scatterplot
            self.update()
            
        return False
                         
    def set_scatterplot(self, plot):
        self.scplot = plot
        self.update_scatterplot()

    def reset_vals(self,*ignored):
        self._offsets = Numeric.zeros(self._data.shape,self._data.asarray().typecode())

    def get_timescale_matrix(self,vector):

        lengde = int(math.ceil(math.log(len(vector),2)))
        new_matrix = Numeric.zeros(map(int,(lengde,2**(lengde-1))),vector.typecode())
        sizes = [2**x for x in xrange(lengde)]                
        for l in range(lengde):            
            for x in xrange(sizes[-(l+1)]):
                new_matrix[l,x*sizes[l]:(x+1)*sizes[l]] = vector[sum(sizes[lengde-l:])+x]
        return new_matrix                
                                                                       
    def update_scatterplot(self,*ignored):        
        if not self.wavelet:            
            return
                                                
        # compute new array   
        arr = self._offsets + self._data.asarray()     
        l = 2**arr.shape[0]
        rpy.r.assign("t",range(l))
        rpy.r("fisk <- dwt(t,n.levels=%s)" % (arr.shape[0],))
        
        for y in range(arr.shape[0]):
            li = [arr[y,x] for x in range(0,arr.shape[1],2**y)]
            rpy.r.assign("t",li) 
            rpy.r("fisk$d"+str(y+1)+"<- t")
        rpy.r.assign("t",self._vectors.asarray()[self._last_slice ,-1])
        rpy.r("fisk$s" + str(y+1) + " <- as.vector(t)")
                
        rpy.r("attributes(fisk)$wavelet <- \"" + self.wavelet + "\"")
        rpy.r("attributes(fisk)$n.levels <- " + str(arr.shape[0]))                                    
        result = rpy.r("idwt(fisk)")
            
        self.scplot._data = dataset.Dataset(result)
        self.scplot._dim = self.scplot._data[0]
        self.scplot._id  = self.scplot._data[0,0]
        self.scplot.update()         
                                      
    def update(self):
        arr = self._data.asarray()+self._offsets
        vmin, vmax = min(Numeric.ravel(arr)),max(Numeric.ravel(arr))
        self.update_scatterplot()        
        [ax.clear for ax in self.axes]
        
        arr = self._data.asarray().copy()
        mi = min(min(arr))
        if self.selection:
            d1, d2 = self._data.get_dim_names()
                    
            if d1 in self.selection:
                ids = self._data.get_identifiers(d1)
                for id in self.selection[d1]:
                    arr[ids.index(id),:] = mi
            if d2 in self.selection:
                ids = self._data.get_identifiers(d2)
                for id in self.selection[d2]:
                    arr[:,ids.index(id)] = mi                                
                                                                                
        image = self.axes[0].imshow(arr,cmap=self.colormap,interpolation=self.interpolation,vmin=vmin,vmax=vmax)
        
        # force update of axes
        self.axes[0].set_xscale("linear")                
        self.axes[0].set_yscale("linear")
                                      
        if self._show_colorbar:       
            ax = self._fig.colorbar(image,cax=self.axes[1])
            ax.set_yscale("linear")
            ax.set_xscale("linear")

        self._canvas.draw()    

    def set_wavelet(self,name):
        self.wavelet = name
                                    
class MultipleLinePlot(plots.Plot):
    """ Display plots of multiple data rows.
    
    
        Class that displays a line plot of a dataset(matrix) one 
        row at the time, and a slider which can be used to browse
        the rows of the matrix"""
        
    def __init__(self, dataset, name="Unnamed Plot"):
        plots.Plot.__init__(self, name)
        self.fig = pylab.Figure(figsize=(5,4), dpi=72)
        self.canvas = FigureCanvas(self.fig)

        self._dataset = dataset
        d = dataset.asarray()
        n = self._dataset.shape[0]
        m = self._dataset.shape[1]
        
        #set up bounds for the slider and the plot
        self.ax = matplotlib.axes.Axes(self.fig, [0.5,0.05,0.4,0.05])
        self.ax2 = matplotlib.axes.Axes(self.fig, [0.1,0.15,0.85,0.80])
        self.ax2 = self.fig.add_axes(self.ax2)
        self.ax = self.fig.add_axes(self.ax)
        self.ax.set_yticks([])
        self.ax.set_xticks([])
        self.ax.set_xticklabels([])        

        #plot the first row
        self.ax2.plot(d[0])
        
        self.add(self.canvas)
        self.canvas.show()

        self.ylims = min(min(self._dataset.asarray())),max(max(self._dataset.asarray()))

        from matplotlib.backends.backend_gtkagg import NavigationToolbar2GTK
        # We use a regular toolbar as we don't need selections
        self._toolbar = NavigationToolbar2GTK(self.canvas, None)
        self._toolbar.set_property('show-arrow', False)

        # Name, min, max and valinit=start_position, 
        #valfmt='%i' makes widget display only ints
        self.slider = pylab.Slider(self.ax, "Wavelength: ", 0, n, valinit=1, valfmt='%i'  )
        # -1 to force redraw on next update
        self.last_val = -1          
        
        # register callback for slider changed
        self.slider.on_changed(self.update)
        
        self.update(1)
        self.canvas.show()

    def update(self, val):
        val = int(val)
        
        # check if int changed (avoiding costly updates)
        if val != self.last_val:
            self.ax2.clear()
            self.ax2.plot(self._dataset.asarray()[val-1])
            self.ax2.set_ylim(*self.ylims)
            self.last_val = val
            self.canvas.draw()
        
    def get_toolbar(self):
        self.canvas.draw()
        return self._toolbar



class KNN(workflow.Function):
    """ <b>knn</b>: Performs k-nearest neighbor classification of a test set using a training set.

Input:
    k: Number of neighbors considered.
    NumbVar: Number of variables from the test set to be used.

Returns a list-object of class 'kknn' including the components: 
    CL: Matrix of classes of the k nearest neighbors.
    W: Matrix of weights of the k nearest neighbors.
    D: Matrix of distances of the k nearest neighbors.
    prob: Matrix of predicted class probabilities.
    plot: predicted values vs. observed values.  
"""
    def __init__(self):
        workflow.Function.__init__(self, 'knn','KNN')
   
    def run(self,x_train,x_test=None):             
        if not ensure_r_libraries(["kknn"]):
            return        
        rpy.r.library("kknn")
        
        WORKFLOWDIR = os.path.split(__file__)[0]
        xml = gtk.glade.XML(os.path.join(WORKFLOWDIR, "classification.glade"),'kknn_dialog')
        dialog = xml.get_widget('kknn_dialog')
        combo_resp    = xml.get_widget('combobox1') 
        spin_k  = xml.get_widget('spinbutton3')
        spin_out= xml.get_widget('spinbutton4')
        spin_out_label = xml.get_widget('label5')
        
        if x_test:
            # no need to leave anything out
            spin_out.set_sensitive(False)
            spin_out_label.set_sensitive(False)
        else:
            spin_out.set_range(2,x_train.shape[0]-1)        
        
        # init combobox
        liststore = gtk.ListStore(gobject.TYPE_STRING)
        combo_resp.set_model(liststore)
        for id in x_train.get_identifiers(x_train.get_dim_names()[1]):
            liststore.append([id])
        cell = gtk.CellRendererText()
        combo_resp.pack_start(cell, True)
        combo_resp.add_attribute(cell, 'text', 0)
        combo_resp.set_active(0)        
        combo_resp.show()
                            
        try:
            response = dialog.run()
        finally:
            dialog.destroy()
            
        if response != gtk.RESPONSE_OK:
            return
    
        if not x_test:
            # generate testset!            
            num_vars = spin_out.get_value()
                        
            ind_train = range(x_train.shape[0])
            ind_test = [ind_train.pop(random.randint(0,len(ind_train)-1)) for x in xrange(num_vars)]
            train_ids = [[dim,x_train.get_identifiers(dim)] for dim in x_train.get_dim_names()]
            train_ids[0][1] = map(lambda x : "".join(map(str,x)) , scipy.take(train_ids[0][1], ind_train))            
            test_ids = [[dim,x_train.get_identifiers(dim)] for dim in x_train.get_dim_names()]
            test_ids[0][1] = map(lambda x : "".join(map(str,x)) ,scipy.take(test_ids[0][1], ind_test))
            
            x_test  = dataset.Dataset(scipy.take(x_train.asarray(),ind_test),test_ids,name="x_test")               
            x_train = dataset.Dataset(scipy.take(x_train.asarray(),ind_train),train_ids,name="x_train")

                                                                                                  
        rpy.r.assign("x_train",x_train.asarray())        
        rpy.r.assign("x_test",x_test.asarray())
        rpy.r("x_train <- as.data.frame(x_train)")
        rpy.r("x_test <- as.data.frame(x_test)")    
        
        # insert code aswell as GUI to find optimal K here later.
        
        com = combo_resp.get_active()+1
        rpy.r('x_train$V'+str(com)+" <- as.factor(x_train$V"+str(com)+")")
        rpy.r('x_test$V'+str(com)+" <- as.factor(x_test$V"+str(com)+")")
        rpy.r('result <- kknn(V'+str(com)+'~., x_train,x_test)')
                        
        fitted_values = dataset.Dataset(map(lambda x : float(x.replace(",",".")), rpy.r("result$fitted.values")),name="Predicted")
        
        plot = plots.ScatterPlot(x_test,fitted_values,(x_test[1],x_test[1,com-1]),(fitted_values[0],fitted_values[0,0]),name="Predicted vs True")        
        values = ['CL', 'W', 'D', 'prob','distance']
        
        datasets = [x_train,x_test,fitted_values,plot]+[dataset.Dataset(rpy.r("result$"+value),name=value) for value in values]                
        return datasets 

class SOM(workflow.Function):
    """ <b>SOM</b>: Produces an object of class "som" which is a Self-Organizing Map fit of the data.

Input:
    xdim: an integer specifying the x-dimension of the map.
    ydim: an integer specifying the y-dimension of the map.
    init: a character string specifying the initializing method. 
    neigh: a character string specifying the neighborhood function type ( 'bubble' or 'gaussian').
    alpha: a vector of initial learning rate parameter for the two training phases.
    radius: a vector of initial radius of the training area in som-algorithm for the two training phases.

Output:
    map and objects on the map.
    plots: Self Organising map and Score plot with clustering.        
"""
    def __init__(self):
        workflow.Function.__init__(self, 'som','SOM')

    def run(self,data):                        
        if not ensure_r_libraries(["som"]):
            return              
        rpy.r.library("som")
                
        rpy.r.assign("data",data.asarray())
                
        WORKFLOWDIR = os.path.split(__file__)[0]
        xml = gtk.glade.XML(os.path.join(WORKFLOWDIR, "classification.glade"),'som_dialog')
        dia = xml.get_widget('som_dialog')
        combo_init  = xml.get_widget('combobox2') 
        combo_init.set_active(1)
        combo_neigh = xml.get_widget('combobox3')
        combo_neigh.set_active(1)
        combo_alphalearn = xml.get_widget('combobox4')
        combo_alphalearn.set_active(1)
        
        spin_x = xml.get_widget('spinbutton5')
        spin_y = xml.get_widget('spinbutton6')
        spin_r = xml.get_widget('spinbutton7')
        
        try:
            response = dia.run()
        finally:
            dia.destroy()
        
        if response != gtk.RESPONSE_OK:
            return
        
        x = int(spin_x.get_value())
        y = int(spin_y.get_value())
        r = int(spin_r.get_value())
        r = (r,max(3,r)) # this is what the R code in SOM is doing atleast.        
                                                                
        init = combo_init.get_active_text()
        neigh = combo_neigh.get_active_text()
        learn = combo_alphalearn.get_active_text()

        nums = map(dataset.make_number_scientific,[x,y,r[0],r[1]])                                                                                         
        som_map = rpy.r('result <- som(data,xdim=%s,ydim=%s,init="%s",neigh="%s",radius=c%s,alphaType="%s")' % (nums[0],nums[1],init,neigh,(nums[2],nums[3]),learn))                 
        
        code_map = rpy.r("result$code")
        dim = data.get_dim_names()[1]
        code_map = dataset.Dataset(scipy.reshape(code_map,(x,y,-1)),[(None,None),(None,None),(dim,data.get_identifiers(dim))],name="map")
        
        num_placed = rpy.r("result$code.sum")['nobs']
        num_placed = dataset.Dataset(scipy.reshape(num_placed,(x,y)),name="objects_on_map")

        newdata = rpy.r("dim(result$code) = c(" + str(x*y) + "," + str(data.shape[1]) + ")")                
        newdata = rpy.r("prcomp(rbind(data,result$code))$x")
        
        # make new dataset with new dim names        
        samples = data.get_dim_names()[0]        
        samples = (samples, data.get_identifiers(samples)+["("+str(num%x)+" "+str(num/x)+")" for num in range(x*y)])
        pcaids  = ("PCA",["PCA"+str(num+1) for num in range(newdata.shape[1])])        
        newdata = dataset.Dataset(newdata,[samples,pcaids],name="PCA Scores")
                                                
        colour_plot = somColourPlot(newdata, x*y)
        
                        
        return [code_map,num_placed, SOMPlot(num_placed,code_map, samples), colour_plot]
        

class somColourPlot(ScatterPlotBrowse):

    def __init__(self, data, num_clusters):
        self._num_clusters = int(num_clusters)
        dim = data.get_dim_names()[1]
        id1, id2 = data.get_identifiers(dim)[0:2]
        self._use_legend = False
        ScatterPlotBrowse.__init__(self,data,data,(dim,id1),(dim,id2),
            name="Score Plot with Clustering",use_objectnames = False)

    def draw_plot(self, data1, data2):
        x = self._num_clusters        
        self._ax.plot(data1[-x:],data2[-x:],"ob",label="Som Positions" )        
        self._ax.plot(data1[:-x],data2[:-x],"oy",label="Samples",markersize=9)        
            
        if self._use_legend:            
            self._ax.legend()
            
    def selection_changed(self,*stuff):
        print stuff
        ScatterPlotBrowse.selection_changed(self,*stuff)                            

    def _init_toolbar(self):
        ScatterPlotBrowse._init_toolbar(self)
        tbutton_legend = self._toolbar.add_legend_button() 
        tbutton_legend.connect('clicked', self.switch_legend)

    def switch_legend(self,*ignored):
        self._use_legend = not self._use_legend
        self.update()    
                       

class SOMPlot(plots.Plot):
    def __init__(self,data,data2,(select_dim, ids)):
        self._dataset = data
        self._dataset2 = data2
        self._select_dim = select_dim
        self._id = None
        
        self._mouse_down = False            
        plots.Plot.__init__(self,"Self Organising Map")
                
        self._fig = pylab.Figure(figsize=(5,4), dpi=72)
        self._canvas = FigureCanvas(self._fig)
        self._ax = self._fig.add_subplot(121)
        self._ax2 = self._fig.add_subplot(122)        
        self.add(self._canvas)
                           
        self._ax2.plot(self._dataset2.asarray()[0,0])       
        self._ylimit = (min(min(min(self._dataset2.asarray()))),max(max(max(self._dataset2.asarray()))))  
        self._ax2.set_ylim(*self._ylimit)
        self.update()

        self._idPress = self._canvas.mpl_connect('button_press_event',
                                                    self.press_select)
        self._idRelease = self._canvas.mpl_connect('button_release_event',
                                                      self.release_select)        

        self._init_toolbar()  
        self._init_popupmenu()                                                    
        self._canvas.show()

    def get_popupmenu(self):
        return self._popupmenu            
   
    def _init_popupmenu(self):
        self._popupmenu = plots.CustomPopupMenu(self)
        self._popupmenu.init_from_toolbar(self.get_toolbar())                            
        
    def _init_toolbar(self):
        # create toolbar
        self._toolbar = plots.ScatterPlotToolbar(self._canvas, None) 
        self._toolbar.set_property('show-arrow', False)
        self._toolbar.set_select_callback(self.rectangle_select_callback)        
                
        # connect various
        self._toolbar.set_select_callback(self.rectangle_select_callback)
    
    def get_toolbar(self):
        return self._toolbar

    def rectangle_select_callback(*stuff):
        pass       

    def press_select(self,event):        
        if event.button == 1:        
            if event.inaxes == self._ax:
                self._mouse_down = True
                
        self.mouse_moved(None,event)                
        self._id = self._canvas.connect("motion_notify_event",self.mouse_moved)                            
    
    def mouse_moved(self,place,event):
        if self._mouse_down:
                         
            y,x = map(int,self._ax.transData.inverse_xy_tup((event.x, event.y)))
            if (x<0 or y<0 or x>=self._dataset2.shape[0] or y>=self._dataset2.shape[1]):
                self.release_select(event)
                return
            
            print y,x
            
            # if not called from press_select invert found y!
            if place:
                y = self._dataset2.shape[1]-y-1            
            print y,x
            print "--"
            
            self._ax2.clear()            
            self._ax2.plot(self._dataset2.asarray()[x,y])
            self._ax2.set_ylim(*self._ylimit)
            self._canvas.draw()
            
            self.selection_listener(self._select_dim,["(%s %s)"%(x,y)])                                             
                   
    def release_select(self,event):
        if self._id:
            self._canvas.disconnect(self._id)
            self._id = None
        self._mouse_down = False      

    def update(self):
        im = self._ax.imshow(self._dataset.asarray(),interpolation="hamming")
        self._fig.colorbar(im,orientation="horizontal")                           
